
//=============================================================================
//	ACCESS LEVEL SYSTEM
//=============================================================================

//	This function adds a new admin mask to the admin system
//	%guest is true if the default guest access can use this mask
function AdminMenu::addAdminMask(%this, %name, %guest)
{
	if(%this.AdminMaskMax $= "")
		%this.AdminMaskMax = 0;

	%this.AdminMask[%name] = %this.AdminMaskMax;
	%this.AdminMaskID[%this.AdminMaskMax] = %name;
	%this.AdminMaskGuest[%name] = %guest;
	if(%this.AdminMaskMax == 0)
	{
		%this.addAdminMenuItem("ListAccessMasks", 1, 1, "Return to Access Options Menu", "true", "AdminMenu.ListMenuData(%client, AccessOptions, 1, \"\");");
		%this.addAdminMenuItem("ListAccessMasks", 2, 1, "Change Login Password", "%client.menuTargetClient $= \"\"", "AdminMenu.setClassPassword(%client, %client.selectActionMenu);");
	}
	%this.addAdminMenuItem("ListAccessMasks", %this.AdminMaskMax+3, 1, "Enable "@%name, "!AdminMenu.TestCurrentMask(%client, "@%name@") && %client.testAdmin("@%name@")", "AdminMenu.ToggleCurrentMask(%client, "@%name@", 1);");
	%this.addAdminMenuItem("ListAccessMasks", %this.AdminMaskMax+3, 2, "Disable "@%name, "AdminMenu.TestCurrentMask(%client, "@%name@") && %client.testAdmin("@%name@")", "AdminMenu.ToggleCurrentMask(%client, "@%name@", 0);");
	%this.AdminMaskMax++;
}


//	This function outputs the GOD level admin.
function calcGodAdminLevel()
{
	%levelMask = "";
	for(%i=0; %i <= Adminmenu.AdminMaskMax; %i++)
		%levelMask = %levelMask @ "1";
	return %levelMask;
}


//	This function outputs the Default Guest Access
function calcGuestAdminLevel()
{
	%level = "";
	for(%mask=0; %mask<Adminmenu.AdminMaskMax; %mask++)
	{
		%name = Adminmenu.AdminMaskID[%mask];
		if(AdminMenu.AdminMaskGuest[%name])
			%level = %level @ "1";
		else
			%level = %level @ "0";
	}
	return %level;
}


//Give Default Player access
function calcDefaultAdminLevel()
{
	if($AdminList::DefaultLevel $= "")
	{
		$AdminList::DefaultLevel = "Guest";
		$AdminList::AccessLevelName[0] = $AdminList::DefaultLevel;
		$AdminList::AccessLevelMax++;
		$AdminList::AccessLevelMask[$AdminList::DefaultLevel] = calcGuestAdminLevel();
	}
	%level = $AdminList::AccessLevelMask[$AdminList::DefaultLevel];
	return %level;
}

//	Determine if client has access for %testmask admin type
//	%testmask = admin id name   eg. "ResetServer"
function GameConnection::testAdmin( %client, %testmask )
{
	%searchMask = Adminmenu.AdminMask[%testmask];
	if(	%searchMask $= "")
		return false;
	%clientMask = %client.adminMask;
	%textData = getSubStr(%clientMask, %searchMask, 1);
	%textData = %textData $= "" ? "0" : %textData;
	eval("%value = "@%textData@";");
	return %value;
}


//	Give admin level to client
//	%testmask = admin id name   eg. "ResetServer"
function GameConnection::grantAdmin( %client, %testmask )
{
	%searchMask = Adminmenu.AdminMask[%testmask];
	if(	%searchMask $= "")
		return false;
	if(%client.testAdmin(%testmask))
		return false;
	if(%searchMask > 1)
		%prefix = getSubStr(%client.adminMask, 0, %searchMask-1);
	%suffix = getSubStr(%client.adminMask, %searchMask, Adminmenu.AdminMaskMax-%searchMask);
	%client.adminMask = %prefix @ "1" @ %suffix;
	return true;
}


//	Give admin class to client
//	%classname = admin class name   eg. "Guest"
function GameConnection::grantAdminClass( %client, %classname, %admin )
{
	%client.adminMask = $AdminList::AccessLevelMask[%classname];
	if(%admin.selectActionMenu)
	{
		%guid = %client.guid;
		$AdminList::PlayerAccessLevel[%guid] = %classname;
	}
	%type = %admin.selectActionMenu ? "permanent" : "temporary";
	messageClient(%admin,'MsgChangeAdmin','\c2You have granted %1 admin class %2 to %3.',%type, %classname ,%client.name);
	messageClient(%client,'MsgChangeAdmin','\c2You have been granted %1 admin class %2 by %3.',%type, %classname ,%admin.name);
	echo("(",%admin,") ",%admin.namebase," granted ",%type," ",%classname," access to ",%client.namebase," (",%client,")");
	CommandToClient(%admin, 'CloseAdminMenu');
}


//	Remove admin level from client
//	%testmask = admin id name   eg. "ResetServer"
function GameConnection::removeAdmin( %client, %testmask )
{
	%searchMask = Adminmenu.AdminMask[%testmask];
	if(	%searchMask $= "")
		return false;
	if(!%client.testAdmin(%testmask))
		return false;
	%prefix = getSubStr(%client.adminMask, 0, %searchMask-1);
	%suffix = getSubStr(%client.adminMask, %searchMask, Adminmenu.AdminMaskMax-%searchMask+1);
	%client.adminMask = %prefix @ "0" @ %suffix;
	return true;
}


//	Set admin masks for client
function GameConnection::SetAdminMasks( %client )
{
	//If client is local host then give him godadmin
	if(%client.getAddress() $= "local")
	{
		%client.adminMask = calcGodAdminLevel();
		return;
	}

	//Test for guid in userlist
	//Note: Player mask has preference over player level
	if((%client.adminMask == 0 || %client.adminMask $= "") && $AdminList::PlayerAccess[%guid])
	{
		%guid = %client.guid;
		%client.adminMask = $AdminList::PlayerAccess[%guid];
		return;
	}
	if((%client.adminMask == 0 || %client.adminMask $= "") && $AdminList::PlayerAccessLevel[%guid])
	{
		%guid = %client.guid;
		%access = $AdminList::PlayerAccessLevel[%guid];
		if(%access !$= "")
			%client.adminMask = $AdminList::AccessLevelMask[%access];
		return;
	}

	//If none then set default access
	if(%client.adminMask == 0 || %client.adminMask $= "")
	{
		%client.adminMask = calcDefaultAdminLevel();
		return;
	}
}

//	Load admin masks for default admin and plugins
function AdminMenu::CreateAdminMask(%this)
{
	%this.AdminMaskMax = 0;
	%this.addAdminMask(ResetServer, 0);		// The ability to reset the server to the default settings;
	%this.addAdminMask(ServerAccess, 0);	// The ability to alter the definition of admin levels and abilitys;
	%this.addAdminMask(CancelVote, 0);		// The ability to cancel initiated votes;
	%this.addAdminMask(SaveSettings, 0);	// The ability to save the current server settings as the default;
	%this.addAdminMask(GrantAdmin, 0);		// The ability to grant admin abilitys to others (Up to your own abilitys);
	%this.addAdminMask(Immunity, 0);		// Makes the player to immune from votes against him


}
AdminMenu.CreateAdminMask();


//	This function saves the admin userlist and set admin level configurations
//	Data backed up includes:
//	$AdminList::DefaultLevel
//	$AdminList::AccessLevelMask[Bla]
//	$AdminList::AccessLevelPassword[Bla]
//	$AdminList::AccessLevelMax
//	$AdminList::AccessLevelName[1]
//	$AdminList::PlayerAccessMask[guid]
//	$AdminList::PlayerAccessLevel[guid]
function AdminMenu::SaveAdminAccessData(%this, %client)
{
	if(%client)
	{
		messageClient(%client,'MsgSaveAdmin','\c2You have saved the current admin access list.');
		echo("(",%client,") ",%client.namebase," saved the current admin access list");
	}

	%test = calcDefaultAdminLevel();

	export("$AdminList::*", "./admin/userlist_admin.cs", False);
	CommandToClient(%client, 'CloseAdminMenu');
}

function AdminMenu::ResetGuestClass(%this, %client)
{
	%test = calcDefaultAdminLevel();
	%mask = calcGuestAdminLevel();
	$AdminList::AccessLevelMask[Guest] = %mask;

	if(%client)
	{
		messageClient(%client,'MsgSaveAdmin','\c2You have reset the Guest Admin Class to its default settings.');
		echo("(",%client,") ",%client.namebase," reset the Guest admin class to the default settings");
	}
	CommandToClient(%client, 'CloseAdminMenu');
}

package AdminMenu
{
function GameConnection::onClientEnterGame(%this)
{
   //Set clients admin levels on connect
   %this.SetAdminMasks();

   parent::onClientEnterGame(%this);
}

function onExit()
{
	AdminMenu.SaveAdminAccessData();
	parent::onExit();
}


};
activatepackage(AdminMenu);


//=============================================================================
//	ADMIN CLASS FUNCTIONS
//=============================================================================

function AdminMenu::TestCurrentMask(%this, %client, %testmask)
{
	%object = %client.selectActionMenu;
	if(%object.name $= "")
	{
		%value = %this.TestClassMask(%object, %testmask);
	}
	else
	{
		%value = %object.testAdmin( %testmask );
	}
	return %value;
}

function AdminMenu::ToggleCurrentMask(%this, %client, %testmask, %addMask)
{
	%object = %client.selectActionMenu;

	if(%object.name $= "")
	{
		if(%addMask)
			%this.grantClassAdmin(%object, %testmask);
		else
			%this.removeClassAdmin(%object, %testmask);
	}
	else
	{
		if(%addMask)
			%object.grantAdmin( %testmask );
		else
			%object.removeAdmin( %testmask );
	}
	%client.selectActionMenu = "";
	%this.ListMenuData(%client, ListAccessMasks, 1, %object);
}


//	Give admin level to admin class
//	%testmask = admin id name   eg. "ResetServer"
function AdminMenu::grantClassAdmin( %this, %classname, %testmask )
{
	%searchMask = %this.AdminMask[%testmask]+1;
	if(	%searchMask $= "")
		return false;
	if(%this.TestClassMask(%classname, %testmask))
		return false;

	%classMask = $AdminList::AccessLevelMask[%classname];
	if(%searchMask > 1)
		%prefix = getSubStr(%classMask, 0, %searchMask-1);
	%suffix = getSubStr(%classMask, %searchMask, Adminmenu.AdminMaskMax-%searchMask+1);
	$AdminList::AccessLevelMask[%classname] = %prefix @ "1" @ %suffix;
	%this.updateClassAdminMembers(%classname);
	return true;
}


//	Remove admin level from admin class
//	%testmask = admin id name   eg. "ResetServer"
function AdminMenu::removeClassAdmin( %this, %classname, %testmask )
{
	%searchMask = %this.AdminMask[%testmask]+1;
	if(	%searchMask $= "")
		return false;
	if(!%this.TestClassMask(%classname, %testmask))
		return false;
	%classMask = $AdminList::AccessLevelMask[%classname];
	if(%searchMask > 1)
		%prefix = getSubStr(%classMask, 0, %searchMask-1);
	%suffix = getSubStr(%classMask, %searchMask, Adminmenu.AdminMaskMax-%searchMask+1);
	$AdminList::AccessLevelMask[%classname] = %prefix @ "0" @ %suffix;
	%this.updateClassAdminMembers(%classname);
	return true;
}

//	Update all clients that have this admin class
function AdminMenu::updateClassAdminMembers( %this, %classname )
{
	%count = ClientGroup.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%client = ClientGroup.getObject(%i);
		%guid = %client.guid;
		if($AdminList::PlayerAccessLevel[%guid] $= %classname)
		{
			%client.SetAdminMasks();
		}
	}
}

function AdminMenu::TestClassMask(%this, %classname, %testmask)
{
	%searchMask = %this.AdminMask[%testmask];
	if(	%searchMask $= "")
		return false;
	%classMask = $AdminList::AccessLevelMask[%classname];
	%textData = getSubStr(%classMask, %searchMask, 1);
	%textData = %textData $= "" ? "0" : %textData;
	eval("%value  = "@%textData@";");
	return %value;
}

//	Create a New Admin Class
function AdminMenu::makeNewAdminClass(%this, %client)
{
	CommandToClient(%client, 'RequestAdminQuery', "Create New Admin Class", "Enter the name of the new class you wish to create.", "QueryBoxResponse(MBQueryTextData.getValue(), MakeAdminClass);", "");
}

function ServerCmdMakeAdminClass(%client, %name)
{
	//Check that new class dosnt already exist
	for(%i=0; %i<$AdminList::AccessLevelMax; %i++)
	{
		if($AdminList::AccessLevelName[%i] $= %name)
		{
			messageClient(%client,'MsgAdminError','\c2The class name of %1 already exists.', %name);
			return;
		}
	}
	%level = $AdminList::AccessLevelMax;
	$AdminList::AccessLevelMax++;
	$AdminList::AccessLevelName[%level] = %name;
	$AdminList::AccessLevelMask[%name] = calcDefaultAdminLevel();
	messageClient(%client,'MsgAdminClassCreate','\c2You created a new admin class called %1 with default access levels.', %name);
	AdminMenu.addAdminMenuItem("EditAccessClass", $AdminList::AccessLevelMax+1, 1, %name, "true", "AdminMenu.ListMenuData(%client, ListAccessMasks, 1, "@%name@");");
	AdminMenu.addAdminMenuItem("GrantAdmin", $AdminList::AccessLevelMax+2, 1, "Grant "@%name, "true", "%client.menuTargetClient.grantAdminClass( "@%name@", %client );");
	AdminMenu.ListMenuData(%client, AccessOptions);
}




//=============================================================================
//	PASSWORD LOGIN SYSTEM
//=============================================================================

function AdminMenu::setClassPassword(%this, %client, %name)
{
	CommandToClient(%client, 'RequestAdminQuery', "Change Admin Password", "Enter the new admin class password for the "@%name@" class.", "QueryBoxResponse(MBQueryTextData.getValue(), ChangePasswordAccess, "@%name@");", "");
}

function ServerCmdChangePasswordAccess(%client, %password, %name)
{
	if(%name $= "God")
		$Pref::Server::AdminPassword = %password;
	else
		$AdminList::AccessLevelPassword[%name] = %password;
	if(%password $= "")
	{
		messageClient(%client,'MsgAdminPassword','\c2Admin password for %1 class disabled.', %name);
		echo("(",%client,") ",%client.namebase," disabled the ",%name," password");
	}
	else
	{
		messageClient(%client,'MsgAdminPassword','\c2Admin password for %1 class changed to %2.', %name, %password);
		echo("(",%client,") ",%client.namebase," changed the ",%name," password to ",%password);
	}
}

function AdminMenu::loginAdminClass(%this, %client)
{
	CommandToClient(%client, 'CloseAdminMenu');
	CommandToClient(%client, 'RequestAdminQuery', "Enter Admin Password", "Enter your admin class password.", "QueryBoxResponse(MBQueryTextData.getValue(), LoginPasswordAccess);", "");
}

function ServerCmdLoginPasswordAccess(%client, %password)
{
	if(%password $= $Pref::Server::AdminPassword)
	{
		messageClient(%client,'MsgAdminLogin','\c2You have logged in as a god class admin.');
		%client.adminMask = calcGodAdminLevel();
		ServerCmdAdminMenuRequest(%client);
		return;
	}
	for(%i=0; %i<$AdminList::AccessLevelMax; %i++)
	{
		%name = $AdminList::AccessLevelName[%i];
		if($AdminList::AccessLevelPassword[%name] $= %password && $AdminList::AccessLevelPassword[%name] !$= "")
		{
			%client.adminMask = $AdminList::AccessLevelMask[%name];
			messageClient(%client,'MsgAdminLogin','\c2You have logged in as a %1 class admin.', %name);
			ServerCmdAdminMenuRequest(%client);
			return;
		}
	}
	messageClient(%client,'MsgAdminError','\c2Incorrect Login Password.');
}






