//======================================================================
//	INTEGRATION WITH CLIENT ADMIN MENU
//======================================================================
//
//	Client was using menu %menu and has pressed request id %request
//	Request id will be menu list request (a number from -2 to 26)
//	-1 will always be a previous option and -2 a next option
function ServerCmdAdminMenuRequest(%client, %request)
{
	if(%request $= "")
	{
		messageClient(%client,'MsgAdminDeselect',"");
		AdminMenu.ListMenuData(%client, "Main", 1);
		%client.selectActionMenu = "";
	}
	else if(%request == -1)
		AdminMenu.ListMenuData(%client, %client.lastMenu, %client.lastStage-1);
	else if(%request == -2)
		AdminMenu.ListMenuData(%client, %client.lastMenu, %client.lastStage+1);
	else
	{
		%command = %client.menuCommand[%request];
		eval(%command);
	}
}

//	Client clicked on a player in the player list
//	%client = viewing client id
//	%target = client id of player that was clicked on
function ServerCmdPlayerListClientSelected(%client, %target)
{
	%client.menuTargetClient = %target;
	AdminMenu.ListMenuData(%client, PlayerSelected);
}

//	Clear this clients menu
function GameConnection::clearMenuData( %client )
{
	%client.menuItem = 0;
	for(%i=1; %i<27; %i++)
	{
		%client.menuText[%i] = "";
		%client.menuCommand[%i] = "";
	}
}

//	Save the menu for this client to the client object
function GameConnection::setMenuData( %client, %text, %command )
{
	%client.menuItem++;
	%client.menuText[%client.menuItem] = %text;
	%client.menuCommand[%client.menuItem] = %command;
}




//=============================================================================
//	ADMIN MENU FUNCTIONS
//=============================================================================

//	Create the admin menu script object if it dosnt already exist
if(!isObject(AdminMenu))
{
	new ScriptObject(AdminMenu)
	{
		class = AdminMenu;
	};
}


function ServerCmdUpdatePlayerListData(%client)
{
	AdminMenu.updatePlayerList(%client);
}

function AdminMenu::updatePlayerList(%this, %client)
{
	%count = ClientGroup.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%cl = ClientGroup.getObject(%i);
		%ping = %cl.getPing();
		%pl = %cl.getPacketLoss();
		%team = %cl.team $= "" ? 0 : %cl.team;
		messageClient(%client,'MsgAdminUpdate',"",%cl, %cl.name, %cl.guid, %ping, %pl, %team);
	}
	messageClient(%client,'MsgAdminUpdateEnd',"");
}



//	Adds a menu item to the admin menu object
//	%menu    = menu name such as "Main" or "ServerSettings"
//	%item    = the menu item number
//	%rank    = the ranking of this item in the mask check order
//	%text    = the text displayed in the menu
//	%mask    = the mask used to determine to display or not.  If true then menu is displayed
//	%command = the command executed by this menu item
function AdminMenu::addAdminMenuItem(%this, %menu, %item, %rank, %text, %mask, %command, %plugin)
{
	if(%this.menuText[%menu, %item, %rank] !$= "")
	{
		error("");
		error("-----------------------------------------------------------------------------");
		error("Error Loading Plugin ",%plugin," Menu ",%text);
		error("Menu ",%menu,"   Item ",%item,"   Rank ",%rank,"  is previously defined as ",%this.menuText[%menu, %item, %rank]);
		error("");
		return;
	}

	%this.menuText[%menu, %item, %rank] = %text;
	%this.menuMaskTest[%menu, %item, %rank] = %mask;
	%this.menuCommand[%menu, %item, %rank] = %command;
	%this.menuPlugin[%menu, %item, %rank] = %plugin;

	if(%plugin !$= "")
	{
		%name = %plugin@"_Object";
		%id = %name.menuItem;
		%name.itemMenu[%id] = %menu;
		%name.itemItem[%id] = %item;
		%name.itemRank[%id] = %rank;
	}
}


function AdminMenu::removeAdminMenuItem(%this, %menu, %item, %rank)
{
	%this.menuText[%menu, %item, %rank] = "";
	%this.menuMaskTest[%menu, %item, %rank] = "";
	%this.menuCommand[%menu, %item, %rank] = "";
	%this.menuPlugin[%menu, %item, %rank] = "";
}



//	This function determines what current menu is display for this client.
//	%client = the viewing client.
//	%menu   = which menu the client is going to look at  eg. "Main", "ServerSettings", etc
//	%stage  = The stage of the menu.  Generally stage is left blank unless you wish to view
//	a large menu that has more than 26 options causing it to be on multiple pages.
//	(Previous and Next options are created automatically)
function AdminMenu::ListMenuData(%this, %client, %menu, %stage, %actionMenu)
{

	if(%actionMenu !$= "")
		%client.selectActionMenu = %actionMenu;

	if(%stage $= "" || %stage < 1)
		%stage = 1;

	%client.clearMenuData();

	//create complete menu list effected by clients access abilitys
	for(%item=1; %item<200; %item++)
	{
		%limit = %item+20;
		%finish = true;
		for(%i=%item; %i<%limit; %i++)
		{
			if(%this.menuText[%menu, %i, 1] !$= "" || %this.menuMaskTest[%menu, %i, 1] !$= "")
			{
				%finish = false;
				break;
			}
		}
		if(%finish)
		{
			%itemMax = %item;
			break;
		}

		for(%rank=1; %rank<200; %rank++)
		{
			if(%this.menuMaskTest[%menu, %item, %rank] $= "")
				break;

			eval("%result = "@%this.menuMaskTest[%menu, %item, %rank]@";");

			if(%result)
			{
				%text = %this.menuText[%menu, %item, %rank];
				%command = %this.menuCommand[%menu, %item, %rank];
				%client.setMenuData(%text, %command);
				break;
			}
		}
	}
	%client.lastMenu = %menu;
	%client.lastStage = %stage;

	//determine start and finish item list numbers to display
	%start = (%stage-1)*24+2;
	%start = %stage==1 ? 1 : %start-2;
	%limit = %stage==1 ? 25 : 24;
	%finish = %start + %limit;

	//create and send menu listing to client
	for(%i=1; %i < 27; %i++)
	{
		%item = %i + %start - 1;
		if(%stage > 1 && %i==1)
			messageClient(%client,'MsgAdminMenuData',"",%i, "Previous", -1);
		else if(%itemMax-1 > %finish && %i==26)
			messageClient(%client,'MsgAdminMenuData',"",%i, "Next", -2);
		else if(%client.menuText[%item] !$= "")
		{
			%text = %client.menuText[%item];
		 	messageClient(%client,'MsgAdminMenuData',"",%i, %text, %item);
		}
	}
	messageClient(%client,'MsgAdminMenuDataEnd',"");
}


