//======================================================================
//	CHANGE MISSION ADMIN OPTIONS
//======================================================================
//
//	This plugin adds admin options to change the missions
//
//	Note:	If a missions, missioninfo object has a property of type,
//			that mission will be listed under its own mission type subheading
//			with other missions of the same type (eg. missioninfo.type = "CTF")
//			Would list under the heading "Capture the Flag"
//
//			The name "Capture the Flag" is taken from the array
//			$MissionTypeName[%missionType]
//			ie. $MissionTypeName[CTF] = "Capture the Flag";
//			which should be defined in your game code.
//
//			If no missioninfo.type is detected, the mission will be listed
//			in the basic menu.
//
//
//----------------------------------------------------------------------
//	Register plugin with admin system

AdminMenu.RegisterPlugin("ChangeMission_Admin");




//----------------------------------------------------------------------
//	Functions Needed for Plugin


//	This function returns the mission name
function getMissionName( %missionFile )
{
   %file = new FileObject();

   %MissionInfoObject = "";

   if ( %file.openForRead( %missionFile ) ) {
		%inInfoBlock = false;

		while ( !%file.isEOF() ) {
			%line = %file.readLine();
			%line = trim( %line );

			if( %line $= "new ScriptObject(MissionInfo) {" )
				%inInfoBlock = true;
			else if( %inInfoBlock && %line $= "};" ) {
				%inInfoBlock = false;
				%MissionInfoObject = %MissionInfoObject @ %line;
				break;
			}

			if( %inInfoBlock )
			   %MissionInfoObject = %MissionInfoObject @ %line @ " ";
		}

		%file.close();
	}
	%MissionInfoObject = "%MissionInfoObject = " @ %MissionInfoObject;
	eval( %MissionInfoObject );

   %file.delete();

   if( %MissionInfoObject.name !$= "" )
      return %MissionInfoObject.name;
   else
      return fileBase(%missionFile);
}

//	This function returns the mission type
function getMissionType( %missionFile )
{
   %file = new FileObject();

   %MissionInfoObject = "";

   if ( %file.openForRead( %missionFile ) ) {
		%inInfoBlock = false;

		while ( !%file.isEOF() ) {
			%line = %file.readLine();
			%line = trim( %line );

			if( %line $= "new ScriptObject(MissionInfo) {" )
				%inInfoBlock = true;
			else if( %inInfoBlock && %line $= "};" ) {
				%inInfoBlock = false;
				%MissionInfoObject = %MissionInfoObject @ %line;
				break;
			}

			if( %inInfoBlock )
			   %MissionInfoObject = %MissionInfoObject @ %line @ " ";
		}

		%file.close();
	}
	%MissionInfoObject = "%MissionInfoObject = " @ %MissionInfoObject;
	eval( %MissionInfoObject );

   %file.delete();

   if( %MissionInfoObject.type !$= "" )
      return %MissionInfoObject.type;
   else
      return "Unknown";
}

function ChangeMission_Admin::ChangeMission(%this, %client, %file)
{
	%admin = %this.adminmenu;
	if($Server::Tournament)
	{
		if(%client.testAdmin(TournamentForceMissionChange))
		{
			CommandToClient(%client, 'CloseAdminMenu');
			messageAll('MsgMissionChange', 'Mission changed by %1.', %client.name);
			loadMission(%file);
			return;
		}
	}
	else
	{
		if(%client.testAdmin(FFAForceMissionChange))
		{
			CommandToClient(%client, 'CloseAdminMenu');
			messageAll('MsgMissionChange', 'Mission changed by %1.', %client.name);
			loadMission(%file);
			return;
		}
	}
	CommandToClient(%client, 'CloseAdminMenu');
	%admin.InitVote(%client, "ChangeMission", %file);
	return;
}


//----------------------------------------------------------------------
//	Declare plugin menus and create plugin object

%admin = "AdminMenu";
%admin.addAdminMenuItem("Main", 10, 2, "Change Mission", "$Server::Tournament && %client.testAdmin(TournamentForceMissionChange)", "AdminMenu.ListMenuData(%client, MissionType);", "ChangeMission_Admin");
%admin.addAdminMenuItem("Main", 10, 3, "Change Mission", "!$Server::Tournament && %client.testAdmin(FFAForceMissionChange)", "AdminMenu.ListMenuData(%client, MissionType);", "ChangeMission_Admin");
%admin.addAdminMenuItem("Main", 10, 4, "Vote to Change Mission", "$Server::Tournament && %client.testAdmin(TournamentVoteMissionChange)", "AdminMenu.ListMenuData(%client, MissionType);", "ChangeMission_Admin");
%admin.addAdminMenuItem("Main", 10, 5, "Vote to Change Mission", "!$Server::Tournament && %client.testAdmin(FFAVoteMissionChange)", "AdminMenu.ListMenuData(%client, MissionType);", "ChangeMission_Admin");

%filespec = "*.mis";
for(%file = findFirstFile(%filespec); %file !$= ""; %file = findNextFile(%filespec))
{
	if (strStr(%file, "/CVS/") == -1)
	{
		%missionType = getMissionType( %file );
		%type = $MissionTypeName[%missionType];
		%type = %type $= "" ? %missionType : %type;
		%name = getMissionName(%file);

		if(%missionType !$= "Unknown")
		{
			if($MissionListMax[%missionType] $= "")
			{
				$MissionListTypeMax++;
				$MissionListType[$MissionListTypeMax] = %missionType;
				%admin.addAdminMenuItem("MissionType", $MissionListTypeMax, 1, %type, "true", "AdminMenu.ListMenuData(%client, "@%missionType@");", "ChangeMission_Admin");
			}
			$MissionListMax[%missionType]++;
			$MissionList[%missionType, $MissionListMax[%missionType]] = %file;
			%admin.addAdminMenuItem(%missionType, $MissionListMax[%missionType], 1, %name, "true", "ChangeMission_Admin_Object.ChangeMission(%client, \""@%file@"\");", "ChangeMission_Admin");
		}
		else
		{
			%unknownType++;
			%admin.addAdminMenuItem("MissionType", $MissionListMax[%missionType]+%unknownType, 1, %name, "true", "ChangeMission_Admin_Object.ChangeMission(%client, \""@%file@"\");", "ChangeMission_Admin");
		}
	}
}

AdminMenu.addAdminMask(TournamentForceMissionChange, 0); 	// "The ability to change missions whilst in Tournament Mode";
AdminMenu.addAdminMask(TournamentVoteMissionChange, 1); 	// "The ability to vote to change missions whilst in Tournament Mode";
AdminMenu.addAdminMask(FFAForceMissionChange, 0); 			// "The ability to change missions whilst in Free for All Mode";
AdminMenu.addAdminMask(FFAVoteMissionChange, 1); 			// "The ability to vote to change missions whilst in Free for All Mode";

%admin.addAdminMask(TournamentForceMissionChange, 0); 	// "The ability to change missions whilst in Tournament Mode";
%admin.addAdminMask(TournamentVoteMissionChange, 1); 	// "The ability to vote to change missions whilst in Tournament Mode";
%admin.addAdminMask(FFAForceMissionChange, 0); 			// "The ability to change missions whilst in Free for All Mode";
%admin.addAdminMask(FFAVoteMissionChange, 1); 			// "The ability to vote to change missions whilst in Free for All Mode";

%admin.addVoteType("ChangeMission", "Change the Mission", "loadMission(%target);");



