
//=============================================================================
//	ADMIN MENU DECLARATION
//=============================================================================
//
//	To add menu items use the following format.
//
//	%this.addAdminMenuItem(menuname, linenumber, optionnumber, Text Display, query, do this);
//
//	eg. if you have three items that have "Main" as the menuname and the line numbers are all 1
//		and the option numbers are 1-3 respectivaly, the code will look at the "query" and if it is
//		true it will display that item, if not it will move on to the next optionnumber.
//		The last option number should always be true.
//
//		The query and dothis strings should be be functiosn but in strings.
//		eg query.   "true"  "$client.team == 1"  "$Server::Tournament && Game.getNumTeams() > 1"
//		eg dothis   "AdminMenu.ListMenuData(%client, JoinTeam);"
//
//		Note: If a query does not have a true result for that linenumber it will just not show up
//

function AdminMenu::createAdminMenu(%this)
{
	//Load the menu items that are inbuilt into the admin system and require fixed menu positions
	%this.addAdminMenuItem("Main", 2, 1, "Vote Yes", "AdminMenu.voteInProgress && !%client.hasVoted", "ServerCmdIntegrateClientVote(%client, true, 1);");
	%this.addAdminMenuItem("Main", 2, 2, "", "AdminMenu.voteInProgress", "");
	%this.addAdminMenuItem("Main", 3, 1, "Vote No", "AdminMenu.voteInProgress && !%client.hasVoted", "ServerCmdIntegrateClientVote(%client, false, 1);");
	%this.addAdminMenuItem("Main", 3, 2, "", "AdminMenu.voteInProgress", "");
	%this.addAdminMenuItem("Main", 4, 1, "Cancel Vote", "AdminMenu.voteInProgress && %client.testAdmin(CancelVote)", "AdminMenu.CancelVote(%client);");
	%this.addAdminMenuItem("Main", 4, 2, "", "AdminMenu.voteInProgress", "");

	for(%item=5; %item<27; %item++)
	{
		%this.addAdminMenuItem("Main", %item, 1, "", "AdminMenu.voteInProgress", "");
		%this.addAdminMenuItem("PlayerSelected", %item, 1, "", "AdminMenu.voteInProgress", "");
	}

	%this.addAdminMenuItem("Main", 20, 2, "Mission Options", "$Server::Tournament", "AdminMenu.ListMenuData(%client, TournamentMissionOptions);");
	%this.addAdminMenuItem("Main", 20, 3, "Mission Options", "true", "AdminMenu.ListMenuData(%client, FFAMissionOptions);");
	%this.addAdminMenuItem("Main", 23, 2, "Server Setup Options", "%client.testAdmin(ResetServer) || %client.testAdmin(SaveSettings)", "AdminMenu.ListMenuData(%client, ServerOptions);");
	%this.addAdminMenuItem("Main", 24, 2, "Admin Access Options", "%client.testAdmin(ServerAccess)", "AdminMenu.ListMenuData(%client, AccessOptions);");
	%this.addAdminMenuItem("Main", 25, 2, "Login", "true", "AdminMenu.loginAdminClass(%client);");


	%this.addAdminMenuItem("ServerOptions", 1, 1, "Return to Main Menu", "true", "AdminMenu.ListMenuData(%client, Main);");
	%this.addAdminMenuItem("ServerOptions", 10, 1, "Reset the Server", "%client.testAdmin(ResetServer)", "AdminMenu.ResetServer(%client);");
	%this.addAdminMenuItem("ServerOptions", 11, 1, "Save Server Settings", "%client.testAdmin(SaveSettings)", "AdminMenu.SaveServerSettings(%client);");

	%this.addAdminMenuItem("AccessOptions", 1, 1, "Return to Main Menu", "true", "AdminMenu.ListMenuData(%client, Main);");
	%this.addAdminMenuItem("AccessOptions", 2, 1, "Create Admin Class", "true", "AdminMenu.makeNewAdminClass(%client);");
	%this.addAdminMenuItem("AccessOptions", 3, 1, "Edit Admin Class", "true", "AdminMenu.ListMenuData(%client, EditAccessClass);");
	%this.addAdminMenuItem("AccessOptions", 4, 1, "Reset Guest Class", "true", "AdminMenu.ResetGuestClass(%client);");
	%this.addAdminMenuItem("AccessOptions", 5, 1, "Save Admin List", "true", "AdminMenu.SaveAdminAccessData(%client);");

	for(%type=0; %type<$AdminList::AccessLevelMax ;%type++)
	{
		if(%type==0)
			%this.addAdminMenuItem("EditAccessClass", 1, 1, "God", "true", "AdminMenu.ListMenuData(%client, ListGodAccess, 1, God);");
		%name = $AdminList::AccessLevelName[%type];
		%this.addAdminMenuItem("EditAccessClass", %type+2, 1, %name, "true", "AdminMenu.ListMenuData(%client, ListAccessMasks, 1, "@%name@");");
	}
	%this.addAdminMenuItem("ListGodAccess", 1, 1, "Return to Access Options Menu", "true", "AdminMenu.ListMenuData(%client, AccessOptions, 1, \"\");");
	%this.addAdminMenuItem("ListGodAccess", 2, 1, "Change Login Password", "true", "AdminMenu.setClassPassword(%client, God);");


	%this.addAdminMenuItem("PlayerSelected", 1, 1, "Vote Yes", "AdminMenu.voteInProgress && !%client.hasVoted", "ServerCmdIntegrateClientVote(%client, true, 1);");
	%this.addAdminMenuItem("PlayerSelected", 1, 2, "", "AdminMenu.voteInProgress", "");
	%this.addAdminMenuItem("PlayerSelected", 1, 3, "Return to Main Menu", "true", "AdminMenu.ListMenuData(%client, Main);");
	%this.addAdminMenuItem("PlayerSelected", 2, 1, "Vote No", "AdminMenu.voteInProgress && !%client.hasVoted", "ServerCmdIntegrateClientVote(%client, false, 1);");
	%this.addAdminMenuItem("PlayerSelected", 2, 2, "", "AdminMenu.voteInProgress", "");
	%this.addAdminMenuItem("PlayerSelected", 2, 3, "Grant Temp Admin", "%client.testAdmin(GrantAdmin)", "AdminMenu.ListMenuData(%client, GrantAdmin, 1, 0);");
	%this.addAdminMenuItem("PlayerSelected", 3, 1, "", "AdminMenu.voteInProgress", "");
	%this.addAdminMenuItem("PlayerSelected", 3, 2, "Grant Permanent Admin", "%client.testAdmin(GrantAdmin)", "AdminMenu.ListMenuData(%client, GrantAdmin, 1, 1);");
	%this.addAdminMenuItem("PlayerSelected", 4, 1, "", "AdminMenu.voteInProgress", "");

	%this.addAdminMenuItem("GrantAdmin", 1, 1, "Grant Single Level", "true", "AdminMenu.ListMenuData(%client, ListAccessMasks, 1, %client.menuTargetClient);");
	for(%type=0; %type<$AdminList::AccessLevelMax ;%type++)
	{
		%name = $AdminList::AccessLevelName[%type];
		%this.addAdminMenuItem("GrantAdmin", %type+2, 1, "Grant "@%name, "true", "%client.menuTargetClient.grantAdminClass( "@%name@", %client );");
	}

}
AdminMenu.createAdminMenu();




//=============================================================================
//	MAIN SYSTEM FUNCTIONS
//=============================================================================

function AdminMenu::SaveServerSettings(%client)
{
   	export("$Pref::Server::*", "fps/server/prefs.cs", False);
   	messageClient(%client,'MsgSaveServer','\c2You have saved the current server settings as default.');
   	echo("(",%client,") ",%client.namebase," saved the current server settings as default.");
	CommandToClient(%client, 'CloseAdminMenu');
}

function AdminMenu::ResetServer(%client)
{
   	exec("fps/server/defaults.cs");
   	exec("fps/server/prefs.cs");
   	messageClient(%client,'MsgResetServer','\c2You have reset the server to its default settings.');
   	echo("(",%client,") ",%client.namebase," reset the server to its default settings.");
	CommandToClient(%client, 'CloseAdminMenu');
}



