
//=============================================================================
//	VOTE SYSTEM
//=============================================================================

//	This function allows plugin to add new votes to the admin system
//	%type    = Name of the vote type  eg.  "ChangeMission"
//	%text    = Text for the vote      eg.  "Change the Mission"
//	%command = Command that is executed on successful vote
//	(In %command, %target can be used as the target that was sent in the initvote call)
function AdminMenu::addVoteType(%this, %type, %text, %command)
{
	%this.voteText[%type] = %text;
	%this.voteCommand[%type] = %command;
}


//	This function is used to initiate new votes.
//	%client   = client id of initiator of vote
//	%votetype = type of vote declared in addVoteType
//	%target   = target is a variable that can be used in this vote's %command
function AdminMenu::InitVote(%this, %client, %votetype, %target )
{
	//Prevent multiple votes at once
	if(%this.voteInProgress)
	{
		messageClient(%client, 'MsgVoteDenied', "There is already a vote in progress.");
		return;
	}

	//Check if client is immune
	AdminMenu.ListMenuData(%client, "Main");
	%votetext = %this.voteText[%votetype];
	if(%target.name !$= "")
	{
		if(%target.testAdmin(Immunity))
		{
			messageClient(%client, 'MsgVoteDenied', "You cannot initiate a vote against this Admin.");
			messageClient(%target, 'MsgVoteDenied', "A vote to "@%votetext@" you was attempted by %1.",%client.name);
			return;
		}
	}


	//Set all clients as not voted
	%count = ClientGroup.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%cl = ClientGroup.getObject(%i);
		%cl.hasVoted = false;
	}

	// Prevent another vote from being launched and clear previous votes
	%this.voteInProgress = true;
	%this.currentVoteTarget = %target;
	%this.currentVoteType = %votetype;
	for(%i=0; %i<9; %i++)
	{
		%this.votesFor[%i] = 0;
		%this.votesAgainst[%i] = 0;
	}

	//Set Person who initiated the vote as true
	ServerCmdIntegrateClientVote(%client, true, 1);


	//Tell the world a vote is in progress
	if(%target.name)
		messageAll('MsgVoteStart', '%1 initiated a vote to %2 %3.', %client.name, %votetext, %target.name);
	else if(%target)
		messageAll('MsgVoteStart', '%1 initiated a vote to %2 to %3.', %client.name, %votetext, %target);
	else
		messageAll('MsgVoteStart', '%1 initiated a vote to %2.', %client.name, %votetext);

	//Set schedule to finish vote
	%timer = $Pref::Server::VoteTime $= "" ? 30000 : $Pref::Server::VoteTime;
	%this.voteloop = %this.schedule(%timer, "FinishVote", %client, %votetype, %target);
}


//	Vote time is up, lets tally the votes.
//	Variables are all as per initvote
function AdminMenu::FinishVote(%this, %client, %votetype, %target)
{
	//Total all known votes
	for(%i=0; %i<9; %i++)
	{
		%voteFor += %this.votesFor[%i];
		%voteAgainst += %this.votesAgainst[%i];
	}

	//Get Abstaining votes
	%count = ClientGroup.getCount();
	%abstaining = %count - %voteFor - %voteAgainst;
	%abstaining = %abstaining < 0 ? 0 : %abstaining;

	//Determine if vote passes or fails
	%passrate = 0.60;
	%passrate = $Pref::Server::VotePassFactor !$= "" ? $Pref::Server::VotePassFactor : %passrate;
	%factor = %voteFor / %count;

	%this.voteInProgress = false;
	%votetext = %this.voteText[%votetype];
	if(%factor >= %passrate)
	{
		//Vote Passes
		if(%target.name)
			messageAll('MsgVotePass', 'Vote to %1 %5 passed %2 - %3 (%4 Abstaining).', %votetext, %voteFor, %voteAgainst, %abstaining, %target.name);
		else if(%target)
			messageAll('MsgVotePass', 'Vote to %1 to %5 passed %2 - %3 (%4 Abstaining).', %votetext, %voteFor, %voteAgainst, %abstaining, %target);
		else
			messageAll('MsgVotePass', 'Vote to %1 passed %2 - %3 (%4 Abstaining).', %votetext, %voteFor, %voteAgainst, %abstaining);

		//exec command
		%command = %this.voteCommand[%votetype];
		eval(%command);
	}
	else
	{
		//Vote Fails
		if(%target.name)
			messageAll('MsgVoteFail', 'Vote to %1 %5 failed %2 - %3 (%4 Abstaining).', %votetext, %voteFor, %voteAgainst, %abstaining, %target.name);
		else if(%target)
			messageAll('MsgVoteFail', 'Vote to %1 to %5 failed %2 - %3 (%4 Abstaining).', %votetext, %voteFor, %voteAgainst, %abstaining, %target);
		else
			messageAll('MsgVoteFail', 'Vote to %1 failed %2 - %3 (%4 Abstaining).', %votetext, %voteFor, %voteAgainst, %abstaining);
	}

}


//	This function allows an admin to cancel a vote started
//	%client = admins client id
function AdminMenu::CancelVote(%this, %client)
{
	if(!%client.testAdmin(CancelVote))
		return;
	%this.voteInProgress = false;
	CommandToClient(%client, 'CloseAdminMenu');

	cancel(%this.voteloop);

	%votetype = %this.currentVoteType;
	%votetarget = %this.currentVoteTarget;
	%votetext = %this.voteText[%votetype];
	if(%target.name)
		messageAll('MsgVoteFail', 'Vote to %1 %2 was cancelled by Admin.', %votetext, %target.name);
	else if(%target)
		messageAll('MsgVoteFail', 'Vote to %1 to %2 was cancelled by Admin.', %votetext, %target);
	else
		messageAll('MsgVoteFail', 'Vote to %1 was cancelled by Admin.', %votetext);

}


//	Allow client to vote yes or no.
//	%client = client id
//	%vote   = true for yes and false for no
function ServerCmdIntegrateClientVote(%client, %vote, %fromMenu)
{
	if(%fromMenu)
		CommandToClient(%client, 'CloseAdminMenu');

	//check that a vote is currently in progress
	if(!AdminMenu.voteInProgress)
		return;

	//check is a valid client
	%clientauth = false;
	%count = ClientGroup.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%cl = ClientGroup.getObject(%i);
		if(%cl == %client)
		{
			%clientauth = true;
			break;
		}
	}
	if(!%clientauth)
		return;

	//check for one vote per client
	if(%client.hasVoted > 0)
		return;
	%client.hasVoted = true;

	//Record client vote
	%team = %client.team;
	%team = %team $= "" ? 0 : %team;
	if(%vote)
		AdminMenu.votesFor[%team]++;
	else
		AdminMenu.votesAgainst[%team]++;
}


function AdminMenu::CreateVoteTypeListing(%this)
{
	//None as yet

}
AdminMenu.CreateVoteTypeListing();



