//-----------------------------------------------------------------------------
// Torque Game Engine
//
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// RecordingsGui is the main TSControl through which the a demo game recording
// is viewed.
//-----------------------------------------------------------------------------

function menuPlaylists::initialize()
{
   mp_AvailableList.clear();
   %filespec = "fps/data/sound/music/*.ogg";
   for(%file = findFirstFile(%filespec); %file !$= ""; %file = findNextFile(%filespec))
   {
      %fileName = fileBase(%file);
      if (strStr(%file, "/CVS/") == -1)
      {
         %name = strreplace(%filename,"_"," ");
         %time = alxGetOggLen(%file);
         %profilename = "Audio-" @ %filename;
         %profile = new AudioProfile(%profilename)
         {
            filename = %file;
            description = "AudioMusic";
            preload = false;
         };
         mp_AvailableList.addRow(%i++, %name TAB %profilename TAB %time);
      }
   }
   mp_AvailableList.sort(0);
   mp_AvailableList.setSelectedRow(0);
   mp_AvailableList.scrollVisible(0);
   mp_CurrentTime.paused = false;
   mP_CurrentPlaylist.currentList = "";
   menuPlayLists::loadList("menu");
}

function menuPlaylists::close()
{
   Canvas.popDialog(menuPlaylists);
   menuPlaylists::saveList();
}

function menuPlaylists::toggle()
{
   if (!menuPlaylists.isAwake())
      Canvas.pushDialog(menuPlaylists);
   else
      menuPlaylists::close();
}

//------------------------------------------------------------------------------
// Playlist Organizer Functions
//------------------------------------------------------------------------------
function menuPlayLists::loadList(%type)
{
   menuPlaylists::saveList();
   menuPlaylists::stop();
   mP_CurrentPlaylist.clear();
   if (%type $= "battle")
   {
      %amount = $music::battleNum;
      for (%i = 0; %i < %amount; %i++)
      {
         mP_CurrentPlaylist.addRow(%i, $music::battle[%i]);
      }
   }
   if (%type $= "menu")
   {
      %amount = $music::menuNum;
      for (%i = 0; %i < %amount; %i++)
      {
         mP_CurrentPlaylist.addRow(%i, $music::menu[%i]);
      }
   }
   mP_CurrentPlaylist.currentList = %type;
   if (mP_CurrentPlaylist.rowCount() != 0)
   {
      mP_CurrentPlaylist.setSelectedRow(0);
      menuPlaylists::play();
   }
}

function menuPlaylists::addSong()
{
   %id = mp_AvailableList.getSelectedId();
   %song = mp_AvailableList.getRowTextById(%id);
   mP_CurrentPlaylist.addRow(mP_CurrentPlaylist.rowCount(),%song);
}

function menuPlaylists::removeSong()
{
   %id = mP_CurrentPlaylist.getSelectedId();
   mP_CurrentPlaylist.removeRowById(%id);
}

function menuPlaylists::saveList()
{
   //Save lists, make sure to re-order numbers as we cant leave blanks.
   if (mP_CurrentPlaylist.currentList $= "battle")
   {
      $music::battleNum = mP_CurrentPlaylist.rowCount();
      for (%i = 0; %i < mP_CurrentPlaylist.rowCount(); %i++)
         $music::battle[%i] = mP_CurrentPlaylist.getRowText(%i);
   }
   if (mP_CurrentPlaylist.currentList $= "menu")
   {
      $music::menuNum = mP_CurrentPlaylist.rowCount();
      for (%i = 0; %i < mP_CurrentPlaylist.rowCount(); %i++)
         $music::menu[%i] = mP_CurrentPlaylist.getRowText(%i);
   }
}

function menuPlaylists::clearList()
{
   mP_CurrentPlaylist.clear();
}

//------------------------------------------------------------------------------
// Music Player Functions
//------------------------------------------------------------------------------

function menuPlaylists::playall()
{
   if ($GUI::MusicStopped != true)
      menuPlaylists::play();
}

function menuPlaylists::play()
{
   if (mP_CurrentPlaylist.rowCount() == 0)
      return;
   if (mP_CurrentPlaylist.getSelectedId() == -1)
      return;
   %id = mP_CurrentPlaylist.getSelectedId();
   %song = getField(mP_CurrentPlaylist.getRowTextById(%id),1);
   %name = getField(mP_CurrentPlaylist.getRowTextById(%id),0);
   %time = getField(mP_CurrentPlaylist.getRowTextById(%id),2);
   if (%time == 0)
   {
      %time = alxGetOggLen(%file);
      if (%time == 0)
          %time = 120;
   }
   %min = mFloor(%time / 60);
   %sec = mFloor(%time - (60*%min));
   if (%sec < 10) %sec = "0" @ %sec;
   if (%min < 10) %min = "0" @ %min;
   //mp_CurrentTime.pauseTime(false);
   mp_CurrentTime.setTime(0);
   mp_TotalTime.setText("/" SPC %min @ ":" @ %sec);
   mp_CurrentSongName.setText("Current Song:" SPC %name);
   //Stop Current Song, Start New Song.
   alxStop($GUI::MusicHandle);
   cancel($GUI::MusicSchedule);
   $GUI::MusicHandle = alxPlay(%song);
   alxPlay($GUI::MusicHandle);
   $GUI::MusicSchedule = schedule(%time * 1000, 0, "nextSong");
}

function menuPlaylists::stopall()
{
   //alxStopAll();
   alxStop($GUI::MusicHandle);
   //mp_CurrentTime.pauseTime(true);
   cancel($GUI::MusicSchedule);
   $GUI::MusicStopped = true;
}

function menuPlaylists::stop()
{
   //alxStopAll();
   alxStop($GUI::MusicHandle);
   //mp_CurrentTime.pauseTime(true);
   cancel($GUI::MusicSchedule);
}

function menuPlaylists::pause()
{
   mp_CurrentTime.paused = !mp_CurrentTime.paused;
   mp_CurrentTime.pauseTime(mp_CurrentTime.paused);
   alxPause($GUI::MusicHandle,mp_CurrentTime.paused);
}

//Schedule Helper Function
function nextSong() { menuPlaylists::fforward(); }
function menuPlaylists::fforward()
{
   %id = mP_CurrentPlaylist.getSelectedId();
   %row = mP_CurrentPlaylist.getRowNumById(%id);
   if (%row + 1 == mP_CurrentPlaylist.rowCount())
      %row = 0;
   else
      %row += 1;
   mP_CurrentPlaylist.setSelectedRow(%row);
   cancel($GUI::MusicSchedule);
   menuPlaylists::play();
}

function menuPlaylists::rewind()
{
   %id = mP_CurrentPlaylist.getSelectedId();
   %row = mP_CurrentPlaylist.getRowNumById(%id);
   if (%row - 1 == -1)
      %row = mP_CurrentPlaylist.rowCount();
   else
      %row -= 1;
   mP_CurrentPlaylist.setSelectedRow(%row);
   cancel($GUI::MusicSchedule);
   menuPlaylists::play();
}
