//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Mission Loading & Mission Info
// The mission loading server handshaking is handled by the
// common/client/missingLoading.cs.  This portion handles the interface
// with the game GUI.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Loading Phases:
// Phase 1: Download Datablocks
// Phase 2: Download Ghost Objects
// Phase 3: Scene Lighting

//----------------------------------------------------------------------------
// Phase 1
//----------------------------------------------------------------------------

function onMissionDownloadPhase1(%missionName, %musicTrack)
{
   // Close and clear the message hud (in case it's open)
   gameMessageHud.close();
   //cls();

   // Reset the loading progress controls:
   LoadingProgress.setValue(0);
   LoadingProgressTxt.setValue("INITIALIZING NEURAL NET");
}

function onPhase1Progress(%progress)
{
   LoadingProgress.setValue(%progress);
   Canvas.repaint();
}

function onPhase1Complete()
{
}

//----------------------------------------------------------------------------
// Phase 2
//----------------------------------------------------------------------------

function onMissionDownloadPhase2()
{
   // Reset the loading progress controls:
   LoadingProgress.setValue(0);
   LoadingProgressTxt.setValue("LINKING NEURAL PATHWAYS TO MAV");
   Canvas.repaint();
}

function onPhase2Progress(%progress)
{
   LoadingProgress.setValue(%progress);
   Canvas.repaint();
}

function onPhase2Complete()
{
}   

//----------------------------------------------------------------------------
// Phase 3
//----------------------------------------------------------------------------

function onMissionDownloadPhase3()
{
   LoadingProgress.setValue(0);
   LoadingProgressTxt.setValue("MAV POWER UP");
   Canvas.repaint();
}

function onPhase3Progress(%progress)
{
   LoadingProgress.setValue(%progress);
}

function onPhase3Complete()
{
   LoadingProgress.setValue( 1 );
   $lightingMission = false;
}

//----------------------------------------------------------------------------
// Mission loading done!
//----------------------------------------------------------------------------

function onMissionDownloadComplete()
{
   // Client will shortly be dropped into the game, so this is
   // good place for any last minute gui cleanup.
}


//------------------------------------------------------------------------------
// Before downloading a mission, the server transmits the mission
// information through these messages.
//------------------------------------------------------------------------------

addMessageCallback( 'MsgLoadInfo', handleLoadInfoMessage );
addMessageCallback( 'MsgLoadDescripition', handleLoadDescriptionMessage );
addMessageCallback( 'MsgLoadObjective', handleLoadObjectiveMessage );
addMessageCallback( 'MsgLoadInfoDone', handleLoadInfoDoneMessage );

//------------------------------------------------------------------------------
function handleLoadInfoMessage( %msgType, %msgString, %mapName ) {
	
	// Need to pop up the loading gui to display this stuff.
	Canvas.setContent("GameLoadingGui");

	// Clear all of the loading info lines:
	for( %line = 0; %line < GameLoadingGui.qLineCount; %line++ )
		gameLoadingGui.qLine[%line] = "";
	gameLoadingGui.qLineCount = 0;
	for( %line = 0; %line < gameHudFS.qLineCount; %line++ )
		gameHudFS.qLine[%line] = "";
	gameHudFS.qLineCount = 0;
   //
	LOAD_MapName.setText( %mapName );
}

//------------------------------------------------------------------------------
function handleLoadDescriptionMessage( %msgType, %msgString, %line )
{
	gameLoadingGui.qLine[gameLoadingGui.qLineCount] = %line;
	gameLoadingGui.qLineCount++;

   // Gather up all the previous lines, append the current one
   // and stuff it into the control
 	for( %line = 0; %line < gameLoadingGui.qLineCount - 1; %line++ )
		%text = %text @ gameLoadingGui.qLine[%line] @ " ";
    %text = %text @ gameLoadingGui.qLine[%line] @ "<spop>";

	LOAD_MapDescription.setText( %text );
}

function handleLoadObjectiveMessage( %msgType, %msgString, %line )
{
	gameHudFS.qLine[gameHudFS.qLineCount] = %line;
	gameHudFS.qLineCount++;

   // Gather up all the previous lines, append the current one
   // and stuff it into the control
 	for( %line = 0; %line < gameHudFS.qLineCount - 1; %line++ )
		%text = %text @ gameHudFS.qLine[%line] NL "";
    %text = %text @ gameHudFS.qLine[%line] NL "";

   EC_objectives.setText( %text );
   FS_objectives.setText(%text);
}

//------------------------------------------------------------------------------

function handleLoadInfoDoneMessage( %msgType, %msgString )
{
   // This will get called after the last description line is sent.
}
