//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

// Variables used by server scripts & code.  The ones marked with (c)
// are accessed from code.  Variables preceeded by Pref:: are server
// preferences and stored automatically in the ServerPrefs.cs file
// in between server sessions.
//
//    (c) Server::ServerType              {SinglePlayer, MultiPlayer}
//    (c) Server::GameType                Unique game name
//    (c) Server::Dedicated               Bool
//    ( ) Server::MissionFile             Mission .mis file name
//    (c) Server::MissionName             DisplayName from .mis file
//    (c) Server::MissionType             Not used
//    (c) Server::PlayerCount             Current player count
//    (c) Server::GuidList                Player GUID (record list?)
//    (c) Server::Status                  Current server status
//
//    (c) Pref::Server::Name              Server Name
//    (c) Pref::Server::Password          Password for client connections
//    ( ) Pref::Server::AdminPassword     Password for client admins
//    (c) Pref::Server::Info              Server description
//    (c) Pref::Server::MaxPlayers        Max allowed players
//    (c) Pref::Server::RegionMask        Registers this mask with master server
//    ( ) Pref::Server::BanTime           Duration of a player ban
//    ( ) Pref::Server::KickBanTime       Duration of a player kick & ban
//    ( ) Pref::Server::MaxChatLen        Max chat message len
//    ( ) Pref::Server::FloodProtectionEnabled Bool

//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------


function initServer()
{
   echo("\n--------- Initializing FPS: Server ---------");

   // Server::Status is returned in the Game Info Query and represents the
   // current status of the server. This string sould be very short.
   $Server::Status = "Unknown";

   // Turn on testing/debug script functions
   $Server::TestCheats = true;

   // Specify where the mission files are.
   $Server::MissionFileSpec = "*/missions/*.mis";

   // The common module provides the basic server functionality
   initBaseServer();

   //read banlist
   exec("./banlist.cs");

   // Load up game server support scripts
   exec("./scripts/commands.cs");
   exec("./scripts/centerPrint.cs");
   exec("./scripts/game.cs");
   exec("./scripts/backend.cs");
}


//-----------------------------------------------------------------------------

function initDedicated()
{
   enableWinConsole(true);
   echo("\n--------- Starting Dedicated Server ---------");
   exec("./dedicated.cs");
   $Backend::PlayerGroup = new simGroup();
   // Make sure this variable reflects the correct state.
   $Server::Dedicated = true;
   // Set the GUI variables based on dedicated server config file
   // This is simpler (and more organized) then putting if statements everywhere else
   
   $GUI::Practice   = $Dedicated::Practice;
   $GUI::ServerName = $Dedicated::ServerName;
   $GUI::ServerPass = $Dedicated::Password;
   $GUI::MaxPlayers = $Dedicated::MaxPlayers;
   if (!$GUI::Practice)
      $GUI::Reporting  = $Dedicated::ReportStats;
   $GUI::Duration   = true;
   $GUI::DurationTime = ($Dedicated::Duration) ? $Dedicated::Duration : 20;
   // The server isn't started unless a mission has been specified.
   if ($missionArg !$= "")
   {
      $Dedicated::Mode = 1;
      $Dedicated::MissionFile = $missionArg;
   }
   if ($typeArg !$= "")
   {
      $Dedicated::Mode = 2;
      $Dedicated::GameType = $typeArg;
   }
   if ($Dedicated::MissionFile !$= "" || $Dedicated::GameType !$= "" || $Dedicated::Map[0] !$= "")
   {
      if ($Server::DedicatedLan)
      {
         createServerLan("MultiPlayer");
      }
      else
      {
         createServer("MultiPlayer");
      }
   }
   else
   {
      echo("No mission type specified (use -type gametype)");
   }
}

