//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

// The AIPlayer C++ class implements the following methods.
//
// AIPlayer::stopMove();
//    Stop the player where he is.
//
// AIPlayer::clearAim();
//    Clear any current aim location or aim object.  The player will
//    maintain his current orientation, or face towards his destination
//    if moving.
//
// AIPlayer::setMoveSpeed( float );
//    Set the speed (0-1) at which the player moves.
//
// AIPlayer::setMoveDestination( "x y z" );
//    Set the destination point to move towards. The Z location is ignored
//    though the player will face upwards (or downards) towards the destination
//    if no other aim target is set.
//
// AIPlayer::getMoveDestination();
//    Returns the current destination point.
//
// AIPlayer::setAimLocation( \"x y z\" );
//    Set a point to look at.  The player maintains his orientation towards
//    this point even while moving towards a destination point.
//
// AIPlayer::setAimObject( obj );
//    Set an object to look at.  The player maintains his orientation towards
//    this object, even while moving towards a destination point. The aim
//    object can also be moving.
//
// AIPlayer::getAimLocation();
//    Returns the current point the player is looking at. If the player has
//    an aimObject set, this would be the current position of the target
//    object.
//
// AIPlayer::getAimObject();
//    Returns the current target object, if there is one.

//-----------------------------------------------------------------------------
// Callback Handlers
//-----------------------------------------------------------------------------

function Mecha::onStuck(%this,%obj)
{
   // Invoked if the player is stuck while moving
   // This method is currently not being invoked correctly.
   echo( "I'm stuck" );
}

function Mecha::onReachDestination(%this,%obj)
{
    // Invoked when the player arrives at his destination point.
  // echo( "onReachDestination" );
   //if more waypoints go to the next

     //if(%this.countMyMoves < BotMarkers.getCount())
     if(%obj.looping)
     %obj.loopWaypoints(%obj.wayPointArea);

     if(%obj.patroling)
     %obj.patrolWaypoints(%obj.wayPointArea);
     
      if(%obj.training)
      %obj.determineNextMove(%obj.wayPointArea);

}

function Mecha::onTargetEnterLOS(%this,%obj)
{
   // If an aim target object is set, this method is invoked when
   // that object becomes visible.
//   echo( "onTargetEnterLOS" );
}

function Mecha::onTargetExitLOS(%this,%obj)
{
   // If an aim target object is set, this method is invoked when
   // the object is not longer visible.
//   echo( "onTargetExitLOS" );
}


//-----------------------------------------------------------------------------

function AIPlayer::spawnPlayer(%name,%client)
{
   if( %client.team.teamId $= 1)
   {
   %player = new AIPlayer() {
          //default player the "BLUE MECHA" let's change the lightmalehumanarmor name PLEASE
          dataBlock = Shaddrack3;
          aiPlayer = true;
          };
   }
   else{
   //TEAM2's default player the SLASHER!!!!!
     %player = new AIPlayer() {
        dataBlock = Locust3;
        aiPlayer = true;
       };
   }

   MissionCleanup.add(%player);

   // Player setup
   %player.setMoveSpeed(1);
   //spawns at location of player who created him
   %player.setTransform(pickSpawnPoint(%client));
   %player.setEnergyLevel(60);
   %player.setShapeName(%name);

   %player.incInventory(gatling2, 1);
   %player.incInventory(gatlingAmmo2, GatlingAmmo2.maxInventory);
   %player.use(Gatling2);

   return %player;
}   


function AIPlayer::spawnEnemyPlayer(%name,%client)
{
   %player = new AIPlayer() {
      //default player the "Grey Mecha"
      dataBlock = Locust3;
      aiPlayer = true;
   };
          

   MissionCleanup.add(%player);

   // Player setup
   %player.setMoveSpeed(1);
   //spawns at location of correct place
   if( %client.team.teamId == 1)
   %player.setTransform(pickSpawnPoint(%client));
   else
   %player.setTransform(pickSpawnPoint(%client));

   %player.setEnergyLevel(60);
   %player.setShapeName(%name);

   %player.incInventory(gatling2, 1);
   %player.incInventory(gatlingAmmo2, GatlingAmmo2.maxInventory);
   %player.use(Gatling2);

   return %player;
}

function pickAIMinerSpawnPoint(%spawnArea)
{

   %groupName = "MissionGroup/AIMinerPoints";
   %groupEnd = -1;
   %sphereCount = 0;

   while(nameToID(%groupName @ %sphereCount)!= -1)
   {
        %sphereCount++;
   }
   if (%sphereCount != 0)
   {
   %groupEnd = getRandom(%sphereCount - 1)  ;
   //set the random generated number to be passed later to player
   %spawnArea.choice =  %groupEnd;
    
   %randomGoupName =  %groupName @ %groupEnd;

   %group = nameToID(%randomGoupName);
   %spawnArea =  %groupEnd;
   if (%group != -1) {
      %count = %group.getCount();
      if (%count != 0) {
         %index = getRandom(%count-1);
         %spawn = %group.getObject(%index);
         //set postion to be use later with player
           return  getTerrainLevel(%spawn.position,%spawn.radius);

      }
      else
         error("No spawn points found in " @ %groupName);
   }
   else
      error("Missing spawn points group " @ %groupName);
   }
   return  false;
   // Could be no spawn points, in which case we'll stick the
   // player at the center of the world.
   //If no spawn point spawnArea.position will still == "" ie nothing our flag
}

// Jimomighty function callbacks
function Mecha::onJump(%this,%obj)
{
//   echo( "onJump" );
}
function Mecha::onUnJump(%this,%obj)
{
}
function Mecha::onShoot(%this,%obj)
{
//   echo( "onShoot" );
}
function Mecha::onUnShoot(%this,%obj)
{
}

function Mecha::onFollowTargetEnterLOS(%this,%obj)
{
//   echo( "onFollowTargetEnterLOS" );
}
function Mecha::onFollowTargetExitLOS(%this,%obj)
{
//   echo( "onFollowTargetExitLOS" );
}
function Mecha::onFollowTargetKilled(%this,%obj)
{
//   echo( "onFollowTargetKilled" );
}

function Mecha::onAttackTargetEnterLOS(%this,%obj)
{
//   echo( "onAttackTargetEnterLOS" );
}
function Mecha::onAttackTargetExitLOS(%this,%obj)
{
 //  echo( "onAttackTargetExitLOS" );
}
function Mecha::onAttackTargetKilled(%this,%obj)
{
 //  echo( "onAttackTargetKilled" );
}

function AIPlayer::processAttackPlayers(%this)
{
 echo("processing Players");
 //determine who is on the bots' team
 echo("this bot is on team " @ %this.team.teamId);
    %count = ClientGroup.getCount();
   for( %cl = 0; %cl < %count; %cl++ ) {
      %client = ClientGroup.getObject( %cl );

      if( %this.team.teamId $= %client.team.teamId)
        %this.setFollowTargetObject(%client.player);
      else
        %this.setAttackTargetObject(%client.player);
        %this.setScanningPlayers(true);
   }

}


//function AIPlayer::processPatrolPlayers(%this)
function AIPlayer::processPlayers(%this)
{
 echo("processing Players");
 //determine who is on the bots' team
// echo("this bot is on team " @ %this.team.teamId);
    %count = ClientGroup.getCount();
   for( %cl = 0; %cl < %count; %cl++ ) {
      %client = ClientGroup.getObject( %cl );

      //find other AI ?!?!?!? WHERE ARE YOU ?
      if(%client.isAIControlled())
      {
           echo("THERE IS A BOT");
           %client.dump();
      }
      

      if( %this.team.teamId $= %client.team.teamId)
      //continue;
      %this.loopWaypoints(%this.wayPointArea);

      if( %this.team.teamId != %client.team.teamId)
      {
      echo("THERE IS A possible enemy");
      //first time thru
         if(!%this.lastTargetDistance)
         {
         echo("First Time ");
         %this.setAttackTargetObject(%client.player);
         %this.lastTargetDistance = %this.getTargetDistance();
         %this.lastTarget =  %client;
         //%this.setFollowTargetObject(%client.player);

         }else//already chasing some one
         {
         echo(" let me see if you are closer?");
         //old target
         %tempDist = %this.lastTargetDistance;
         //clear aim  pause to analize
          %tempOb =  %this.lastTarget;

         %this.clearAllTargets();

         //lock on new target
         %this.setAttackTargetObject(%client.player);
         %this.lastTargetDistance = %this.getTargetDistance();

         //if the old target is closer follow it
         if( %this.lastTargetDistance > %tempDist )
         {
             %this.clearAllTargets();
             echo("You are not closer!!! Lucky YOU !!!");
             %this.setAttackTargetObject( %tempOb.player);
             %this.lastTargetDistance = %tempDist;
             // %this.setFollowTargetObject(%tempObject);
         }else
         %this.lastTarget =  %client;
       }
       
     }
  }
 // echo("schedualing");
  //%this.schedule(9999, "processPlayers");
}

function AIPlayer::followWaypoints(%this)
{
// but first look for some waypoints...
// if there are some waypoints to follow
   if(isObject(BotMarkers))
   {

   %num = getNumberBotMarkers();
   // %this.countMyMoves tracks the number of waypoints already passed by this AI
   if(%this.countMyMoves $= "")
         {  //echo number of waypoint once
             echo(%num @ " BotMarkers found!");
         }
         
   // as long as this bot hasn't completed the waypoint path:
   if(%this.countMyMoves < BotMarkers.getCount())      {
       %dest = BotMarkers.getObject(%this.countMyMoves).position;
       echo("Next destination: " @ %dest);
       echo("The distance is: " @ getMarkerDistance(%this.countMyMoves, %this));
       echo("myMoves along the waypoint path:" SPC %this.countMyMoves);
       this.setAimLocation(%dest);
       %this.setMoveDestination(%dest);
       //%this.move();
       %this.countMyMoves++;
       }
   }
}
function AIPlayer::loopWaypoints(%this, %wayPointArea)
{
// but first look for some waypoints...
// if there are some waypoints to follow
    %botArea = "BotMarkers" @ %wayPointArea;
    //name with local area
    
   if(isObject(%botArea))
   {
   %num = getNumberBotMarkers(%wayPointArea);
   // %this.countMyMoves tracks the number of waypoints already passed by this AI
   if(%this.countMyMoves $= "")
         {  //echo number of waypoint once
            // echo(%num @ " BotMarkers found!");
         }
   %this.looping = true;

   
   // as long as this bot hasn't completed the waypoint path:
   if(%this.countMyMoves < %botArea.getCount())      {
       %dest = %botArea.getObject(%this.countMyMoves).position;
       //echo("Next destination: " @ %dest);
       //echo("The distance is: " @ getMarkerDistance(%this.countMyMoves, %this,%wayPointArea));
       //echo("myMoves along the waypoint path:" SPC %this.countMyMoves);
       %this.setAimLocation(%dest);
       %this.setMoveDestination(%dest);
       //%this.move();
       %this.countMyMoves++;
       if(%this.countMyMoves == %botArea.getCount())
       %this.countMyMoves = 0;
       //echo("my moves " @  %this.countMyMoves);
       }
   }
}


function AIPlayer::patrolWaypoints(%this,%wayPointArea)
{
// but first look for some waypoints...
// if there are some waypoints to follow
   %botArea = "BotMarkers" @ %wayPointArea;

   if(isObject(%botArea))
   {
   %num = getNumberBotMarkers(%wayPointArea);
   // %this.countMyMoves tracks the number of waypoints already passed by this AI
   if(%this.countMyMoves $= "")
         {  //echo number of waypoint once
            // echo(%num @ " BotMarkers found!");
         }

   if(%this.reverse $= "")
     {
     %this.reverse = false;
     }

   %this.patroling = true;
   // as long as this bot hasn't completed the waypoint path:
   if(%this.countMyMoves < %botArea.getCount() && (!%this.reverse))
      {
       %dest = %botArea.getObject(%this.countMyMoves).position;
       //echo("Next destination: " @ %dest);
       //echo("The distance is: " @ getMarkerDistance(%this.countMyMoves, %this,%wayPointArea));
       //echo("myMoves along the waypoint path:" SPC %this.countMyMoves);
       %this.setAimLocation(%dest);
       %this.setMoveDestination(%dest);
       //%this.move();
       %this.countMyMoves++;
       if(%this.countMyMoves == %botArea.getCount())
         {
            %this.reverse = true;
            %this.countMyMoves--;
         }
      }
   if(  %this.reverse )
      {
      %dest = %botArea.getObject(%this.countMyMoves).position;
       //echo("Next destination: " @ %dest);
       //echo("The distance is: " @ getMarkerDistance(%this.countMyMoves, %this));
       //echo("myMoves along the waypoint path:" SPC %this.countMyMoves);
       %this.setAimLocation(%dest);
       %this.setMoveDestination(%dest);
       //%this.move();
       %this.countMyMoves--;
       if(%this.countMyMoves <= 0)
          {
           %this.reverse = false;
           %this.countMyMoves = 0;
           }
       //echo("my moves REVERSE" @  %this.countMyMoves);
      }
   }
}


function getNumberBotMarkers(%wayPointArea){
         %botArea = "BotMarkers" @ %wayPointArea;
         %count = %botArea.getCount();
         return %count;
}
function getMarkerDistance(%markernumber, %bot,%wayPointArea){
   %botArea = "BotMarkers" @ %wayPointArea;
   %botPosition = %bot.position;
   %markerObject = %botArea.getObject(%markernumber);
   %dist = vectorDist( %botPosition, %markerObject.position );
   return %dist;
}

function updateAITargetSets()
{
  echo("updating AI targetSets");
  for(%i = 0; %i < $botCounter; %i++)
  {
  $bots[%i].clearAim();
  $bots[%i].processPlayers();
  }
}
function AddBots(%client) {
         MissionCleanup.add($bots);
	     %npcName = "Bot" @ $botCounter;
         echo("adding Bot" @  $botCounter);

         //NEW CODE
         //creat temp to hold bot to see if it exisits
          %newBot = AIPlayer::spawnSpiderMinerPlayer(%npcName,%client);
         echo("NEW BOT is " @ %newBot);
         if(  %newBot !$= "noBOT" )
         { //it DOES add it to the ARRAY
         $bots[$botCounter] = %newBot ;
         MissionCleanup.add($bots[$botCounter]);
         //$bots[$botCounter].processPlayers();
         //PASS AREA TO LOOPING FUNCTION TO SET TO POSITION
         $bots[$botCounter].loopWaypoints($bots[$botCounter].wayPointArea);
         $botCounter++;
         }
       else //NO BOT WAS MADE, IE NO AREA FOR THE BOT TO BE MADE
           error("no bot made") ;

}


function generateBots(%maxBots)
{
$once = false;
if(isObject(BotMarkers0))
   {
   %i = 0;
    while(%i < %maxBots)
     {
     echo("Max Bots for mission is " @ %maxBots);
      addBots();
       echo("Bots is " @ %i);
      %i++;
     }
    }
else
echo("No BOTS CAN BE HERE");
}
// This is for the scripted way point code - uses botWayfinding.cs
//  http://www.garagegames.com/index.php?sec=mg&mod=resource&page=view&qid=3968
// Player reached destination waypoint, continue to next waypoint
// NOTE: This is called from C++ whenever a bot has reached it&#180;s moveDestination

//"%this" is the bot's "%player"
//function Armor::onReachDestination(%this,%obj)
//{
//   if (%this.waypointsSet == true)
//   {
//   echo("[BOT] Next Waypoint in List:" SPC %this.nextWaypoint);
//      %next = %this.waypointList[%this.nextWaypoint];
//   BOT_setNextWaypoint(%this, %next);
//   %this.nextWaypoint++;
//   if (%this.nextWaypoint == (%this.numWaypoints))
//   {
//      BOT_clearWaypoints(%this);
//      %this.setMoveSpeed(0.0);
//   }
//   }
//}


//Now all you need is to call BOT_findPath(%bot, %targetLocation), and the
//bot will automatically start to head towards the target location using the
//waypoints. Again, the target location should be near a waypoint, or it wont
//work. Even though this code is scripted, its more than capable of providing
//navigation support for 20-30 bots without noticeable framerate drops.
//
//A last word of warning: While this code works perfectly, its just navigation
//code - not a complete bot. Its your responsibility to keep your bots CPU
//usage low. One word of advice: Split up the bots thinking into a high- and
//a low-level part. Reschedule the low-level thinking (shoot, engage targets of
//opportunity) every 100-200ms, and the high-level stuff (objectives, which
//mission goal to go after) every 3-5 seconds.

function AIPlayer::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc)
{
   if (%this.getShapeName() $= "Garvin")
   {
      if($singlePlayerLevel = 4)
      {
      youLose();
      }
   }
   if (%sourceClient.aiPlayer)
   {
      %Name = %sourceClient.getShapeName();
      %this.incDeaths(1);
      if(%this.team.teamID == %sourceClient.team.teamID)
      {
         messageAll('MsgClientKilled','\c5%1 killed his \c4team member \c5%2!',%Name,%this.name, %this.team.name);
      } else {
         //.death on killed, .kills on killer
         messageAll('MsgClientKilled','\c5%1 is destroyed by %2!',%this.name,%Name);
      }
   }
   else
   {
   // Doll out points and display an appropriate message
   if(%this.team.teamID == %sourceClient.team.teamID)
   {
      %sourceClient.incScore(-$Game::PointsForKill);
      %sourceClient.incTeamScore(-$Game::PointsForKill);
      %sourceClient.incKills(1);
      messageAll('MsgClientKilled','\c5%1 killed his \c4team member \c5%2!',%sourceClient.name,%this.getShapeName(), %this.team.name);
   } else {
      //.death on killed, .kills on killer
      %sourceClient.incScore($Game::PointsForKill);
      %sourceClient.incTeamScore($Game::PointsForKill);
      %sourceClient.incKills(1);
      messageAll('MsgClientKilled','\c5%1 is destroyed by %2!',%this.getShapeName(),%sourceClient.name);
   }
   }
   // Clear out corpse name and delete player.
   %this.setShapeName("");

   if ($Game::EndGameScore > 0) {
      if (%sourceClient.score >= $Game::EndGameScore)
      {
        cycleGame();
      }
   }
}
