//-----------------------------------------------------------------------------
// Torque Game Engine
//
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

// Load dts shapes and merge animations
exec("~/data/shapes/player/player.cs");

//----------------------------------------------------------------------------
// Mecha Datablock methods
//----------------------------------------------------------------------------

//NEVER CALLED
function Mecha::onAdd(%this,%obj)
{
    // phdana strafe anim ->
    Parent::onAdd(%this,%obj);
    // <- phdana strafe anim
    
   %obj.setHeat(%this.heat);
   // Vehicle timeout
   %obj.mountVehicle = true;
   // Default dynamic armor stats
   %obj.setRechargeRate(%this.rechargeRate);
   %obj.setRepairRate(0);
}

function Mecha::onRemove(%this, %obj)
{
   // phdana strafe anims ->
   Parent::onRemove(%this,%obj);
   // <- phdana strafe anims

   if (%obj.client.player == %obj)
      %obj.client.player = 0;
}


function Mecha::onNewDataBlock(%this,%obj)
{
}


//----------------------------------------------------------------------------

function Mecha::onMount(%this,%obj,%vehicle,%node)
{
   if (%node == 0)  {
      %obj.setTransform("0 0 0 0 0 1 0");
      %obj.setActionThread(%vehicle.getDatablock().mountPose[%node],true,true);
      %obj.lastWeapon = %obj.getMountedImage($WeaponSlot);

      %obj.unmountImage($WeaponSlot);
      %obj.setControlObject(%vehicle);
      %obj.client.setObjectActiveImage(%vehicle, 2);
   }
}

function Mecha::onUnmount( %this, %obj, %vehicle, %node )
{
   if (%node == 0)
      %obj.mountImage(%obj.lastWeapon, $WeaponSlot);
}

function Mecha::doDismount(%this, %obj, %forced)
{
   // This function is called by player.cc when the jump trigger
   // is true while mounted
   if (!%obj.isMounted())
      return;

   // Position above dismount point
   %pos    = getWords(%obj.getTransform(), 0, 2);
   %oldPos = %pos;
   %vec[0] = " 0  0  1";
   %vec[1] = " 0  0  1";
   %vec[2] = " 0  0 -1";
   %vec[3] = " 1  0  0";
   %vec[4] = "-1  0  0";
   %impulseVec  = "0 0 0";
   %vec[0] = MatrixMulVector( %obj.getTransform(), %vec[0]);

   // Make sure the point is valid
   %pos = "0 0 0";
   %numAttempts = 5;
   %success     = -1;
   for (%i = 0; %i < %numAttempts; %i++) {
      %pos = VectorAdd(%oldPos, VectorScale(%vec[%i], 3));
      if (%obj.checkDismountPoint(%oldPos, %pos)) {
         %success = %i;
         %impulseVec = %vec[%i];
         break;
      }
   }
   if (%forced && %success == -1)
      %pos = %oldPos;

      // phdana turrets ->
   if(%obj.mVehicle)
   {
      %data = %obj.mVehicle.getDataBlock();
      if (%data.className $= TurretData)
         %data.playerDismounted(%obj.mVehicle, %obj);

      %obj.mVehicle.mountable = true;
	  %client = %obj.client;
 	  if(%obj.mVehicle.mountedTurret.currentDriver == %client)
 		%obj.mVehicle.mountedTurret.currentDriver = "";
   }
   // <- phdana turrets

   %obj.mountVehicle = false;
   %obj.schedule(4000, "setMountVehicle", true);

   // Position above dismount point
   %obj.setTransform(%pos);
   %obj.applyImpulse(%pos, VectorScale(%impulseVec, %obj.getDataBlock().mass));
   %obj.setPilot(false);
   %obj.vehicleTurret = "";
}


//----------------------------------------------------------------------------

function Mecha::onCollision(%this,%obj,%col,%vec,%speed)
{
   if (%obj.getState() $= "Dead")
      return;
   if (%col.getType() & $TypeMasks::DamagableItemObjectType)
      return;
   if (%col.getType() & $TypeMasks::PlayerObjectType)
   {
      if (%col.getState() $= "Dead")
         return;
   }
   if (%speed < 10)
      return;
      
   // Try and pickup all items
   if (%col.getClassName() $= "Item")
      %obj.pickup(%col);
   if (%obj.getVelocity() $= "0 0 0") {
       //Unmoving target. Return dmg is 1/2 of colliding objects speed
      %damage = %col.getDatablock().mass * %speed /2000;
   } else if (%col.getVelocity() $= "0 0 0") {
      //Unmoving target (Both Target and Collider call onCollision so both events must be catched)
      %damage = %obj.getDatablock().mass * %speed /4000;
   } else { // 2 moving targets
      %normal = VectorNormalize(VectorCross(%col.getVelocity(),%obj.getVelocity()));
      %normalscaled = VectorLen(VectorScale(%col.getVelocity(),%normal));
      %damage =  (%speed * %normalscaled) * (%col.getDatablock().mass / 2000);
   }
   if (%damage > 20)
   {
      %obj.damage(%col,%obj.getPosition(),mFloor(%damage),"COL");
   }
}



function Mecha::onImpact(%this, %obj, %collidedObject, %vec, %vecLen)
{
   %obj.damage(%obj, VectorAdd(%obj.getPosition(),%vec),
      %vecLen * %this.speedDamageScale, "Impact");
}


//----------------------------------------------------------------------------

function Mecha::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
   if (%obj.getState() $= "Dead")
      return;
   if (%obj.getShapeName() $= "Captain Garvin")
      return;
      
   %obj.applyDamage(%damage);
   // Deal with client callbacks here because we don't have this
   // information in the onDamage or onDisable methods
   %sourceClient = %sourceObject ? %sourceObject.client : 0;
   if (%obj.getState() $= "Dead")
   {
      %obj.onDeath(%sourceObject, %sourceClient, %damageType, %location);
      // Cause us some radius damage!
      %pos = %obj.getPosition();
      radiusDamage(%sourceObject,%pos,%this.damageRadius,%this.radiusDamage,"MAV",100);

   }
      //change from %client. to %obj. so AI will report death
}

function Mecha::onDamage(%this, %obj, %delta)
{
   // This method is invoked by the ShapeBase code whenever the
   // object's damage level changes.
   if (%delta > 0 && %obj.getState() !$= "Dead") {

      // Increment the flash based on the amount.
      %flash = %obj.getDamageFlash() + ((%delta / %this.maxDamage) * 2);
      if (%flash > 0.75)
         %flash = 0.75;
      %obj.setDamageFlash(%flash);

      // If the pain is excessive, let's hear about it.
      if (%delta > 10)
         %obj.playPain();
   }
}

function Mecha::onDisabled(%this,%obj,%state)
{
   // The player object sets the "disabled" state when damage exceeds
   // it's maxDamage value.  This is method is invoked by ShapeBase
   // state mangement code.
   if ( %obj.getMountedImage($FlagSlot) != 0) {
   echo("disabled dropping flag");
     $Flag::fromDeath = true;
   	newflag(%obj.getMountedImage($FlagSlot).item,%obj);
   	%obj.unmountImage($FlagSlot);
   	}
   // If we want to deal with the damage information that actually
   // caused this death, then we would have to move this code into
   // the script "damage" method.
   %obj.playDeathCry();
   %obj.playDeathAnimation();
   %obj.setDamageFlash(0.75);
     // alxstopall();

   // Release the main weapon trigger
   %obj.setImageTrigger(0,false);

   // Schedule corpse removal.  Just keeping the place clean.
   %obj.schedule($CorpseTimeoutValue - 1000, "startFade", 1000, 0, true);
   %obj.schedule($CorpseTimeoutValue, "delete");
}


//-----------------------------------------------------------------------------

function Mecha::onLeaveMissionArea(%this, %obj)
{
   // Inform the client
   %obj.client.onLeaveMissionArea();
   messageClient(%obj.client,"","\c4**WARNING You have left the mission area!**");
           %force = MissionArea.bounce;
           %min_x = getWord(MissionArea.area, 0);
           %min_y = getWord(MissionArea.area, 1);
           %max_x = %min_x + getWord(MissionArea.area, 2);
           %max_y = %min_y + getWord(MissionArea.area, 3);
           %x = getWord(%obj.getPosition(), 0);
           %y = getWord(%obj.getPosition(), 1);
           %vx = 0;%vy = 0;%vz = 0;
           if (%x<(%min_x+10)) %vx = %force;
           if (%y<(%min_y+10)) %vy = %force;
           if (%x>(%max_x-10)) %vx = -%force;
           if (%y>(%max_y-10)) %vy = -%force;
           %vec = %vx @ " " @ %vy @ " " @ %vz;
           %obj.setVelocity(%vec);
}

function Mecha::onLeaveMissionAreaViaRoof(%this, %obj)
{
          %obj.client.onLeaveMissionArea();
          %force = MissionArea.bounce;
          %min_x = getWord(MissionArea.area, 0);
          %min_y = getWord(MissionArea.area, 1);
          %max_x = %min_x + getWord(MissionArea.area, 2);
          %max_y = %min_y + getWord(MissionArea.area, 3);
          %x = getWord(%obj.getPosition(), 0);
          %y = getWord(%obj.getPosition(), 1);
          %vx = 0; %vy = 0; %vz = %force * -1;
          if (%x<(%min_x+10)) %vx = %force;
          if (%y<(%min_y+10)) %vy = %force;
          if (%x>(%max_x-10)) %vx = -%force;
          if (%y>(%max_y-10)) %vy = -%force;
          %vec = %vx @ " " @ %vy @ " " @ %vz;
          %obj.setVelocity(%vec);
          }

function Mecha::onEnterMissionArea(%this, %obj)
{
   // Inform the client
   %obj.client.onEnterMissionArea();
   messageClient(%obj.client,"","\c4**Welcome back to the mission area!**");
}

//-----------------------------------------------------------------------------


function Mecha::onEnterLiquid(%this, %obj, %coverage, %type)
{      //       echo("entering water type is " @ %type);
   switch(%type)
   {
      case 0:    %obj.setHeat(0);//Water
      case 1:  %obj.setHeat(0);//Ocean Water
      case 2:  %obj.setHeat(0);//River Water
      case 3:  %obj.setHeat(0);//Stagnant Water
      case 4: //Lava
         %obj.setDamageDt(%this, $DamageLava, "Lava");
      case 5: //Hot Lava
         %obj.setDamageDt(%this, $DamageHotLava, "Lava");
      case 6: //Crusty Lava
         %obj.setDamageDt(%this, $DamageCrustyLava, "Lava");
      case 7: //Quick Sand
   }
}

function Mecha::onLeaveLiquid(%this, %obj, %type)
{   // echo("exiting water type is " @ %type);
switch(%type)
   {
      case 0:    %obj.setHeat(%this.heat);//Water
      case 1: %obj.setHeat(%this.heat);//Ocean Water
      case 2: %obj.setHeat(%this.heat);//River Water
      case 3: %obj.setHeat(%this.heat);//Stagnant Water
      case 4: %obj.setHeat(%this.heat);//Lava
         %obj.setDamageDt(%this, $DamageLava, "Lava");
      case 5: //Hot Lava
         %obj.setDamageDt(%this, $DamageHotLava, "Lava");
      case 6: //Crusty Lava
         %obj.setDamageDt(%this, $DamageCrustyLava, "Lava");
      case 7: //Quick Sand
   }
   %obj.clearDamageDt();
}


//-----------------------------------------------------------------------------

function Mecha::onTrigger(%this, %obj, %triggerNum, %val)
{
   // This method is invoked when the player receives a trigger
   // move event.  The player automatically triggers slot 0 and
   // slot one off of triggers # 0 & 1.  Trigger # 2 is also used
   // as the jump key.
}

function Mecha::onDestroyed(%this,%obj,%state)
{
   // The player object sets the "disabled" state when damage exceeds
   // it's maxDamage value.  This is method is invoked by ShapeBase
   // state mangement code.
   //FXLIGHTDB
    //%fxldb = new(fxLight)()
    //{
    //  dataBlock = fxDefaultLight;
    //};
    //
    //%fxldb.attachToObject(%obj);
    //%obj.light = %fxldb;
    //MissionCleanup.add(%fxldb);
    //%fxldb.schedule(1000,"delete");
    

   // If we want to deal with the damage information that actually
   // caused this death, then we would have to move this code into
   // the script "damage" method.
   %obj.playDeathCry();
   %obj.playDeathAnimation();
   %obj.setDamageFlash(0.75);

   // Release the main weapon trigger
   %obj.setImageTrigger(0,false);

   // Schedule corpse removal.  Just keeping the place clean.
   %obj.schedule($CorpseTimeoutValue - 1000, "startFade", 1000, 0, true);
   %obj.schedule($CorpseTimeoutValue, "delete");
   //%obj.respawn();
}

function Mecha::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec)
{
  error("Jeez!! I've been damaged with" SPC %amount);
  %data.damage(%targetObject, %sourceObject, %position, %amount, %damageType);
}


