//-----------------------------------------------------------------------------
// Torque Game Engine
// Copyright (c) BraveTree Productions
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

// Timeouts for corpse deletion.
$CorpseTimeoutValue = 2 * 1000;  //was 22


//----------------------------------------------------------------------------
// TankData Datablock methods
//----------------------------------------------------------------------------

function TankData::onAdd(%this,%obj)
{
   // Vehicle timeout
   %obj.mountVehicle = false;
   %obj.MaxCounters = 36;
   // Default dynamic TankData stats
   %obj.setRechargeRate(%this.rechargeRate);
   %obj.setRepairRate(0);
   %obj.setHeat(%this.heat);
}

function TankData::onRemove(%this, %obj)
{
   if (%obj.client.player == %obj)
      %obj.client.player = 0;
}

function TankData::onNewDataBlock(%this,%obj)
{
}

//----------------------------------------------------------------------------

function TankData::onMount(%this,%obj,%vehicle,%node)
{
   if (%node == 0)  {
      %obj.setTransform("0 0 0 0 0 1 0");
      %obj.setActionThread(%vehicle.getDatablock().mountPose[%node],true,true);
      %obj.lastWeapon = %obj.getMountedImage($WeaponSlot);
      %obj.unmountImage($WeaponSlot);
      %obj.setControlObject(%vehicle);
   }
}

function TankData::onUnmount( %this, %obj, %vehicle, %node )
{
   if (%node == 0)
      %obj.mountImage(%obj.lastWeapon, $WeaponSlot);
}

function TankData::doDismount(%this, %obj, %forced)
{
   // This function is called by player.cc when the jump trigger
   // is true while mounted
   if (!%obj.isMounted())
      return;
   // Position above dismount point
   %pos    = getWords(%obj.getTransform(), 0, 2);
   %oldPos = %pos;
   %vec[0] = " 0  0  1";
   %vec[1] = " 0  0  1";
   %vec[2] = " 0  0 -1";
   %vec[3] = " 1  0  0";
   %vec[4] = "-1  0  0";
   %impulseVec  = "0 0 0";
   %vec[0] = MatrixMulVector( %obj.getTransform(), %vec[0]);

   // Make sure the point is valid
   %pos = "0 0 0";
   %numAttempts = 5;

   %success     = -1;
   for (%i = 0; %i < %numAttempts; %i++) {
      %pos = VectorAdd(%oldPos, VectorScale(%vec[%i], 3));
      if (%obj.checkDismountPoint(%oldPos, %pos)) {
         %success = %i;
         %impulseVec = %vec[%i];
         break;
      }
   }
   if (%forced && %success == -1)
      %pos = %oldPos;

         %obj.unmount();
         %obj.setControlObject(%obj);
         if(%obj.mVehicle)
         //    %obj.mVehicle.getDataBlock().playerDismounted(%obj.mVehicle, %obj);
         %obj.mountVehicle = false;
        %obj.schedule(4000, "MountVehicles", true);



   // Position above dismount point
   %obj.setTransform(%pos);
   %obj.applyImpulse(%pos, VectorScale(%impulseVec, %obj.getDataBlock().mass));
   //%obj.setPilot(false);
   %obj.vehicleTurret = "";
}

//----------------------------------------------------------------------------

function TankData::onCollision(%this,%obj,%col)
{
   if (%obj.getState() $= "Dead")
      return;

   // Try and pickup all items
   if (%col.getClassName() $= "Item")
      %obj.pickup(%col);

   // Mount vehicles
   %this = %col.getDataBlock();

   if((%this.className $= WheeledVehicleData || %this.className $= FlyingVehicleData ||%this.className $= HoverVehicleData)
      && %obj.mountVehicle && %obj.getState() $= "Move")
   {
	if(%col.mountable)
        {
// Only mount drivers for now.
            %node = 0;
            //%col.mountable=false;// Set Vehicle Unmountable
            %col.mountObject(%obj,%node);
            %obj.mVehicle = %col;
        }
        else
        {
            %obj.damage(0, VectorAdd(%obj.getPosition(),%vec), 20, "Impact");// to Run over players
        }
   }


}


//----------------------------------------------------------------------------

function TankData::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
   if (%obj.getState() $= "Dead")
      return;
   %obj.applyDamage(%damage);
   %location = "Body";

   // Deal with client callbacks here because we don't have this
   // information in the onDamage or onDisable methods
   %client = %obj.client;
   %sourceClient = %sourceObject ? %sourceObject.client : 0;

   if (%obj.getState() $= "Dead")
      %client.onDeath(%sourceObject, %sourceClient, %damageType, %location);
}

function TankData::onDamage(%this, %obj, %delta)
{
   // This method is invoked by the ShapeBase code whenever the
   // object's damage level changes.
   if (%delta > 0 && %obj.getState() !$= "Dead") {

      // Increment the flash based on the amount.
      %flash = %obj.getDamageFlash() + ((%delta / %this.maxDamage) * 2);
      if (%flash > 0.75)
         %flash = 0.75;
      %obj.setDamageFlash(%flash);

      // If the pain is excessive, let's hear about it.
      if (%delta > 10)
         %obj.playPain();
   }
}

function TankData::onDisabled(%this,%obj,%state)
{
   // The player object sets the "disabled" state when damage exceeds
   // it's maxDamage value.  This is method is invoked by ShapeBase
   // state mangement code.

   // If we want to deal with the damage information that actually
   // caused this death, then we would have to move this code into
   // the script "damage" method.
   %obj.playDeathCry();
   %obj.playDeathAnimation();
   %obj.setDamageFlash(0.75);
   //alxstopall();

   // Release the main weapon trigger
   %obj.setImageTrigger(0,false);

   // Schedule corpse removal.  Just keeping the place clean.
   %obj.schedule($CorpseTimeoutValue - 1000, "startFade", 1000, 0, true);
   %obj.schedule($CorpseTimeoutValue, "delete");
}

//-----------------------------------------------------------------------------

function TankData::onLeaveMissionArea(%this, %obj)
{
   // Inform the client
   %obj.client.onLeaveMissionArea();
   messageClient(%obj.client,"","\c4**WARNING You have left the mission area!**");
   %force = MissionArea.bounce;
   %min_x = getWord(MissionArea.area, 0);
   %min_y = getWord(MissionArea.area, 1);
   %max_x = %min_x + getWord(MissionArea.area, 2);
   %max_y = %min_y + getWord(MissionArea.area, 3);
   %x = getWord(%obj.getPosition(), 0);
   %y = getWord(%obj.getPosition(), 1);
   %vx = 0;%vy = 0;%vz = 0;
   if (%x<(%min_x+10)) %vx = %force;
   if (%y<(%min_y+10)) %vy = %force;
   if (%x>(%max_x-10)) %vx = -%force;
   if (%y>(%max_y-10)) %vy = -%force;
   %vec = %vx @ " " @ %vy @ " " @ %vz;
   %obj.setVelocity(%vec);
   messageClient(%obj.client,"","\c4**WARNING Bouncing you back to mission area!**");
}

function TankData::onLeaveMissionAreaViaRoof(%this, %obj)
{
%obj.client.onLeaveMissionArea();
%force = MissionArea.bounce;
%min_x = getWord(MissionArea.area, 0);
%min_y = getWord(MissionArea.area, 1);
%max_x = %min_x + getWord(MissionArea.area, 2);
%max_y = %min_y + getWord(MissionArea.area, 3);
%x = getWord(%obj.getPosition(), 0);
%y = getWord(%obj.getPosition(), 1);
%vx = 0; %vy = 0; %vz = %force * -1;
if (%x<(%min_x+10)) %vx = %force;
if (%y<(%min_y+10)) %vy = %force;
if (%x>(%max_x-10)) %vx = -%force;
if (%y>(%max_y-10)) %vy = -%force;
%vec = %vx @ " " @ %vy @ " " @ %vz;
%obj.setVelocity(%vec);
}

function TankData::onEnterMissionArea(%this, %obj)
{
   // Inform the client
   %obj.client.onEnterMissionArea();
   messageClient(%obj.client,"","\c4**Welcome back to the mission area!**");
}

//-----------------------------------------------------------------------------


function TankData::onEnterLiquid(%this, %obj, %coverage, %type)
{
   switch(%type)
   {
      case 0:    %obj.setHeat(0);//Water
      case 1:  %obj.setHeat(0);//Ocean Water
      case 2:  %obj.setHeat(0);//River Water
      case 3:  %obj.setHeat(0);//Stagnant Water
      case 4: //Lava
         %obj.setDamageDt(%this, $DamageLava, "Lava");
      case 5: //Hot Lava
         %obj.setDamageDt(%this, $DamageHotLava, "Lava");
      case 6: //Crusty Lava
         %obj.setDamageDt(%this, $DamageCrustyLava, "Lava");
      case 7: //Quick Sand
   }
}

function TankData::onLeaveLiquid(%this, %obj, %type)
{
switch(%type)
   {
      case 0:    %obj.setHeat(%this.heat);//Water
      case 1: %obj.setHeat(%this.heat);//Ocean Water
      case 2: %obj.setHeat(%this.heat);//River Water
      case 3: %obj.setHeat(%this.heat);//Stagnant Water
      case 4: %obj.setHeat(%this.heat);//Lava
         %obj.setDamageDt(%this, $DamageLava, "Lava");
      case 5: //Hot Lava
         %obj.setDamageDt(%this, $DamageHotLava, "Lava");
      case 6: //Crusty Lava
         %obj.setDamageDt(%this, $DamageCrustyLava, "Lava");
      case 7: //Quick Sand
   }
   %obj.clearDamageDt();
}


//-----------------------------------------------------------------------------

function TankData::onTrigger(%this, %obj, %triggerNum, %val)
{
   // This method is invoked when the player receives a trigger
   // move event.  The player automatically triggers slot 0 and
   // slot one off of triggers # 0 & 1.  Trigger # 2 is also used
   // as the jump key.
}


