//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

// Global movement speed that affects all cameras.  This should be moved
// into the camera datablock.
$Camera::movementSpeed = 40;

//-----------------------------------------------------------------------------
// Define a datablock class to use for our observer camera
//-----------------------------------------------------------------------------

datablock CameraData(Observer)
{
   mode = "Observer";
};


//-----------------------------------------------------------------------------

function Observer::onTrigger(%this,%obj,%trigger,%state)
{
   // state = 0 means that a trigger key was released
   if (%state == 0)
      return;

   // Default player triggers: 0=fire 1=altFire 2=jump
   %client = %obj.getControllingClient();
   switch$ (%obj.mode)
   {
      case "Observer":
         // Do something interesting.

      case "Corpse":
         // Viewing dead corpse, so we probably want to respawn.
         //%client.Canvas.setContent("Team");
         //%client.spawnPlayer();

         // Set the camera back into observer mode, since in
         // debug mode we like to switch to it.
         //%this.setMode(%obj,"Observer");
   }
}

function Observer::setMode(%this,%obj,%mode,%arg1,%arg2,%arg3)
{
   echo("mode is "@ %mode);
   switch$ (%mode)
   {
      case "Observer":
         // Let the player fly around
         %obj.setFlyMode();

      case "Corpse":
         //deathCam(%arg1);
        //deathCamToSpawn(%arg1);
        %transform = %arg1.player.getPosition();
        %obj.setOrbitMode(%arg1.player, %transform, 3.5, 40.5, 4.5);
   }
}
//-----------------------------------------------------------------------------
// Path Camera
//-----------------------------------------------------------------------------

datablock PathCameraData(SateliteCam)
{
   mode = "";
};

function PathCamera::onNode(%this,%camera,%node)
{
   //echo("onNode " SPC %node.position SPC %camear.position);
   if (%node == %camera.loopNode) {
      %camera.pushPath(%camera.path);
      %camera.loopNode += %camera.path.getCount();
   }
}

function PathCamera::followPath(%this,%path)
{   //          echo("followpath ");
   %this.path = %path;
   if (!%this.speed)
      %this.speed = 1;
   if (%path.isLooping)
      %this.loopNode = %path.getCount() - 2;
   else
      %this.loopNode = -1;

   %this.pushPath(%path);
   %this.popFront();
}

function PathCamera::pushPath(%this,%path)
{
   for (%i = 0; %i < %path.getCount(); %i++)
      %this.pushNode(%path.getObject(%i));
}

function PathCamera::pushNode(%this,%node)
{
   if (!(%speed = %node.speed))
      %speed = %node.speed;
   if ((%type = %node.type) $= "")
      %type = "Normal";
   if ((%smoothing = %node.smoothing) $= "")
      %smoothing = "Linear";
   %this.pushBack(%node.getTransform(),%speed,%type,%smoothing);
}



//-----------------------------------------------------------------------------
// Camera methods
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function Camera::onAdd(%this,%obj)
{
   // Default start mode
   %this.setMode(%this.mode);
}

function Camera::setMode(%this,%mode,%arg1,%arg2,%arg3)
{
   // Punt this one over to our datablock
   %this.getDatablock().setMode(%this,%mode,%arg1,%arg2,%arg3);
}

function DeathCam(%client) {

         // Lock the camera down in orbit around the corpse,
         // which should be arg1
         %transform = %client.player.getPosition();
         %rotation = getWord(%client.player.getEyeVector(),0) SPC getWord(%client.player.getEyeVector(),1) SPC "-0.586309 53.858";
         //%rotation = "0 0 -1 0";

         //take transform and result and make a path with two nodes beging/end
         //put camera on path, setTarget for camera to transform  pull away

         //echo(%client);
         //path in range file will need to creat and destroy one on the fly
          %client.PathCamera = new PathCamera() {
                             dataBlock = SateliteCam;
                             position = %client.player.getPosition();
                             rotation = %client.player.getEyeTransform();
                            };
         %client.PathCamera.scopeToClient(%client);
         %client.PathCamera.setTransform(%client.player.getEyeTransform());
         MissionCleanup.add( %client.PathCamera );
         %client.setControlObject(%client.PathCamera);
         //%client.PathCamera.setPosition(%obj.getPosition());

         %result = %transform;
         %result = VectorSub(%result,"0 0 10");

         %client.Path =  new Path();
         %client.Path.isLooping = "0";




         //echo(%rotation);
         for(%i = 0; %i < 20; %i++){

            %result = vectorAdd(%result,"0 0 10");
            %client.Path.add( new Marker() {
               position = %result;
               rotation = %rotation;
               seqNum = %i - 1;
               type = "normal";
               msToNext = 1000;
                smoothingType = "Linear";
                     smoothing = "Spline";
               speed = 30;
               });

               %camEyeTrans = getWord(%client.Path.getObject(%i).getTransform(),0) SPC getWord(%client.Path.getObject(%i).getTransform(),1) SPC getWord(%client.Path.getObject(%i).getTransform(),2) SPC "0.988123 -0.0989949 0.117531 1.41179";
            %client.Path.getObject(%i).setTransform(%camEyeTrans);
             %client.Path.getObject(%i).position = %result;
             }
         %client.PathCamera.followPath(%client.Path);
         %client.PathCamera.popFront();
}

function DeathCamToSpawn(%client) {

        %transform = %client.player.getPosition();
         %rotation = getWord(%client.player.getEyeVector(),0) SPC getWord(%client.player.getEyeVector(),1) SPC "-0.586309 53.858";

          %client.PathCamera = new PathCamera() {
                             dataBlock = SateliteCam;
                             position = %client.player.getPosition();
                             rotation = %client.player.getEyeTransform();
                            };
         %client.PathCamera.scopeToClient(%client);
         %client.PathCamera.setTransform(%client.player.getEyeTransform());
         MissionCleanup.add( %client.PathCamera );
         %client.setControlObject(%client.PathCamera);
         %result = %transform;
         %result = VectorSub(%result,"0 0 10");
          %client.Path =  new Path();
         %client.Path.isLooping = "0";
          %sPos =  pickSpawnPoint(%client);
          %newCoordes = %transform;
          %spawnPosition =  getWord(%sPos,0) SPC  getWord(%sPos,1) SPC getWord(%sPos,2);
         for(%i = 0; %i <= 20; %i++){
            //%newX =  getWord(%newCoordes,1) * getWord(%newCoordes,1);
            //%newY =  mSqrt(getWord(%newCoordes,0));
            %b = (%i * 10);
            %newX = getWord(%newCoordes,0) *%b;
            %newY = getWord(%newCoordes,1) *%b;
            %newZ = getWord(%newCoordes,2) *%b;
            %newCoordes =  %newX  SPC  %newY  SPC  %newZ;
            switch$ (%i)
            {
             case "0":
              %result = %transform;

             case "20":
             %result = %spawnPosition;

             default :
              %result = %newCoordes;
             }
            echo(%result);
            %client.Path.add( new Marker() {
               position = %result;
               rotation = %rotation;
               seqNum = %i - 1;
               type = "normal";
               msToNext = 10000;
               smoothingType = "Linear";
                smoothing = "Spline";
               speed = 30;
               });
               // y = squRt(x);
               // x = y * y;
            //%camEyeTrans = %client.player.getEyeTransform();

             %camEyeTrans = getWord(%client.Path.getObject(%i).getTransform(),0) SPC getWord(%client.Path.getObject(%i).getTransform(),1) SPC getWord(%client.Path.getObject(%i).getTransform(),2) SPC "0.988123 -0.0989949 0.117531 1.41179";
             %client.Path.getObject(%i).setTransform(%camEyeTrans);
             %client.Path.getObject(%i).position = %result;
             }
         %client.PathCamera.followPath(%client.Path);
         %client.PathCamera.popFront();
}
