//-----------------------------------------------------------------------------
// Torque Game Engine
// Copyright (c) 2001 GarageGames.Com
// Portions Copyright (c) 2001 by Sierra Online, Inc.
//-----------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Customization Server Commands
// functions
//       serverCmdSetCustomization(DatablockNumber,
//            PrimaryWeaponNumber,SecondaryWeaponNumber);
//       serverCmdGetWeapons(Datablock Name);
//
//------------------------------------------------------------------------------

// Client is sending information
function serverCmdSetCustomization(%client,%BaseNum,%PNum,%SNum)
{
  //Set client variables
  %client.primaryNum = %PNum;
  %client.secondaryNum = %SNum;
  %client.selectedMAVnum = mFloor(%BaseNum);
}

// This is a client request for information
function serverCmdGetWeapons(%client,%MAV)
{
  // Get Primary/Secondary Datablock names
  %Primary = (%MAV).primaryName;
  %Secondary =  (%MAV).secondaryName;
  %PTotal = (%MAV).primaryTotal;
  %STotal = (%MAV).secondaryTotal;

  // Send Data to Client (Fills in Weapon Dropdowns for Customization UI)
  MessageClient(%client,'MsgConfigMavClear','',"bah");
  for (%i = 1; %i <= %PTotal; %i++) {
     MessageClient(%client,'MsgConfigMavWeapon','',%i,"Primary",(%Primary @ %i).pickUpName,(%Primary @ %i).amount,(%Primary @ %i).refire,(%Primary @ %i).damage,(%Primary @ %i).range);
  }
  for (%i = 1; %i <= %STotal; %i++) {
     MessageClient(%client,'MsgConfigMavWeapon','',%i,"Second",(%Secondary @ %i).pickUpName,(%Secondary @ %i).amount,(%Secondary @ %i).refire,(%Secondary @ %i).damage,(%Secondary @ %i).range);
  }
  MessageClient(%client,'MsgConfigMavEnd','',(%MAV).primaryNum,(%MAV).secondaryNum);
}

//------------------------------------------------------------------------------
// Team Specific Hud Code
//------------------------------------------------------------------------------
// serverCmdGetHud(Client);
//------------------------------------------------------------------------------

// This is a client request for Information
function serverCmdGetHud(%client)
{
  //Get Hud Name from Team and send it to client
  commandToClient(%client,'SetHud',%client.team.hud);
}

//------------------------------------------------------------------------------
// Singleplayer Commands
//   Used on end screen to replay or go to next mission
//------------------------------------------------------------------------------
// functions
//    serverCmdCycleSPGame();
//    serverCmdReplayGame();
//------------------------------------------------------------------------------
function serverCmdCycleSPGame(%client)
{ // echo("singlePlayerLevel is " @ $singlePlayerLevel);
   $Server::Replay = false;
   %client.ready = true;
   onCyclePauseEnd();
}

function serverCmdReplayGame(%client)
{
  // echo("mssion is " @ $Server::MissionFile);
  $Server::Replay = true;
  %client.ready = true;
  onCyclePauseEnd();
  // echo("mssion is NOW" @ $Server::MissionFile);
}
//------------------------------------------------------------------------------
//  Staging Area Commands
//------------------------------------------------------------------------------
function serverCmdLeftGame(%client)
{
   if(%client.team){
      %client.team.numPlayers--;
   }
   if (isObject(%client.player))
      %client.player.delete();
   %client.setControlObject(0);
   cancel(%client.schedule);
}

function serverCmdAbortMission(%client)
{
  if ($missionRunning) {
     if (%client.isAdmin) {
       MessageAll('MsgSetLaunch', '', false);
       cycleGame();
     }
  }
}

function serverCmdSetGame(%client,%mission)
{
  if (!$missionRunning) {
     if (%client.isAdmin) {
       MessageAll('MsgSetLaunch', '', true);
       loadMission(%mission, true);
     }
  }
}

function serverCmdSetReady(%client,%ready)
{
   %client.ready = %ready;
}

function serverCmdLaunchGame(%client)
{
  if ($missionRunning)
     %client.onLaunch(%client);
}

function serverCmdPlayerUpdate(%client,%type,%value)
{
  if (%type == 0)
    %client.selectedMAV = %value;
  if (%type == 1)
    %client.selectedTEAM = %value;
  messageAll('MsgHSPlayerUpdate','',%client, %client.name, %client.selectedMAV, %client.selectedTEAM);
}
function serverCmdRelayHostData(%client)
{
   if ($missionRunning)
   {
      messageClient(%client, 'MsgHSUpdate','',$GUI::MissionName,$GUI::MissionTypeName,$GUI::DurationTime,
         $GUI::EndScore, $GUI::RespawnCount, $GUI::Practice, $GUI::Reporting, $GUI::Attacker, $GUI::Defender);
   }
   else
   {
      messageAll('MsgHSUpdate','',$GUI::MissionName,$GUI::MissionTypeName,$GUI::DurationTime,
         $GUI::EndScore, $GUI::RespawnCount, $GUI::Practice, $GUI::Reporting, $GUI::Attacker, $GUI::Defender);
   }
}

//-----------------------------------------------------------------------------
//------ Camera Commands ------------------------------------------------------
function serverCmdToggleCamera(%client)
{
   if ($Server::TestCheats || $Server::ServerType $= "SinglePlayer")
   {
      %control = %client.getControlObject();
      if (%control == %client.player)
      {
         %control = %client.camera;
         %control.mode = toggleCameraFly;
      }
      else if (%control == %client.PathCamera)
      {
         %control = %client.camera;
         %control.mode = toggleCameraFly;
      }
      else
      {
         %control = %client.player;
         %control.mode = observerFly;
      }
      %client.setControlObject(%control);
   }
}

function serverCmdDropPlayerAtCamera(%client)
{
   if ($Server::TestCheats)
   {
      if (!%client.player.isMounted())
         %client.player.setTransform(%client.camera.getTransform());
      %client.player.setVelocity("0 0 0");
      %client.setControlObject(%client.player);
   }
}

function serverCmdDropCameraAtPlayer(%client)
{
   if ($Server::TestCheats)
   {
      %client.camera.setTransform(%client.player.getEyeTransform());
      %client.camera.setVelocity("0 0 0");
      %client.setControlObject(%client.camera);
   }
}


//-----------------------------------------------------------------------------

function serverCmdSuicide(%client)
{
   if (isObject(%client.player))
      %client.player.kill("Suicide");
}   

function serverCmdPlayCel(%client,%anim)
{
   if (isObject(%client.player))
      %client.player.playCelAnimation(%anim);
}

function serverCmdPlayDeath(%client)
{
   if (isObject(%client.player))
      %client.player.playDeathAnimation();
}

function serverCmdSelectObject(%client, %mouseVec, %cameraPoint)
{
   //Determine how far should the picking ray extend into the world?
   %selectRange = 200;
   // scale mouseVec to the range the player is able to select with mouse
   %mouseScaled = VectorScale(%mouseVec, %selectRange);
   // cameraPoint = the world position of the camera
   // rangeEnd = camera point + length of selectable range
   %rangeEnd = VectorAdd(%cameraPoint, %mouseScaled);

   // Search for anything that is selectable  below are some examples
   %searchMasks = $TypeMasks::PlayerObjectType | $TypeMasks::CorpseObjectType |
      				$TypeMasks::ItemObjectType | $TypeMasks::TriggerObjectType;

   // Search for objects within the range that fit the masks above
   // If we are in first person mode, we make sure player is not selectable by setting fourth parameter (exempt
   // from collisions) when calling ContainerRayCast
   %player = %client.player;
   if ($firstPerson)
   {
	  %scanTarg = ContainerRayCast (%cameraPoint, %rangeEnd, %searchMasks, %player);
   }
   else //3rd person - player is selectable in this case
   {
	  %scanTarg = ContainerRayCast (%cameraPoint, %rangeEnd, %searchMasks);
   }

   // a target in range was found so select it
   if (%scanTarg)
   {
      %targetObject = firstWord(%scanTarg);

      %client.setSelectedObj(%targetObject);
   }
}

function serverCmdjoinTeam(%client, %teamid){
  %client.joinTeam(%teamid);
}

//------------------------------------------------------------------------------
// BOT COMMANDS
//------------------------------------------------------------------------------
$botCounter = 0;

function serverCmdAddBot(%client) {

  MissionCleanup.add($bots);
  %npcName = "Bot" @ $botCounter;
  //NEW CODE
  //creat temp to hold bot to see if it exisits
  %newBot = AIPlayer::spawnSpiderMinerPlayer(%npcName,%client);
  if( %newBot !$= "noBOT" )
  { //it DOES add it to the ARRAY
     $bots[$botCounter] = %newBot ;
     MissionCleanup.add($bots[$botCounter]);
     $bots[$botCounter].patrolwaypoints($bots[$botCounter].wayPointArea);
     $botCounter++;
  }
  else //NO BOT WAS MADE, IE NO AREA FOR THE BOT TO BE MADE
  {
     error("no bot made") ;
  }
}

function serverCmdAddBadBot(%client)
{
  MissionCleanup.add($bots);
  %npcName = "Bot" @ $botCounter;
  //join at same place as player who called the bot
  //hack because spawn shpere is determined by the player not the
  //AI's team this is due to the team being added after player is made
  if (%client.team.teamID == 2)
     $bots[$botCounter].team = $Team[1];
  if (%client.team.teamID == 1)
     $bots[$botCounter].team = $Team[2];
  $bots[$botCounter] = AIPlayer::spawnEnemyPlayer(%npcName,%client);
  MissionCleanup.add($bots[$botCounter]);

  //joins opposite team as the player who called him
  //hack should be added to the exisiting joinTeam function

  $bots[$botCounter].processPlayers();
  $botCounter++;
  echo("Bot #" @ $botCounter @ " added to team " @ $bots[$botCounter].team.teamID);
  echo("ID NUMBER IS:" SPC $bots[$botCounter]);
}

//this may not work :)
function serverCmdpositionEcho(%client)
{
// position echo
%pos_x    = getWords(%client.player.getTransform(), 0, 0);
%pos_y    = getWords(%client.player.getTransform(), 1, 1);
%pos_z    = getWords(%client.player.getTransform(), 2, 2);
messageClient(%client.player.client,"","X = " @ %pos_x @ " Y = " @ %pos_y @ " Z = " @ %pos_z);

//speed echo
%speed = getWords(getControlObjectSpeed(), 0, 0);
messageClient(%client.player.client,"","speed = " @ %speed);

}


