//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2003 ToRK / 3D-Diggers
// Portions Copyright (c) 2001 by GarageGames.Com, Sierra Online, Inc.
//-----------------------------------------------------------------------------

//*************************************
// Destroyable barrel by 3D-Diggers/ToRK
//*************************************

$barrelExplosionSound = "~/data/sound/shake.wav";
$barrelExplosionSound2 = "~/data/sound/explode.wav";
$barrelDamageSound = "~/data/sound/hit.wav";


//*************************************++
// Damage sounds
//*************************************++

new AudioDescription(BlueBarrelDestroyDmgDesc)
{
   volume   = 1.2;
   isLooping= false;
   is3D     = true;
   type     = $SimAudioType;
};
function playBlueBarrelDamageSound(%soundfile)
{
	if(%soundfile $= "")
		%soundfile = $barrelDamageSound;
	if (!alxIsPlaying($BlueBarrelDestroyDmgHandle))
	{
		$BlueBarrelDestroyDmgHandle = alxCreateSource("BlueBarrelDestroyDmgDesc", expandFilename(%soundfile));
		alxPlay($BlueBarrelDestroyDmgHandle);
	}
}

//**************************************
// Explosion
//**************************************
datablock AudioProfile(BlueBarrelDestroyExplosionSound)
{
   filename = $barrelExplosionSound;
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(BlueBarrelDestroyExplosionSound2)
{
   filename = $barrelExplosionSound2;
   description = AudioDefault3d;
   preload = true;
};

datablock ParticleData(BlueBarrelDestroyExplosionParticle)
{
   textureName          = "~/data/shapes/particles/smoke";
   dragCoefficient      = 2;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 150;

   colors[0]     = "0.56 0.56 0.56 1.0";
   colors[1]     = "0.26 0.36 0.36 0.0";

   sizes[0]      = 0.5;
   sizes[1]      = 1.0;
};

datablock ParticleEmitterData(BlueBarrelDestroyExplosionEmitter)
{
   ejectionPeriodMS = 7;
   periodVarianceMS = 0;
   ejectionVelocity = 2;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "BlueBarrelDestroyExplosionParticle";
};

datablock ParticleData(BlueBarrelDestroyExplosionSmoke)
{
   textureName          = "~/data/shapes/particles/smoke";
   dragCoeffiecient     = 100.0;
   gravityCoefficient   = 0;
   inheritedVelFactor   = 0.25;
   constantAcceleration = -0.80;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 300;
   useInvAlpha =  false;
   spinRandomMin = -80.0;
   spinRandomMax =  80.0;

   colors[0]     = "0.56 0.36 0.26 1.0";
   colors[1]     = "0.2 0.2 0.2 1.0";
   colors[2]     = "0.0 0.0 0.0 0.0";

   sizes[0]      = 1.0;
   sizes[1]      = 1.5;
   sizes[2]      = 2.0;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;

};

datablock ParticleEmitterData(BlueBarrelDestroyExplosionSmokeEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 0;
   ejectionVelocity = 4;
   velocityVariance = 0.5;
   thetaMin         = 0.0;
   thetaMax         = 180.0;
   lifetimeMS       = 220;
   particles = "BlueBarrelDestroyExplosionSmoke";
};

datablock ParticleData(BlueBarrelDestroyExplosionSparks)
{
   textureName          = "~/data/shapes/particles/spark";
   dragCoefficient      = 1;
   gravityCoefficient   = 0.0;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 600;
   lifetimeVarianceMS   = 350;

   colors[0]     = "0.30 0.30 0.30 1.0";
   colors[1]     = "0.30 0.30 0.30 1.0";
   colors[2]     = "0.10 0.10 0.10 0.0";

   sizes[0]      = 0.5;
   sizes[1]      = 0.25;
   sizes[2]      = 0.25;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(BlueBarrelDestroyExplosionSparkEmitter)
{
   ejectionPeriodMS = 3;
   periodVarianceMS = 0;
   ejectionVelocity = 13;
   velocityVariance = 6.75;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 180;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = true;
   lifetimeMS       = 200;
   particles = "BlueBarrelDestroyExplosionSparks";
};

datablock ExplosionData(BlueBarrelDestroySubExplosion1)
{
   offset = 1.0;
   emitter[0] = BlueBarrelDestroyExplosionSmokeEmitter;
   emitter[1] = BlueBarrelDestroyExplosionSparkEmitter;
};

datablock ExplosionData(BlueBarrelDestroySubExplosion2)
{
   offset = 1.0;
   emitter[0] = BlueBarrelDestroyExplosionSmokeEmitter;
   emitter[1] = BlueBarrelDestroyExplosionSparkEmitter;
};

datablock ExplosionData(BlueBarrelDestroyExplosion)
{
   soundProfile   = BlueBarrelDestroyExplosionSound;
   lifeTimeMS = 1800;

   // Volume particles
   particleEmitter = BlueBarrelDestroyExplosionEmitter;
   particleDensity = 110;
   particleRadius = 1.0;

   // Point emission
   emitter[0] = BlueBarrelDestroyExplosionSmokeEmitter;
   emitter[1] = BlueBarrelDestroyExplosionSparkEmitter;

   // Sub explosion objects
   subExplosion[0] = BlueBarrelDestroySubExplosion1;
   subExplosion[1] = BlueBarrelDestroySubExplosion2;

   // Camera Shaking
   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "1.0 1.0 1.0";
   camShakeDuration = 1.5;
   camShakeRadius = 50.0;

   // Dynamic light
   lightStartRadius = 6;
   lightEndRadius = 3;
   lightStartColor = "0.5 0.5 0";
   lightEndColor = "0 0 0";
};

datablock ExplosionData(BlueBarrelDestroyExplosion2)
{
   soundProfile   = BlueBarrelDestroyExplosionSound2;
   lifeTimeMS = 500;

   // Volume particles
   particleEmitter = BlueBarrelDestroyExplosionEmitter;
   particleDensity = 50;
   particleRadius = 0.5;

   // Point emission
   emitter[0] = BlueBarrelDestroyExplosionSmokeEmitter;
   emitter[1] = BlueBarrelDestroyExplosionSparkEmitter;

   // Sub explosion objects
   subExplosion[0] = BlueBarrelDestroySubExplosion1;
   subExplosion[1] = BlueBarrelDestroySubExplosion2;

   // Camera Shaking
   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "1.0 1.0 1.0";
   camShakeDuration = 1.2;
   camShakeRadius = 30.0;

   // Dynamic light
   lightStartRadius = 4;
   lightEndRadius = 2;
   lightStartColor = "0.5 0.5 0";
   lightEndColor = "0 0 0";
};


//*****************************************
// debris and barrel shapes
//*****************************************

datablock DebrisData(BlueBarrelDestroyShapeDebris)
{
	explodeOnMaxBounce = false;

	elasticity = 0.1;
	friction = 0.6;

	lifetime = 3.0;
	lifetimeVariance = 1.0;

	minSpinSpeed = 40;
	maxSpinSpeed = 180;

	numBounces = 2;
	bounceVariance = 1.5;

	staticOnMaxBounce = true;
	gravModifier = 1.0;

	useRadiusMass = true;
	baseRadius = 1;

	velocity = 13.0;
	velocityVariance = 3.0;
};

datablock StaticShapeData(BrokenBlueBarrelDestroyShape1)
{
   //category = "Barrels";
   destroyable = "1";
   shapeFile = "~/data/shapes/objects/barrel02_wrack.dts";
   mass = 9.8;
   maxDamage = 100;
   destroyedLevel = 100;
   explosion = BlueBarrelDestroyExplosion2;
   numDamageMeshes = "5";
   numDamageFragments = "35";
   explosionDamage = 4.5; 
   explosionRadius = 11.0; 

   debrisShapeName = "~/data/shapes/objects/barrel02_debris.dts";
   debris = BlueBarrelDestroyShapeDebris;
   renderWhenDestroyed = true;
};

datablock StaticShapeData(BlueBarrelDestroyShape)
{
   category = "FX1";
   destroyable = "1";
   shapeFile = "~/data/shapes/objects/barrel02.dts";
   mass = 12.8;
   maxDamage = 80;
   destroyedLevel = 80;
   explosion = BlueBarrelDestroyExplosion;
   numDamageMeshes = "6";
   numDamageFragments = "40";
   explosionDamage = 8.5; 
   explosionRadius = 12.0; 

   debrisShapeName = "~/data/shapes/objects/barrel02_debris.dts";
   debris = BlueBarrelDestroyShapeDebris;
   renderWhenDestroyed = true;
};

function BlueBarrelDestroyShape::onAdd(%this,%obj)
{
   %obj.destroyable = "1";
}

function StaticShapeData::create(%block)
{
   %obj = new StaticShape()
   {
      dataBlock = %block;
   };
   return(%obj);
}

//*********************************
// Stage 1
//*********************************

function BlueBarrelDestroyShape::onCollision(%this,%obj,%collider)
{
}

function BlueBarrelDestroyShape::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
   if(%obj.destroyable || %obj.getDatablock().destroyable)
   {
	   %currDamage = %obj.getDamageLevel();
	   %obj.applyDamage(%damage);
   }
}

function BlueBarrelDestroyShape::onDamage(%this, %obj)
{
   %damage = %obj.getDamageLevel();
   if (%damage >= %this.destroyedLevel)
   {
      if(%obj.getDamageState() !$= "Destroyed")
      {
         if(%obj.respawnTime !$= "")
            %obj.marker.schedule = %obj.marker.data.schedule(%obj.respawnTime, "respawn", %obj.marker); 
         %obj.setDamageState(Destroyed);
      }
   }
   else
   {
      playBlueBarrelDamageSound();
      if(%obj.getDamageState() !$= "Enabled")
         %obj.setDamageState(Enabled);
   }
}

function BlueBarrelDestroyShape::onDestroyed(%data, %obj, %prevState)
{
   radiusBlueBarrelDestroyExplosion(%data, %obj);
   %newPosition = %obj.position;
   %finalPos = %obj.getTransform();
   %newScale = %obj.scale;
   %brokenshape = new StaticShape() {
      position = %newPosition;
      rotation = "1 0 0 0";
      dataBlock = "BrokenBlueBarrelDestroyShape1";
      scale = "1 1 1";
   };
   %brokenshape.setTransform(%finalPos);
   // delete object
   %obj.schedule(100, "delete");
}

function BlueBarrelDestroyShape::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec)
{
}

function BlueBarrelDestroyShape::onCollision(%this,%obj,%collider)
{
}

//***********************************+
// functions
//***********************************+

function radiusBlueBarrelDestroyExplosion(%data, %vehicle)
{
  // this is a modified version of RadiusExplosion() from projectiles.cs
  %position = %vehicle.getPosition();
   // damage players
   InitContainerRadiusSearch(%position, %data.explosionRadius, $TypeMasks::PlayerObjectType);

   %numTargets = 0;
   while ((%targetObject = containerSearchNext()) != 0)
   {
		if(%targetObject == %vehicle)
			continue;

      %dist = containerSearchCurrRadiusDist();

      if (%dist > %data.explosionRadius)
         continue;

      if (%targetObject.isMounted())
      {
         %mount = %targetObject.getObjectMount();
			if(%mount == %vehicle)
				continue;

         %found = -1;
         for (%i = 0; %i < %mount.getDataBlock().numMountPoints; %i++)
         {
            if (%mount.getMountNodeObject(%i) == %targetObject)
            {
               %found = %i;
               break;
            }
         }
      
         if (%found != -1)
         {
            if (%mount.getDataBlock().isProtectedMountPoint[%found] && (%mount != %vehicle))
               continue;
         }
      }

      %targets[%numTargets]     = %targetObject;
      %targetDists[%numTargets] = %dist;
      %numTargets++;
   }

   for (%i = 0; %i < %numTargets; %i++)
   {
      %targetObject = %targets[%i];
      echo("Found target to damage:" SPC %targetObject);
      %dist = %targetDists[%i];

      %coverage = calcExplosionCoverage(%position, %targetObject,
                                        ($TypeMasks::InteriorObjectType |
                                         $TypeMasks::TerrainObjectType |
                                         $TypeMasks::ForceFieldObjectType));
      if (%coverage == 0)
         continue;

      %amount = (1.0 - (%dist / %data.explosionRadius)) * %coverage * %data.explosionDamage;
      %targetData = %targetObject.getDataBlock();

      %momVec = "0 0 1";
      
      if(%amount > 0)
      {
         echo("About to damage object with amount:" SPC %amount);
         // Every shapebase that should be damaged needs to provide this function!
         %targetData.damageObject(%targetObject, %sourceObject, %position, %amount, $DamageType::Explosion, %momVec);
      }
   }
}



