//*****************************************+
// PYLON object as ITEM
//*****************************************+
$deletePylonPieces = false;
$deletePylonTime = 20000;
$pylonExplosionSound = "~/data/sound/ex-smal1.wav";
$pylonDamageSound = "~/data/sound/hit.wav";


//*************************************
// Damage sounds
//*************************************

new AudioDescription(PylonDmgDesc)
{
   volume   = 1.2;
   isLooping= false;
   is3D     = true;
   type     = $SimAudioType;
};
function playPylonDamageSound(%soundfile)
{
	if(%soundfile $= "")
		%soundfile = $pylonDamageSound;
	if (!alxIsPlaying($PylonDmgHandle))
	{
		$PylonDmgHandle = alxCreateSource("PylonDmgDesc", expandFilename(%soundfile));
		alxPlay($PylonDmgHandle);
	}
}

//**************************************
// Explosion
//**************************************
datablock AudioProfile(PylonExplosionSound)
{
   filename = $pylonExplosionSound;
   description = AudioDefault3d;
   preload = true;
};

datablock ParticleData(PylonExplosionParticle)
{
   textureName          = "~/data/shapes/particles/smoke";
   dragCoefficient      = 2;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 150;

   colors[0]     = "0.56 0.56 0.56 1.0";
   colors[1]     = "0.26 0.36 0.36 0.0";

   sizes[0]      = 0.5;
   sizes[1]      = 1.0;
};

datablock ParticleEmitterData(PylonExplosionEmitter)
{
   ejectionPeriodMS = 7;
   periodVarianceMS = 0;
   ejectionVelocity = 2;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "PylonExplosionParticle";
};

datablock ParticleData(PylonExplosionSmoke)
{
   textureName          = "~/data/shapes/particles/smoke";
   dragCoeffiecient     = 100.0;
   gravityCoefficient   = 0;
   inheritedVelFactor   = 0.25;
   constantAcceleration = -0.80;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 300;
   useInvAlpha =  false;
   spinRandomMin = -80.0;
   spinRandomMax =  80.0;

   colors[0]     = "0.56 0.36 0.26 1.0";
   colors[1]     = "0.2 0.2 0.2 1.0";
   colors[2]     = "0.0 0.0 0.0 0.0";

   sizes[0]      = 1.0;
   sizes[1]      = 1.5;
   sizes[2]      = 2.0;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;

};

datablock ParticleEmitterData(PylonExplosionSmokeEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 0;
   ejectionVelocity = 4;
   velocityVariance = 0.5;
   thetaMin         = 0.0;
   thetaMax         = 180.0;
   lifetimeMS       = 220;
   particles = "PylonExplosionSmoke";
};

datablock ParticleData(PylonExplosionSparks)
{
   textureName          = "~/data/shapes/particles/spark";
   dragCoefficient      = 1;
   gravityCoefficient   = 0.0;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 600;
   lifetimeVarianceMS   = 350;

   colors[0]     = "0.30 0.30 0.30 1.0";
   colors[1]     = "0.30 0.30 0.30 1.0";
   colors[2]     = "0.10 0.10 0.10 0.0";

   sizes[0]      = 0.5;
   sizes[1]      = 0.25;
   sizes[2]      = 0.25;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(PylonExplosionSparkEmitter)
{
   ejectionPeriodMS = 3;
   periodVarianceMS = 0;
   ejectionVelocity = 13;
   velocityVariance = 6.75;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 180;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = true;
   lifetimeMS       = 200;
   particles = "PylonExplosionSparks";
};

datablock ExplosionData(PylonSubExplosion1)
{
   offset = 1.0;
   emitter[0] = PylonExplosionSmokeEmitter;
   emitter[1] = PylonExplosionSparkEmitter;
};

datablock ExplosionData(PylonSubExplosion2)
{
   offset = 1.0;
   emitter[0] = PylonExplosionSmokeEmitter;
   emitter[1] = PylonExplosionSparkEmitter;
};

datablock ExplosionData(PylonExplosion)
{
   soundProfile   = PylonExplosionSound;
   lifeTimeMS = 1800;

   // Volume particles
   particleEmitter = PylonExplosionEmitter;
   particleDensity = 110;
   particleRadius = 1.0;

   // Point emission
   emitter[0] = PylonExplosionSmokeEmitter;
   emitter[1] = PylonExplosionSparkEmitter;

   // Sub explosion objects
   subExplosion[0] = PylonSubExplosion1;
   subExplosion[1] = PylonSubExplosion2;

   // Camera Shaking
   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "1.0 1.0 1.0";
   camShakeDuration = 1.5;
   camShakeRadius = 50.0;

   // Dynamic light
   lightStartRadius = 6;
   lightEndRadius = 3;
   lightStartColor = "0.5 0.5 0";
   lightEndColor = "0 0 0";
};

//*****************************************+
// the pylon shape
//*****************************************+
datablock ShapeBaseImageData(PylonItemImage)
{
   shapeFile = "~/data/shapes/objects/pylon.dts";
   item = PylonItem;
   mountPoint = 0;
   offset = "0 0.16 0";

   lightType = "NoLight";
   lightColor = "0.5 0.5 0.5 1.0";
   lightTime = "1000";
   lightRadius = "3";

   dynamicType = "0";
};
datablock ItemData(PylonItem)
{
   category = "FX1";
   className = "PylonItemClass";
   crosshair_usable = 1;
   crosshair_text = "Pylon";
   isMoveable = 1;
   
   image = PylonItemImage;
   shapeFile = "~/data/shapes/objects/pylon.dts";
   mass = 11.5;
   elasticity = 0.1;
   gravityMod = 1;
   maxVelocity = 1;
   friction = 0.30;
   pickUpName = "a pylon";
   collideable = "1";
   destroyable = "1";
   minDamage = 8;
   maxDamage = 120;
   destroyedLevel = 120;
   explosion = PylonExplosion;
   numDamageMeshes = "5";
   numDamageFragments = "35";
   explosionDamage = 4.5; 
   explosionRadius = 12.0; 

   //sticky = true;
};

function PylonItem::create(%data)
{
   %obj = new Item() {
      dataBlock = %data;
      static = false;
      rotate = false;
      collideable = "1";
      dynamicType = "1";
      destroyable = "1";
      bounce = "1";
   };
   return %obj;
}
function PylonItem::onAdd(%this,%obj)
{
   %obj.destroyable = "1";
   %obj.rotate = "0";
   %obj.bounce = "1";
   %obj.static = false;
}


function PylonItem::onCollision(%this,%obj,%collider)
{
   if(%obj == %collider || getWord(%collider.getTransform(),2) > getWord(%obj.getTransform(),2))
	   return;
   if(%collider.getClassName() $= "Item")
   {
	   %throwForce = 20;
	   %eye = %collider.getTransform();
	   %rotx = getWord(%eye, 3);
	   %roty = getWord(%eye, 4);
	   %rotz = getWord(%eye, 5);
	   %rot = %rotx SPC %roty SPC %rotz;
	   //%vec = vectorScale(%rot, %throwForce);
	   
	   %vel = %collider.getVelocity();
	   if(VectorLen(%vel) > %this.maxVelocity)
		   return;
	   %vec = vectorScale(%vel, %throwForce);
	   
	   // scale down by the mass of the obj
	   %vec = VectorScale(%vec, %obj.getDataBlock().mass);
	   %pos = %obj.getPosition();
	   %obj.applyImpulse(%pos,%vec);
   }
   else
   {
	   %vel = VectorScale(%collider.getVelocity(), %obj.getDataBlock().mass);
	   //%obj.applyImpulse(%obj.getTransform(), %vel);
	
	   %throwForce = %collider.throwForce;
	   if (!%throwForce)
	      %throwForce = 10;
	
	   // Start with the shape's eye vector...
	   %eye = %collider.getEyeVector();
	   %vec = vectorScale(%eye, %throwForce);
	   // scale down by the mass of the obj
	   %vec = VectorScale(%vec, %obj.getDataBlock().mass);
	   // Add the shape's velocity
	   %vec = vectorAdd(%vec,%collider.getVelocity());
	   %pos = %obj.getPosition();
	   %obj.applyImpulse(%pos,%vec);
   }
}

function PylonItem::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
   if(%obj.destroyable || %obj.getDatablock().destroyable)
   {
	   %currDamage = %obj.getDamageLevel();
	   if (%currDamage < %this.destroyedLevel)
	   {
		   // move it
		   %throwForce = %damage * 2;
		
		   %trans = %sourceObject.getTransform();
		   %rotx = getWord(%trans,3);
		   %roty = getWord(%trans,4);
		   %rotz = getWord(%trans,5);
		   %eye = %rotx SPC %roty SPC %rotz;
		   //%vec = vectorScale(%eye, %throwForce);
		   
		   %newEye = VectorSub(%obj.getTransform(), %trans);
		   %vec = vectorScale(%newEye, %throwForce);


		   // scale down by the mass of the obj
		   %vec = VectorScale(%vec, %obj.getDataBlock().mass);
		   %pos = %obj.getPosition();
		   %obj.applyImpulse(%pos,%vec);
	   }

	   %obj.applyDamage(%damage);
   }
}

function PylonItem::onDamage(%this, %obj)
{
   %damage = %obj.getDamageLevel();
   if (%damage >= %this.destroyedLevel)
   {
      if(%obj.getDamageState() !$= "Destroyed")
      {
         if(%obj.respawnTime !$= "")
            %obj.marker.schedule = %obj.marker.data.schedule(%obj.respawnTime, "respawn", %obj.marker); 
         %obj.setDamageState(Destroyed);
      }
   }
   else
   {
      playPylonDamageSound();
      if(%obj.getDamageState() !$= "Enabled")
         %obj.setDamageState(Enabled);
   }
}

function PylonItem::onDestroyed(%data, %obj, %prevState)
{
   radiusPylonExplosion(%data, %obj);
   %obj.schedule(300, "delete");
}

function radiusPylonExplosion(%data, %vehicle)
{
  // this is a modified version of RadiusExplosion() from projectiles.cs
  %position = %vehicle.getPosition();
   // damage players
   InitContainerRadiusSearch(%position, %data.explosionRadius, $TypeMasks::PlayerObjectType);

   %numTargets = 0;
   while ((%targetObject = containerSearchNext()) != 0)
   {
		if(%targetObject == %vehicle)
			continue;

      %dist = containerSearchCurrRadiusDist();

      if (%dist > %data.explosionRadius)
         continue;

      if (%targetObject.isMounted())
      {
         %mount = %targetObject.getObjectMount();
			if(%mount == %vehicle)
				continue;

         %found = -1;
         for (%i = 0; %i < %mount.getDataBlock().numMountPoints; %i++)
         {
            if (%mount.getMountNodeObject(%i) == %targetObject)
            {
               %found = %i;
               break;
            }
         }
      
         if (%found != -1)
         {
            if (%mount.getDataBlock().isProtectedMountPoint[%found] && (%mount != %vehicle))
               continue;
         }
      }

      %targets[%numTargets]     = %targetObject;
      %targetDists[%numTargets] = %dist;
      %numTargets++;
   }

   for (%i = 0; %i < %numTargets; %i++)
   {
      %targetObject = %targets[%i];
      echo("Found target to damage:" SPC %targetObject);
      %dist = %targetDists[%i];

      %coverage = calcExplosionCoverage(%position, %targetObject,
                                        ($TypeMasks::InteriorObjectType |
                                         $TypeMasks::TerrainObjectType |
                                         $TypeMasks::ForceFieldObjectType));
      if (%coverage == 0)
         continue;

      %amount = (1.0 - (%dist / %data.explosionRadius)) * %coverage * %data.explosionDamage;
      %targetData = %targetObject.getDataBlock();

      %momVec = "0 0 1";
      
      if(%amount > 0)
      {
         echo("About to damage object with amount:" SPC %amount);
         // Every shapebase that should be damaged needs to provide this function!
         %targetData.damageObject(%targetObject, %sourceObject, %position, %amount, $DamageType::Explosion, %momVec);
      }
   }
}
function PylonItem::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec)
{
}


function PylonItem::interact(%this,%obj,%client)
{
	   %collider = %client.player;
	   %throwForce = %collider.throwForce;
	   if (!%throwForce)
	      %throwForce = 15;
	
	   // Start with the shape's eye vector...
	   %eye = %collider.getEyeVector();
	   %vec = vectorScale(%eye, %throwForce);
	   // scale down by the mass of the obj
	   %vec = VectorScale(%vec, %obj.getDataBlock().mass);
	   %pos = %obj.getPosition();
	   %obj.applyImpulse(%pos,%vec);
}

