//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function onServerCreated()
{
   %random = getRandom(1,10000);
   // Server::GameType is sent to the master server.
   // This variable should uniquely identify your game and/or mod.
   $Server::GameType = "DHLDEV2";
   $Pref::Server::Name = $GUI::ServerName;
   $Pref::Server::Password = $GUI::ServerPass;
   $Pref::Server::MaxPlayers = $GUI::MaxPlayers;
   // Server::MissionType sent to the master server.  Clients can
   // filter servers based on mission type.
   $Server::MissionType = "Deathmatch";
   $Server::ChanName = ($Server::Dedicated) ? firstWord($Pref::Server::Name) @ %random : $Pref::Player::Name ;
   $Server::ChanKey = ($Server::Dedicated) ? firstWord($Pref::Server::Name) @ %random : firstWord($GUI::ServerName) @ $GUI::MaxPlayers;
   // GameStartTime is the sim time the game started. Used to calculated
   // game elapsed time.
   $Game::StartTime = 0;

   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.
   exec("./gameConnection.cs");
   exec("./audioDescriptions.cs");
   exec("./audioProfiles.cs");
   exec("./camera.cs");
   exec("./markers.cs"); 
   exec("./triggers.cs"); 
   exec("./inventory.cs");
   exec("./shapeBase.cs");
   exec("./item.cs");
   exec("./staticShape.cs");
   exec("./health.cs");
   exec("./radiusDamage.cs");
   exec("./weather.cs");
   exec("./lightning.cs");
   exec("./lights.cs");
   exec("./specialfx.cs");

   //Load all animated objects
   exec("./objects/radara.cs");
   exec("./FountainEmitter.cs");	
   exec("./Fountain1.cs");

   //all Globals go in HERE
    exec("./globals.cs");
    exec("./lights.cs");

   //Load Team Scripts
   exec("./team.cs");
   exec("./teamobjectives.cs");
   
   //triggers for GameTyes
   exec("./gametypes/ctfTrigger.cs");
   exec("./gametypes/healthTrigger.cs");
   exec("./gametypes/ammoTrigger.cs");

   //Load Weapon, Player, Vehicle Classes
   exec("./baseWeapon.cs");
   exec("./basePlayer.cs");
   exec("./baseArmor.cs");
   exec("./baseMecha.cs");
   exec("./baseTankData.cs");
   exec("./baseDestroyables.cs");

   //Base Weapons Stuff
   exec("./weapons/weaponFunctions.cs");
   exec("./weapons/baseAC.cs");
   exec("./weapons/baseEAC.cs");
   exec("./weapons/basePlasma.cs");
   exec("./weapons/baseLaser.cs");
   exec("./weapons/baseHoming.cs");
   exec("./weapons/baseRocket.cs");
   exec("./weapons/baseLARB.cs");
   exec("./weapons/baseGatling.cs");
   exec("./weapons/itemFlare.cs");
   exec("./weapons/itemMine.cs");
   exec("./weapons/weaponFX.cs");

   //Variants
   exec("./weapons/shaddrackWeapons.cs");
   exec("./weapons/hunterscoutWeapons.cs");
   exec("./weapons/abolisherWeapons.cs");
   exec("./weapons/mantisWeapons.cs");
   exec("./weapons/locustWeapons.cs");
   exec("./weapons/juggernautWeapons.cs");
   exec("./weapons/beastWeapons.cs");
   exec("./weapons/predatorWeapons.cs");

   // Loads all player datablocks, and thier child datablocks, etc.
   exec("./addvehicle.cs");

   //Load Explodables
   exec("./explodables/barrels.cs");
   exec("./explodables/barrel.cs");
   exec("./explodables/barrel2.cs");
   exec("./explodables/pylon.cs");
   exec("./explodables/hydrant.cs");
   exec("./explodables/vent.cs");
   exec("./explodables/truck.cs");
   exec("./explodables/hydrantEmitter.cs");

   //Load Turrets
   exec("./turret.cs");
   exec("./genericTurret.cs");

   //FLAG
   exec("./flag.cs");
   
   //Load AI Scripts
   exec("./aiPlayer.cs");
   exec("./aiTurret.cs");
   exec("./genericAITurret.cs");

   //special Weapons
   exec("./weapons/specialWeapons.cs");
   exec("./weapons/deployableTurret.cs");
   exec("./weapons/flamer.cs");
   exec("./weapons/grenadeLauncher.cs");
   //AI TRAINING
   exec("./SP/AI_training.cs");
   exec("./SP/AI_AudioProfile.cs");
   exec("./SP/AI_training_movement.cs");
   exec("./SP/AI_training_weapons.cs");
   exec("./SP/AI_training_objectives.cs");
   exec("./SP/SP_triggers.cs");
   exec("./SP/AI_escort.cs");
   
   //particle editer
   exec("~/ParticleEditor/ParticleEditorStartup.cs");

   // Keep track of when the game started
   $Game::StartTime = $Sim::Time;

   if ($Server::Dedicated)
   {
     $next = true;
     onCyclePauseEnd();
   }
}

function onServerDestroyed()
{
   // This function is called as part of a server shutdown.
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
   // Nothing special for now, just start up the game play.
   resetGlobals();
   startGame();
   //start mission music
}

function onMissionEnded()
{
   // Normally the game should be ended first before the next
   // mission is loaded, this is here in case loadMission has been
   // called directly.  The mission will be ended if the server
   // is destroyed, so we only need to cleanup here.

   cancel($Game::Schedule);
   $Game::Running = false;
   $Game::Cycling = false;
   $missionRunning = false;
}


//-----------------------------------------------------------------------------

function startGame()
{
   // Called once the mission has finished loading
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }

   // Inform the client we're starting up
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      %cl.score = 0;
      %cl.kills = 0;
      %cl.deaths = 0;
      %cl.suicides = 0;
      %cl.objectivesTaken = 0;
      %cl.time = $Sim::Time;
      %cl.endtime = 0;
      //zero the score board
      %cl.incScore(%cl.score);
      %cl.incKills(%cl.kills);
      %cl.incDeaths(%cl.deaths);
      %cl.incObjectives(%cl.objectivesTaken);
      commandToClient(%cl, 'StopAllSounds',"");
   }
   messageAll('MsgClientTeamInfo',"",$Team[1].numPlayers, $Team[2].numPlayers,$Team[1].score, $Team[2].score,$Team[1].objectives,$Team[2],objectives);

   // Start the game timer
   if ($Game::Duration)
   {
      $Game::Schedule = schedule($Game::Duration * 1000, 0, "onGameDurationEnd" );
   }
   if($Server::ServerType $= "SinglePlayer")
      $Game::WinSchedule = schedule($Game::ScoreFrequency, 0, "onGameCheckWinner" );
   $Game::Running = true;
}

function resetGlobals()
{
  $Team[1].score = 0;
  $Team[1].objectives = 0;
  $Team[1].numPlayers = 0;
  $Team[2].score = 0;
  $Team[2].objectives = 0;
  $Team[2].numPlayers = 0;
  $Game::winner = 0;
  $Game::Type = $GUI::MissionType;
  $Game::Report = $GUI::Reporting;
  $Server::Reporting = $Game::Report;
  $Game::PointsForCapture = 5;
  $Game::CycleMaps = $GUI::CycleMaps;
  $Game::PointsForKill = 1;
  if ($GUI::Duration) {
    $Game::Duration = $GUI::DurationTime * 60;
  }
  if ($GUI::EndGameScore)
  {
    $Game::EndGameScore =  $GUI::EndScore;
  }
  if ($GUI::MaxObjectives)
  {
    $Game::MaxObjectives =  $GUI::MaxObjectives;
  }

  $Game::RespawnLimit = $GUI::Respawn;
  $Game::Respawns =  $GUI::RespawnCount;
  echo ("Respawn:" SPC $Game::RespawnLimit SPC $Game::Respawns);
  $Game::GlobalObjectiveTimer = 0;
  $Game::Respawns = $GUI::Respawns;
  $Game::ScoreFrequency = $Game::Duration * 10;
  $Game::EndGamePause = 25;
  $Game::StartTime = $Sim::Time;
  $Game::TeamFlagCaptures[1] = 0;
  $Game::TeamFlagCaptures[2] = 0;
  $Game::FlagTaken[1] = false;
  $Game::FlagTaken[2] = false;
  //gamePlayerListGui.clear(); //clears the scoreboard uneeded
    //PL_EC_Team_List.clear();
   //PL_EC_List.clear();
   //PL_FS_Team_List.clear();
   //PL_FS_List.clear();
   //PL_OB_List.clear();
}

function endGame()
{
   if (!$Game::Running)  {
      error("endGame: No game running!");
      return;
   }
   // Stop any game timers
   cancel($Game::Schedule);
   cancel($Game::WinSchedule);
   // Inform the client the game is over
   if ($Server::MissionFile $= "fps/data/missions/SP_6_blackhawk.mis") {%hide = true; }

   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameEnd',%hide,$Game::winner);
      %cl.selectedMAV = "";
      %cl.selectedTeam = "";
      %cl.missionLoaded = false;
      %cl.ready = false;
      if (%this.endTime == 0)
         %this.endTime = $Sim::time - %this.time;
   }
   resetMission();
   $Game::Running = false;
}

function onGameCheckWinner()
{
   if($Server::ServerType $= "SinglePlayer")
   {  error("checking sp");
   //singleplayer wins
      %bCycling = false;
      if($singlePlayerLevel == 1)
      {
         if($leftMove && $rightMove && $forwardMove && $backwardsMove && $jumpMove && $jetMove)
         {
            $SP::AIDone = true;
         }
      }
      if ($SP::AIDone && $SP::phaseOneDone)
      {
         messageAll( 'MsgGame', '\c2Training Level: %1 complete!', $singlePlayerLevel );
         cycleGame();
         %bCycling = true;
         clearSPL();
      }
      if (%bCycling == false)
      {
         // dont schedule another check if we already have a winner!
         // schedule another score update soon.
         $Game::WinSchedule = schedule($Game::ScoreFrequency, 0, "onGameCheckWinner" );
      }
   }
}

function onGameDurationEnd()
{
   // This "redirect" is here so that we can abort the game cycle if
   // the $Game::Duration variable has been cleared, without having
   // to have a function to cancel the schedule.

   if($Server::ServerType $= "MultiPlayer")
   {
      if ($Game::Duration && !isObject(EditorGui))
      {
         cycleGame();
      }
   }
   else
   {
      youLose();
   }
}


//-----------------------------------------------------------------------------

function cycleGame()
{
   // This is setup as a schedule so that this function can be called
   // directly from object callbacks.  Object callbacks have to be
   // carefull about invoking server functions that could cause
   // their object to be deleted.
   if (!$Game::Cycling) {
      if ($Team[1].score == $Team[2].score)
          $Game::winnner = 0;
      else if ($Team[1].score > $Team[2].score)
          $Game::winner = 1;
      else
          $Game::winner = 2;
            
      cancel($Game::Schedule);
      cancel($Game::WinSchedule);
      $Game::Cycling = true;
      $Game::Schedule = schedule(0,0,"onCycleExec");
   }
}

function onCycleExec()
{
   // End the current game and start another one, we'll pause for a little
   // so the end game victory screen can be examined by the clients.
   endGame();
   if ($Server::ServerType $= "MultiPlayer")
   {
      $Game::Schedule = schedule(0,0,"onCyclePauseEnd");
   }
}

function onCyclePauseEnd()
{
   if ($Game::Running)  {
      error("game running!");
      return;
   }
   if ($Server::ServerType $= "SinglePlayer") {
      if ($Server::Replay)
      {
         loadMission($Server::MissionFile);
         return;
      }
      // Just cycle through the missions for now.
      switch$($Server::MissionFile)
      {
          case "fps/data/missions/release/SP_1_range.mis":
              %file ="fps/data/missions/release/SP_2_range.mis";
          case "fps/data/missions/release/SP_2_range.mis":
               %file = "fps/data/missions/release/SP_3_range.mis";
          case "fps/data/missions/release/SP_3_range.mis":
              %file = "fps/data/missions/release/SP_fortress.mis";
          case "fps/data/missions/release/SP_fortress.mis":
               %file = "fps/data/missions/release/SP_sector1.mis";
          case "fps/data/missions/release/SP_sector1.mis":
               %file = "fps/data/missions/release/SP_blackhawk.mis";
          case "fps/data/missions/release/SP_blackhawk.mis":
               %file ="fps/data/missions/release/SP_1_range.mis";
      }
      loadMission(%file);
   }
   //Clear out schedules
   for (%cl = 0; %cl < ClientGroup.getCount(); %cl++)
   {
      %client = ClientGroup.getObject(%cl);
      cancel(%client.schedule);
   }
   
   if ($Server::Dedicated)
   {
      if ($Dedicated::Mode == 1)
      {
         %missionfile = $Dedicated::MissionFile;
      }
      if ($Dedicated::Mode == 2)
      {
         // Just cycle through the missions for now.
         %search = "*/missions/*.mis";
         //If this is the first mission, mission file is blank
         echo($Server::MissionFile);
         if ($Server::MissionFile $= "")
            %next = true;
         for (%file = findFirstFile(%search); %file !$= ""; %file = findNextFile(%search))
         {
            echo(%file);
            if(%next)
            {
               %MissionInfoObject = readMissionFile(%file);
               if (%MissionInfoObject.missiontype == $Dedicated::GameType)
               {
                  %missionfile = %file;
                  echo(%file);
                  break;
               }
            }
            if (%file $= $Server::MissionFile)
            {
               %next = true;
            }
         }
         if (%missionfile $= "")
         {
            for (%file = findFirstFile(%search); %file !$= ""; %file = findNextFile(%search)) {
               if(%next) {
                  %MissionInfoObject = readMissionFile(%file);
                  if (%MissionInfoObject.missiontype == $Dedicated::GameType)
                  {
                     %missionfile = %file;
                     echo(%file);
                     break;
                  }
               }
            }
         }
      }
      if ($Dedicated::Mode == 3)
      {
         //Increment number.
         $Dedicated::CurrentMapNum++;
         //Check to see if new number is valid
         if ($Dedicated::Map[$Dedicated::CurrentMapNum] $= "")
            $Dedicated::CurrentMapNum = 0;

         %missionfile = $Dedicated::Map[$Dedicated::CurrentMapNum];
      }
      echo("File" SPC %missionFile);
      //Load map
      %MissionInfoObject = readMissionFile(%missionfile);
      if (%MissionInfoObject.missiontype > 1)
      {
         $GUI::MaxObjectives = %MissionInfoObject.totalObjectives;
         $GUI::EndScore = %MissionInfoObject.totalscore;
         $GUI::EndGameScore = true;
      }
      $Server::MissionID = %MissionInfoObject.map_id;
      $Server::MissionType = %MissionInfoObject.missiontype;
      $Server::MissionName = %MissionInfoObject.name;
      $GUI::MissionName = $Server::MissionName;
      $GUI::MissionTypeName = $Server::MissionType;
      MessageAll('MsgSetLaunch', '', true);
      MessageAll('MsgHSUpdate','',$GUI::MissionName,$GUI::MissionTypeName,$GUI::DurationTime,
                                  $GUI::EndScore, $GUI::RespawnCount, $GUI::Practice, $GUI::Reporting,
                                  $GUI::Attacker, $GUI::Defender);
      loadMission(%missionfile);
   }
}

function readMissionFile (%missionFile)
{
   %file = new FileObject();

   %MissionInfoObject = "";

   if ( %file.openForRead( %missionFile ) ) {
		%inInfoBlock = false;

		while ( !%file.isEOF() ) {
			%line = %file.readLine();
			%line = trim( %line );

			if( %line $= "new ScriptObject(MissionInfo) {" )
				%inInfoBlock = true;
			else if( %inInfoBlock && %line $= "};" ) {
				%inInfoBlock = false;
				%MissionInfoObject = %MissionInfoObject @ %line;
				break;
			}

			if( %inInfoBlock )
			   %MissionInfoObject = %MissionInfoObject @ %line @ " ";
		}

		%file.close();
	}
	%MissionInfoObject = "%MissionInfoObject = " @ %MissionInfoObject;
	eval( %MissionInfoObject );

   %file.delete();
   return %MissionInfoObject;
}
