
//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

// Client Connection
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// This script function is the first called on a client accept
//
function GameConnection::onConnect( %client, %name )
{
   // Send down the connection error info, the client is
   // responsible for displaying this message if a connection
   // error occures.
   messageClient(%client,'MsgConnectionError',"",$Pref::Server::ConnectionError);

   // Send mission information to the client


   // if hosting this server, set this client to superAdmin
   if (%client.getAddress() $= "local") {
      %client.isAdmin = true;
      %client.isSuperAdmin = true;
   }

   // Get the client's unique id:
   // %authInfo = %client.getAuthInfo();
   // %client.guid = getField( %authInfo, 3 );
   //%client.guid = 0;
   //addToServerGuidList( %client.guid );
   %client.name = %name;
   // Staging Area Values
   %client.selectedMAV = "";
   %client.selectedTeam = "";
   %client.missionLoaded = false;
   %client.ready = false;
   
   $ServerGroup = ServerGroup;
   $instantGroup = MissionCleanup;

   //ai stuff
   if ($localClientDefined != 1)
   {
      $localClient = %client;
      $localClientDefined = 1;
   }
   $latestClient = %client;

   if ($Server::ServerType $= "MultiPlayer")
   {
     //Have them connect to Server IRC channel
     commandToClient(%client, 'GameStart', "",$Server::ChanName, $Server::ChanKey);
     //Relay Staging Area Host Data
     serverCmdRelayHostData(%client);

     commandToClient(%client, 'NewStagingArea');
     MessageClient(%client,'MsgSetLaunch', '', $missionRunning);
     commandToClient(%client, 'SyncClock', $Sim::Time - $Game::StartTime);
   } else {
     sendLoadInfoToClient( %client );
     %client.ready = true;
     %client.loadMission();
   }

   // Inform the client of all the other clients
   for (%cl = 0; %cl < ClientGroup.getCount(); %cl++) {
      %other = ClientGroup.getObject(%cl);
      if ((%other != %client)) {
         messageClient(%client, 'MsgClientJoinInfo', "", %other.name,
               %other, %other.score, %other.isAIControlled(), %other.isAdmin,
               %other.isSuperAdmin,%other.deaths,%other.kills, %other.objectivesTaken,%other.selectedTeam,%other.selectedMAV);
      }
   }
   // Send Client Team Information
   messageClient(%client,'MsgClientTeamInfo',"",$Team[1].numPlayers, $Team[2].numPlayers,$Team[1].score, $Team[2].score,$Team[1].objectives,$Team[2],objectives);
   // Send Client Join Text
   messageClient(%client,'MsgClientJoin', '\c1Welcome to the battle %1. We\'ve been expecting you.',
      %client.name, %client, %client.sendGuid, %client.isAiControlled(),
      %client.isAdmin, %client.isSuperAdmin);
   if ($Server::Dedicated)
      messageClient(%client,'MsgClient',$Dedicated::MOTD);
   // Let everyone else know the client has joined
   messageAllExcept(%client, -1, 'MsgClientJoin', '\c1%1 joined the battle.',
      %client.name, %client, %client.sendGuid, %client.isAiControlled(),
      %client.isAdmin, %client.isSuperAdmin);
   %client.MaxMinesDeployed = 10;
   %client.MaxTurretsDeployed = 3;
   %client.currentDeployed = 0;
   //Increment Player Count
   $Server::PlayerCount++;
   //Check if Server is Multiplayer or Singleplayer
   //Use Staging area if Multiplayer, otherwise load mission
}


function GameConnection::onLaunch(%this,%client)
{
   if (%this.selectedTeam $= "")
   {
      MessageClient(%this, -1, '',"Cannot Launch! You have not joined a team yet.");
      return; //No team, No Join
   }
   if (%this.selectedMAV $= "")
   {
      MessageClient(%this, -1, '',"Cannot Launch! You have not joined selected a MAV yet.");
      return; //No Mav, No Join
   }

   //Launch Client into Mission (Staging Area)
   if (!%this.missionLoaded) {
      commandToClient(%this, 'StopMissionMusic',"");
      sendLoadInfoToClient( %this );
      %this.loadMission();
      %this.missionLoaded = true;
   }
   else
   {
      spawnPlayer(%this);
   }
}

function GameConnection::onClientEnterGame(%this)
{
   commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);
   commandToClient(%this, 'PlayMissionMusic');
   // Create a new camera object.
   %this.camera = new Camera() {
      dataBlock = Observer;
   };
   MissionCleanup.add( %this.camera );
   %this.camera.scopeToClient(%this);
   // Setup game parameters, the onConnect method currently starts
   // everyone with a 0 score.
   %this.score = 0;
   %this.suicides = 0;
   %this.team = 0;
   %this.kills = 0;
   %this.deaths = 0;
   %this.time = $Sim::time;
   %this.endtime = 0;
   %this.objectivesTaken = 0;
   %this.currentDeployed = 0;
   
   if($Server::ServerType $= "SinglePlayer")
   {
      //sendLoadInfoToClient( %this );
      //Spawn as camera by default
      //spawn where you need to be for SP
      //need to add team here  = FS
      //pick spawn point       = FS team1 spawnpoint
      //lock mecha             = ?!?!?!?! stop from moving for a while
      //start AI               = see below
      %this.team = $Team[1];
      %msg = "\c1You have joined " @ %this.team.name;
      messageClient(%this, 'MsgClientJoinTeam', %msg, %this.name, %this, %this.team.teamID);
      //  %this.camera.setTransform(pickObserverPoint(%this));

      %this.spawnPoint = pickSpawnPoint(%this);
      %this.camera.setTransform(%this.spawnPoint);
      %x = getWord(%this.spawnPoint,0);
      %y = getWord(%this.spawnPoint,1);
      %camEyeTrans = %x SPC %y SPC "300 0.988123 -0.0989949 0.117531 1.41179";
      echo(%camEyeTrans);
      %this.camera.setTransform(%camEyeTrans);
      %this.camera.setVelocity("0 0 0");
      %this.setControlObject(%this.camera);
   
      error("*******************Single*Player*AI*SYSTEM*************************");
      switch$($singlePlayerLevel)
      {
         case "1":
            %this.startTraining1();
         case "2":
            %this.startTraining2();
         case "3":
            %this.startTraining3();
         case "4":
            %this.startTraining4();
         case "5":
            %this.startTraining5();
         case "6":
            %this.startTraining6();
         default:
            generateBots($maxBots);
       }
   }
   else
   {
      //general location for MP
      spawnPlayer(%this);
   }
}

function GameConnection::onClientLeaveGame(%this)
{
   if(%this.team){
      %this.team.numPlayers--;
   }
   if (isObject(%this.PathCamera))
      %this.PathCamera.delete();
   if (isObject(%this.Path))
      %this.Path.delete();
   if (isObject(%this.light))
      %this.light.delete();
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
   %this.endTime = $Sim::time - %this.time;
   if($singlePlayerLevel){
      clearSPL();
   }
}

// Mission Area
//-----------------------------------------------------------------------------

function GameConnection::onLeaveMissionArea(%this)
{
   // The control objects invoked this method when they
   // move out of the mission area.
}

function GameConnection::onEnterMissionArea(%this)
{
   // The control objects invoked this method when they
   // move back into the mission area.
}

// on Death
//-----------------------------------------------------------------------------
function GameConnection::checkScores(%this){
   if ($Game::EndGameScore > 0) {
      if (%this.score >= $Game::EndGameScore || $Team[%this.team.teamId].score >= $Game::EndGameScore)
      {
         $Game::winner = %this.team.teamId;
         cycleGame();
      }
   }
}

function GameConnection::checkObjectiveScores(%this){
   if ($Game::EndGameScore > 0 && $Game::MaxObjectives != 0) {
      if ($Team[%this.team.teamId].objectives >= $Game::MaxObjectives)
      {
         if($Game::GlobalObjectiveTimer == 0){
            $Game::GlobalObjectiveTimer = 60;
            messageAll('', '\c2 %1 Seconds left until %2 wins',$Game::GlobalObjectiveTimer,%this.team.name);
         }
         cancel($Game::ObjTimer);
         $Game::ObjTimer = schedule(10000, 0, "continueObjectiveTimer", %this.team.teamId );
      }
   }
}
function continueObjectiveTimer(%currentOwningTeam){
   cancel($Game::ObjTimer);
   if ($Game::EndGameScore > 0 && $Game::MaxObjectives != 0)
      if ($Team[%currentOwningTeam].objectives >= $Game::MaxObjectives)
      {
          $Game::GlobalObjectiveTimer -= 10;
          messageAll('', '\c2 %1 Seconds left until %2 wins',$Game::GlobalObjectiveTimer,$Team[%currentOwningTeam].name);
          if($Game::GlobalObjectiveTimer <= 0)
          {
             $Game::winner = %this.team.teamId;
             cycleGame();
          }
          else
             $Game::ObjTimer = schedule(10000, 0, "continueObjectiveTimer", %currentOwningTeam );
      }
      else
      {
         $Game::GlobalObjectiveTimer = 0;
      }

      
}

function GameConnection::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc)
{
   echo("OnDeath");
   // Clear out the name on the corpse
   %this.player.setShapeName("");
   // Switch the client over to the death cam and unhook the player object.
   if (isObject(%this.camera) && isObject(%this.player)) {
      %this.setControlObject(%this.camera);
      %this.camera.setMode("Corpse",%this);
   }
   %this.player = 0;
   if (%sourceClient.isTurret)
   {
      %this.incDeaths(1);
      if (%sourceClient.getTeam() == 1)
         messageAll('MsgClientKilled','\c5%1 is destroyed by Federated States Turret!',%this.name);
      else
         messageAll('MsgClientKilled','\c5%1 is destroyed by Eastern Confederation Turret!',%this.name);
   }
   else if (%sourceClient.aiPlayer)
   {
      %Name = %sourceClient.getShapeName();
      %this.incDeaths(1);
      if(%this.team.teamID == %sourceClient.team.teamID)
      {
         messageAll('MsgClientKilled','\c5%1 killed his \c4team member \c5%2!',%Name,%this.name, %this.team.name);
      } else {
         //.death on killed, .kills on killer
         messageAll('MsgClientKilled','\c5%1 is destroyed by %2!',%this.name,%Name);
      }
   }
   else
   {
      %Name = %sourceClient.name;
      // Doll out points and display an appropriate message
      if (%damageType $= "Suicide" || %sourceClient == %this) {
         //death on suicide
         %this.incScore(-$Game::PointsForKill);
         %this.suicides++;
         %this.incDeaths(1);
         messageAll('MsgClientKilled','\c5%1 takes \c4his own life\c5!',%this.name);
      } else {
         if(%this.team.teamID == %sourceClient.team.teamID)
         {
            %sourceClient.incScore(-$Game::PointsForKill);
            %sourceClient.incTeamScore(-$Game::PointsForKill);
            %sourceClient.incKills(1);
            %this.incDeaths(1);
            messageAll('MsgClientKilled','\c5%1 killed his \c4team member \c5%2!',%Name,%this.name, %this.team.name);
         } else {

            //.death on killed, .kills on killer
            %sourceClient.incScore($Game::PointsForKill);
            %sourceClient.incTeamScore($Game::PointsForKill);
            %sourceClient.incKills(1);
            %this.incDeaths(1);
            messageAll('MsgClientKilled','\c5%1 is destroyed by %2!',%this.name,%Name);

         }
      }
   }

   if ($Server::ServerType $= "MultiPlayer")
   {
      if ($Game::RespawnLimit && $Game::Respawns <= %this.deaths)
         %this.schedule = schedule(5000,0,"noSpawns",%this);
      else
         %this.schedule = schedule(5000,0,"spawnPlayer",%this);
   }

}

// Pick Spawn Point
//-----------------------------------------------------------------------------
function spawnPlayer(%this)
{
   // Combination create player and drop him somewhere
   // Create a player object.
   %this.camera.setVelocity("0 0 0");
   %this.setControlObject(%this.camera);
   if (isObject(%this.PathCamera))
      %this.PathCamera.delete();
   if (isObject(%this.Path))
      %this.Path.delete();
   if (isObject(%this.light))
      %this.light.delete();
   %this.joinTeam(%this.selectedTeam);
   %this.camera.setTeam(%this.team.teamID);
   if (!%this.selectedMAVNum) { %this.selectedMAVNum = 3; }
   serverCmdAddVehicle(%this,%this.selectedMAV,%this.selectedMAVNum);

}
function noSpawns(%this)
{
   commandToClient(%this,'noSpawnsLeft');
}

//--------------------------------------------------------------------------
// Sync the clock on the client.

function GameConnection::syncClock(%client, %time)
{
   commandToClient(%client, 'syncClock', %time);
}

//--------------------------------------------------------------------------
// Update all the clients with the new score

function GameConnection::incScore(%this,%delta)
{
   %this.score += %delta;
   messageAll('MsgClientScoreChanged', "", %this.score, %this);
   %this.checkScores();
}

function GameConnection::incDeaths(%this,%delta)
{
   %this.deaths += %delta;
    messageAll('MsgClientDeathsChanged', "", %this.deaths, %this);
}

function GameConnection::incKills(%this,%delta)
{
   %this.kills += %delta;
   messageAll('MsgClientKillsChanged', "", %this.kills, %this);
}

function GameConnection::incObjectives(%this,%delta)
{
   %this.objectivesTaken += %delta;
   messageAll('MsgClientObjectivesChanged', "", %this.objectivesTaken, %this);
   %this.checkObjectiveScores();
}

function GameConnection::incTeamScore(%this,%delta)
{
   $Team[%this.team.teamId].score += %delta;
   messageAll('MsgClientTeamScoreChanged', "", $Team[%this.team.teamId].score, %this);
   %this.checkScores();
}

function GameConnection::incTeamObjectives(%this,%delta)
{
   messageAll('MsgClientTeamObjectiveChanged', "", $Team[%this.team.teamId].objectives, %this);
}
// Set Ammo Amount to Hud
function GameConnection::setAmmoAmountHud(%client, %amount){
    commandToClient(%client, 'SetAmmoAmountHud', %amount);
}
function GameConnection::setCounterAmountHud(%client, %amount){
    commandToClient(%client, 'SetCounterAmountHud', %amount);
}
