//-----------------------------------------------------------------------------
// Realm Wars - Team Objective Module
//
// Author:     James Lupiani (JL) <defiant@flyingtemple.com>
// Created:    21 Mar 2002 (JL)
// Revised:    22 Mar 2002 (JL)
//
// Notes:   This is a hack, but it more or less works.
//          - It will have to be overhauled once team support is fully in.
//            The number and names of teams and the number of objectives to win
//            are right in here. That sort of info should come from the mission.
//
//          - If the mission time runs out, the number of objectives from the
//            last mission will remain. See above note for suggested fix. +)
//
//          - Does the particle emitter need to be added to MissionCleanup?
//
//          - Isn't there a cleaner way of implementing smoke that's simply
//            a different color?
//
// Copyright (c) 2001 GarageGames.Com
//-----------------------------------------------------------------------------

//--------------------------------------------------------------
// Particle data
//--------------------------------------------------------------
datablock ParticleData( ObjectiveSmoke )
{
   textureName          = "~/data/specialfx/smoke2";
   dragCoeffiecient     = 0.0;
   gravityCoefficient   = -0.5;   // rises slowly
   inheritedVelFactor   = 90.00;
   lifetimeMS           = 3500;
   lifetimeVarianceMS   = 250;
   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0] = "0.7 0.7 0.0 0.7";      //Red,Green,Blue,Intensity 
   colors[1] = "0.9 0.8 0.0 0.3";
   colors[2] = "0.8 0.8 0.0 0.2";

   sizes[0]      = 2.0;
   sizes[1]      = 5.0;
   sizes[2]      = 10.0;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData( ObjectiveSmokeEmitter )
{
   ejectionPeriodMS = 20;
   periodVarianceMS = 5;

   ejectionVelocity = 0.25;
   velocityVariance = 0.10;

   thetaMin         = 0.0;
   thetaMax         = 90.0;

   particles = ObjectiveSmoke;
};

datablock ParticleEmitterNodeData( ObjectiveEmitterNode )
{
   timeMultiple = 1;
};


//--------------------------------------------------------------
// Derived particle datablocks
//--------------------------------------------------------------
datablock ParticleData( RedObjSmoke : ObjectiveSmoke )
{
   colors[0]     = "1.0 1.0 1.0 0.1";
   colors[1]     = "0.8 0.8 0.8 0.1";
   colors[2]     = "0.6 0.6 0.6 0.0";
};

datablock ParticleData( BlueObjSmoke : ObjectiveSmoke )
{
   colors[0] = "0.7 0.7 0.0 0.7";      //Red,Green,Blue,Intensity 
   colors[1] = "0.9 0.8 0.0 0.3";
   colors[2] = "0.0 0.0 0.0 0.2";
};

datablock ParticleEmitterData( RedObjSmokeEmitter : ObjectiveSmokeEmitter )
{
   particles = RedObjSmoke;
};

datablock ParticleEmitterData( BlueObjSmokeEmitter : ObjectiveSmokeEmitter )
{
   particles = BlueObjSmoke;
};


//--------------------------------------------------------------
// Trigger datablock
//--------------------------------------------------------------
datablock TriggerData(TeamTrigger)
{
    tickPeriodMS = 100;
    team = -1;
    teamEmitter = 0;
    showInMap = 1;
};
function TeamTrigger::OnAdd(%this,%obj){
      if(%this.team){
         %obj.setTeam(%this.team);
         }

}
//--------------------------------------------------------------
// Events
//--------------------------------------------------------------
function TeamTrigger::onEnterTrigger( %this, %trigger, %obj )
{
   // This method is called whenever an object enters the %trigger
   // area, the object is passed as %obj.  The default onEnterTrigger
   // method (in the C++ code) invokes the ::onTrigger(%trigger,1) method on
   // every object (whatever it's type) in the same group as the trigger.
   Parent::onEnterTrigger( %this, %trigger, %obj );
   //echo("Player's team is " @ %obj.client.team.teamId);
   //echo("Trigger's team is " @%trigger.team);
   // Does the objective belong to the capper's team?
  // if (%obj.AIPlayer == 1 )
   //{
   // return ;
   // }
   if( %obj.client.team.teamId != %trigger.team )
   {
      %num = 1;
      //echo("TRUE LEVEL " @ %num);
      %num++;
      // Has this objective been taken yet?
      if( %trigger.team != -1 )
      {
         %num++;
     	   $Team[%trigger.team].objectives--;
         $Team[%obj.client.team.teamId].objectives++;
      }
      else
      {

         $Team[%obj.client.team.teamId].objectives++;
      }
      if(%trigger.scorePoints)
      {
        %obj.client.incTeamScore(%trigger.scorePoints);
        %obj.client.incScore(%trigger.scorePoints);
      }
      else
      {
        %obj.client.incTeamScore($Game::PointsForCapture);
        %obj.client.incScore($Game::PointsForCapture);
      }
      %obj.client.incObjectives(1);
      if (isObject(%trigger.teamEmitter))
         %trigger.teamEmitter.delete();
            // Make a new particle emitter for the capping team.
      if( %obj.client.team.teamId == 1 )
   	  {
        %trigger.teamEmitter = new ParticleEmitterNode(%trigger.getName() @ "Emitter")
        {
           position = %trigger.getWorldBoxCenter();
           datablock = ObjectiveEmitterNode;
           emitter = RedObjSmokeEmitter;
           velocity = 1;
        };
      }
      else if( %obj.client.team.teamId == 2 )
      {
        %trigger.teamEmitter = new ParticleEmitterNode(%trigger.getName() @ "Emitter")
        {
           position = %trigger.getWorldBoxCenter();
           datablock = ObjectiveEmitterNode;
           emitter = BlueObjSmokeEmitter;
           velocity = 1;
        };
      }
      if(%trigger.turretGroup)
      {
         setTurretTeam(%obj.client.team.teamId, %trigger.turretGroup);
      }
       // Do the actual claiming.
      %trigger.team = %obj.client.team.teamId;
      %trigger.setTeam(%obj.client.team.teamId);
      //%1 = %obj.client.team.name;
      messageAll( 'MsgObjective', '\c2The %1 took an objective!', %obj.client.team.name );
      messageAll( 'MsgObjectiveUpdate', '\c2FS Objectives: %1', $Team[1].objectives,1);
      messageAll( 'MsgObjectiveUpdate', '\c2EC Objectives: %1', $Team[2].objectives,2 );
    }
}

function TeamTrigger::onLeaveTrigger( %this, %trigger, %obj )
{    echo("leaving team trigger");
   // This method is called whenever an object leaves the %trigger
   // area, the object is passed as %obj.  The default onLeaveTrigger
   // method (in the C++ code) invokes the ::onTrigger(%trigger,0) method on
   // every object (whatever it's type) in the same group as the trigger.
   Parent::onLeaveTrigger( %this, %trigger, %obj );
}


function TeamTrigger::onTickTrigger( %this, %trigger )
{
   // This method is called every tickPerioMS, as long as any
   // objects intersect the trigger. The default onTriggerTick
   // method (in the C++ code) invokes the ::onTriggerTick(%trigger) method on
   // every object (whatever it's type) in the same group as the trigger.

   // You can iterate through the objects in the list by using these
   // methods:
   //    %this.getNumObjects();
   //    %this.getObject(n);
   Parent::onTickTrigger( %this, %trigger );

}

function setTurretTeam( %teamId, %groupNum)
{
//echo("TeamTrigger::onLeaveTrigger called");

//echo(" teamid " @ %teamId);
//echo("group "@ %groupNum);

   %triggerGroup = "TurretGroup" @ %groupNum;

   if(isObject(%triggerGroup))
   {
      //echo("found " @ %triggerGroup);
      %maxTurrets  = %triggerGroup.getCount();
      for(%i = 0; %i < %maxTurrets; %i++){
         %turret = %triggerGroup.getObject(%i);
         %turret.team = %teamId;
         if (%turret.getType() & $TypeMasks::GameBaseObjectType)
            %turret.setTeam(%teamId);
         if (%turret.isTurret)
            %turret.getDatablock().resetTarget(%turret);
      }
   }
}


