//-----------------------------------------------------------------------------
// Torque Game Engine
//
// Copyright (c) 2001 GarageGames.Com
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Gatling weapon. This file contains all the items related to this weapon
// including explosions, ammo, the item and the weapon item image.
// These objects rely on the item & inventory support system defined
// in item.cs and inventory.cs
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Projectile trail emitter
datablock ParticleData(gatlingSmokeParticle)
{
   textureName          = "~/data/specialfx/smoke1";

   dragCoeffiecient     = 0.0;
   gravityCoefficient   = 0.1;
   inheritedVelFactor   = 0.00;

   lifetimeMS           = 600;   // time in ms org 300
   lifetimeVarianceMS   = 0;   // ...more or less

   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "0.56 0.36 0.26 1.0";
   colors[1]     = "0.56 0.36 0.26 0.33"; // org 0.56 0.36 0.26 1.0
   colors[2]     = "1 1 1 0"; // org 0 0 0 0

   sizes[0]      = 0.25; // org 0.15
   sizes[1]      = 0.3; // org 0.2
   sizes[2]      = 0.5; // org 0.4
   sizes[3]      = 0.5; // org 0.4

   times[0]      = 0.0;
   times[1]      = 0.3;
   times[2]      = 1.0;
   times[3]      = 1.0;
};

datablock ParticleEmitterData(gatlingSmokeEmitter)
{
   ejectionPeriodMS = 8;
   periodVarianceMS = 0;
   ejectionVelocity = 0.0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 0;
   overrideAdvances = false;
   particles = gatlingSmokeParticle;
};


//-----------------------------------------------------------------------------
// Weapon fire emitter
//-----------------------------------------------------------------------------

datablock ParticleData(gatlingFireParticle)
{
   textureName          = "~/data/specialfx/smoke1";

   dragCoeffiecient     = 50;
   gravityCoefficient   = 0;  // rises -- org -0.1
   inheritedVelFactor   = 0.3;

   lifetimeMS           = 150;   // Time in ms -- org 100
   lifetimeVarianceMS   = 25;    // ...more or less

   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "1 0.84 0.44 0.66"; // org 0 0.2 1 1.0
   colors[1]     = "0.9 0.6 0.21 1"; // org 1 1 1 1.0
   colors[2]     = "1 0 0 0"; // org 0 0 0 0

   sizes[0]      = 0.75;
   sizes[1]      = 0.5;
   sizes[2]      = 0.25;

   times[0]      = 0.0;
   times[1]      = 0.3;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(gatlingFireEmitter)
{
   ejectionPeriodMS = 10; /// org 30
   periodVarianceMS = 2; // org 5

   ejectionVelocity = 1;
   ejectionOffset   = 0.1;
   velocityVariance = 0.10;

   thetaMin         = 0.0;
   thetaMax         = 180.0; // org 10

   particles = gatlingFireParticle;
   
   
};


//-----------------------------------------------------------------------------
// Projectile Explosion

datablock ParticleData(gatlingExplosionParticle)
{
   dragCoefficient      = 2;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 750;
   lifetimeVarianceMS   = 150;
   textureName          = "~/data/specialfx/particleTest";
   colors[0]     = "0.56 0.36 0.26 1.0";
   colors[1]     = "0.56 0.36 0.26 0.0";
   sizes[0]      = 0.15; // org 0.5
   sizes[1]      = 1.0;
};

datablock ParticleEmitterData(gatlingExplosionEmitter)
{
   ejectionPeriodMS = 7;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "gatlingExplosionParticle";
};

datablock ExplosionData(gatlingExplosion)
{
   particleEmitter = gatlingExplosionEmitter;
   particleDensity = 10;
   particleRadius = 0.15;

   faceViewer     = true;
   explosionScale = "1 1 1";

   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "1.0 1.0 1.0";
   camShakeDuration = 0.2;
   camShakeRadius = 3.0;
   
   // Dynamic light
   lightStartRadius = 0.5; // org 6
   lightEndRadius = 0;
   lightStartColor = "0.56 0.36 0.26";  //new
   lightEndColor = "0 0 0";  //new
};


//-----------------------------------------------------------------------------
// Projectile Object

datablock ProjectileData(gatlingProjectile)
{
//   projectileShapeName = "~/data/shapes/weapons/gatling/projectile.dts";
   directDamage        = 5;
   radiusDamage        = 0.5;
   damageRadius        = 0.5;
   explosion           = gatlingExplosion;
   particleEmitter     = gatlingSmokeEmitter;

   muzzleVelocity      = 300;
   velInheritFactor    = 0.1;

   armingDelay         = 0;
   lifetime            = 2000;
   fadeDelay           = 1500;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = true;
   gravityMod = 0.10;

   hasLight    = true;
   lightRadius = 1.0;
   lightColor  = "0.5 0.5 0";
};

//-----------------------------------------------------------------------------
// Ammo Item

datablock ItemData(gatlingAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

	// Dynamic properties defined by the scripts
	pickUpName = "gatling ammo";
   maxInventory = 5000;
};


//--------------------------------------------------------------------------
// gatling shell that's ejected during reload.

datablock DebrisData(gatlingShell)
{
   shapeFile = "~/data/shapes/weapons/shell.dts";
   lifetime = 3.0;
   minSpinSpeed = 300.0;
   maxSpinSpeed = 400.0;
   elasticity = 0.5;
   friction = 0.2;
   numBounces = 4;
   staticOnMaxBounce = true;
   snapOnMaxBounce = false;
   fade = true;
};


//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the RifleImage is used.

datablock ItemData(gatling)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/gatling.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

	// Dynamic properties defined by the scripts
	pickUpName = "a gatling";
	image = gatlingImage;
};


//--------------------------------------------------------------------------
// gatling image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(gatlingImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/gatling.dts";
   emap = true;
   mountPoint = 0;
   correctMuzzleVector = true;
   className = "WeaponImage";
   fireEmitter = GatlingFireEmitter;
   
   item = gatling;
   ammo = gatlingAmmo;
   projectile = gatlingProjectile;
   projectileType = Projectile;

   projectileSpread = 12/1500; // **** # projectile/spread variance smaller number more spread
   // This defines the spread of the projectiles. If no value is set the gun will fire
   // deadly accurate.

   isShotGun = false; // **** Added in Projectile Spread Tutorial
   // This defines whether the weapon should act like a shotgun or not.
   // Feel free to change as you see fit for fun ;)

   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.5;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "SpinUp";

   //--------------------------------------
   stateName[3]                     = "SpinUp";
   stateSound[3]                    = GatlingFireStartSound;
   stateTimeoutValue[3]             = 0.25;
   stateWaitForTimeout[3]           = false;
   stateTransitionOnTimeout[3]      = "FullSpeed";

   stateName[4]                     = "FullSpeed";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTriggerDown[4]  = "Fire";
   stateTransitionOnTriggerUp[4]    = "SpinDown";

   // Fire the weapon. Calls the fire script which does
   // the actual work.
   stateName[5]                     = "Fire";
   stateFire[5]                     = true;
   stateRecoil[5]                   = LightRecoil;
   stateAllowImageChange[5]         = false;
   stateSequence[5]                 = "Fire";
   stateScript[5]                   = "onFire";
   stateEmitter[5]                  = GatlingFireEmitter;
   stateEmitterTime[5]              = 0.5;
   stateSound[5]                    = GatlingFireSound;
   stateTimeoutValue[5]             = 0.05;
   stateTransitionOnTimeout[5]      = "Reload";
   //
   stateName[9]                    = "Spindown";
   stateSound[9]                   = GatlingFireStopSound;
   stateTimeoutValue[9]            = 0.25;
   stateTransitionOnTimeout[9]     = "Ready";

   // Play the relead animation, and transition into
   stateName[6]                     = "Reload";
   stateTransitionOnNoAmmo[6]       = "NoAmmo";
   stateTransitiononTimeout[6]      = "FullSpeed";
   stateTimeoutValue[6]             = 0.05;
   stateAllowImageChange[6]         = false;
   stateSequence[6]                 = "Reload";
   stateEjectShell[6]               = true;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[7]                     = "NoAmmo";
   stateTransitionOnAmmo[7]         = "Reload";
   stateSequence[7]                 = "NoAmmo";
   stateTransitionOnTriggerDown[7]  = "DryFire";

   // No ammo dry fire
   stateName[8]                     = "DryFire";
   stateTimeoutValue[8]             = 1.0;
   stateTransitionOnTimeout[8]      = "NoAmmo";
   stateSound[8]                    = "GatlingDryFireSound";
};

function baseGatling::onFire(%this, %obj, %slot,%ammo)
{
		// Decrease the weapons ammo on fire
		//%obj.decInventory(%this.ammo,1);


		// Get the type of projectile we are gonna fire
		%projectile = %this.projectile;
         if(!%this.usesEnergy)
		%obj.decInventory(%this.ammo,%ammo);

        if(!isSpecialWeapon(%this.item))
        if(!%obj.aiPlayer)
           %obj.client.setAmmoAmountHud(%obj.getInventory(%this.ammo));

        if(!%obj.aiPlayer)
           %clientID = %obj.client;
        else
           %clientID = %obj;
		
		// Get the weapons projectile spread and ensure it is never 0 
		//   (we need some spread direction even if it is extremely tiny)
		%spread = %this.projectileSpread;
		%spread = %spread $= "" ? 0.001 : %spread;
		%spread = %spread <= 0 ? 0.001 : %spread;

		// Determine if we are using a shotgun class weapon or not
		//   If we are using a shotgun, set the number of projectiles we are going to fire to 12
		%shellcount = 1;
		if(%this.isShotGun)
			%shellcount = 12;

		// Create each projectile and send it on its way 
		for(%shell=0; %shell<%shellcount; %shell++)
		{
			// Get the muzzle vector.  This is the dead straight aiming point of the gun
			%vector = %obj.getMuzzleVector(%slot);
			
			// Get our players velocity.  We must ensure that the players velocity is added 
			//   onto the projectile
			%objectVelocity = %obj.getVelocity();

			// Determine scaled projectile vector.  This is still in a straight line as 
			//   per the default example
			%vector1 = VectorScale(%vector, %projectile.muzzleVelocity);
			%vector2 = VectorScale(%objectVelocity, %projectile.velInheritFactor);
			%velocity = VectorAdd(%vector1,%vector2);

			// Determine our random x, y and z points in our spread circle and create 
			//   a spread matrix.
			%x = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
			%y = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
			%z = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
			%mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);
			
			// Alter our projectile vector with our spread matrix
			%velocity = MatrixMulVector(%mat, %velocity);

			
			// Create our projectile
			%p = new (%this.projectileType)()
			{
				dataBlock        = %projectile;
				initialVelocity  = %velocity;
				initialPosition  = %obj.getMuzzlePoint(%slot);
				sourceObject     = %obj;
				sourceSlot       = %slot;
				client           = %clientID;
			};
			MissionCleanup.add(%p);
		}

		return %p;
	}


