//-----------------------------------------------------------------------------
// Torque Game Engine
// 
// Copyright (c) 2001 GarageGames.Com
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// LRM weapon. This file contains all the items related to this weapon
// including explosions, ammo, the item and the weapon item image.
// These objects rely on the item & inventory support system defined
// in item.cs and inventory.cs
//-----------------------------------------------------------------------------


datablock AudioProfile(LARBFireSound)
{
   filename = "~/data/sound/missile.ogg";
   description = AudioDefault3d;
   preload = true;
};

datablock AudioProfile(LARBMountSound)
{
   filename = "~/data/sound/boltpull.ogg";
   description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(LARBExplosionSound)
{
   filename = "~/data/sound/bomb_explosion_4.ogg";
   description = AudioDefault3d;
   preload = true;
};


//-----------------------------------------------------------------------------
// Weapon fire emitter
//-----------------------------------------------------------------------------

datablock ParticleData(lrabFireParticle)
{
   textureName          = "~/data/specialfx/smoke1";

   dragCoeffiecient     = -25; // org 0
   gravityCoefficient   = 0.35;  // rises org -0.1
   inheritedVelFactor   = 0.3; // org 0.3

   lifetimeMS           = 1500;   // Time in ms -- org 250
   lifetimeVarianceMS   = 200;    // ...more or less -- org 10
   //particleDensity = 10; // new
   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "1 0.84 0.44 0.75"; // org 1 1 1 1
   colors[1]     = "0.9 0.6 0.21 1"; // org 0.7 0.7 0.7 0.66
   colors[2]     = "0.75 0.75 0.75 0.66"; // org 1 0 0 0.33
   colors[3]     = "0 0 0 0.75"; // new
   colors[4]     = "0 0 0 0"; // new

   sizes[0]      = 2; // org 0.2
   sizes[1]      = 3; // org 0.5
   sizes[2]      = 3.25; // org 1
   sizes[3]      = 3.5; // new
   sizes[4]      = 3.75; // new
   
   times[0]      = 0.0;
   times[1]      = 0.75;
   times[2]      = 0.85;
   times[3]      = 0.9;
   times[4]      = 1;
   
  
};



datablock ParticleEmitterData(lrabFireEmitter)
{
   ejectionPeriodMS = 10; // org 10
   periodVarianceMS = 0; // org 5

   ejectionVelocity = 4; // org 0.25
   velocityVariance = 0.25; // org 0.1

   thetaMin         = 0.0;
   thetaMax         = 180.0; // org 90

   particles = lrabFireParticle;
     
   
};


//-----------------------------------------------------------------------------
// Particle Effects
//-----------------------------------------------------------------------------

datablock ParticleData(LARBSmokeParticle)
{
   dragCoeffiecient     = 0.0;
   gravityCoefficient   = -0.1;   // rises slowly
   inheritedVelFactor   = 0.1;

   lifetimeMS           = 700;  // lasts 0.7 second
   lifetimeVarianceMS   = 50;   // ...more or less -- org 150

   textureName          = "~/data/specialfx/smoke1";

   useInvAlpha = true;
   spinRandomMin = -5.0;
   spinRandomMax = 5.0;

   colors[0]     = "1 0.8 0.4 1";
   colors[1]     = "1 1 1 1"; // new
   colors[2]     = "0.35 0.35 0.35 0.5";
   colors[3]     = "0 0 0 0"; // org 0 0 1 1

   sizes[0]      = 1;
   sizes[1]      = 1.5;
   sizes[2]      = 2;
   sizes[3]      = 2.5;

   times[0]      = 0.1;
   times[1]      = 0.33;
   times[2]      = 0.75;
   times[3]      = 1;
};

datablock ParticleEmitterData(LARBSmokeEmitter)
{
   ejectionPeriodMS = 15;
   periodVarianceMS = 2;

   ejectionVelocity = 0.25;
   velocityVariance = 0.10;

   thetaMin         = 0.0;
   thetaMax         = 90.0;  

   particles = LARBSmokeParticle;
};

//-----------------------------------------------------------------------------
// Projectile Explosion
//-----------------------------------------------------------------------------

datablock ParticleData(LARBExplosionParticle)
{
   dragCoefficient      = 0; // org 2
   gravityCoefficient   = 1; // org 0.2
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 1250; // org 750
   lifetimeVarianceMS   = 150;
   textureName          = "~/data/specialfx/smoke1";
   colors[0]     = "0.56 0.36 0.26 1.0";
   colors[1]     = "0.56 0.36 0.26 0.0";
   sizes[0]      = 10.0; // org 15
   sizes[1]      = 30.0;
};

datablock ParticleEmitterData(LARBExplosionEmitter)
{
   ejectionPeriodMS = 2000; // org 7
   periodVarianceMS = 350; // org 0
   ejectionVelocity = 5; // org 8
   velocityVariance = 2; // org 5
   ejectionOffset   = 0.0;
   thetaMin         = 0; // x rotation
   thetaMax         = 60; // org 60
   phiReferenceVel  = 0; // y rotation
   phiVariance      = 180; // org 180
   overrideAdvances = false;
   particles = "LARBExplosionParticle";
};

datablock ExplosionData(LARBExplosion)
{
   explosionShape = "~/data/shapes/weapons/lrabExpl.dts";
   //playSpeed = 10; // org 10
   //explosionScale = "0.1 0.1 0.1";
   //scale = "0.1 0.1 0.1";

   lifeTimeMS = 3000;
   particleEmitter = LARBExplosionEmitter;
   particleDensity = 100;
   particleRadius = 8;
   soundProfile = LARBExplosionSound;
   faceViewer     = true;
   explosionScale = "1 1 1";

   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "1.0 1.0 1.0";
   camShakeDuration = 2.0;
   camShakeRadius = 50.0;

   // Dynamic light
   lightStartRadius = 6;
   lightEndRadius = 3;
   lightStartColor = "0.5 0.5 0";
   lightEndColor = "0 0 0";
};


//-----------------------------------------------------------------------------
// Projectile Object

datablock ProjectileData(LARBProjectile)
{
   //projectileShapeName = "~/data/shapes/weapons/weapon_missile_projectile.dts";
   directDamage        = 75;
   damageRadius = 50;
   areaImpulse = 150;
   explosion           = LARBExplosion;
   particleEmitter     = LARBSmokeEmitter;

   muzzleVelocity      = 75;
   velInheritFactor    = 0.1;

   armingDelay         = 3;
   lifetime            = 5000;
   fadeDelay           = 5000;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = true;
   gravityMod          = 1.0;

   hasLight    = true;
   lightRadius = 20.0;
   lightColor  = "0.5 0.5 0";
};

function LARBProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal)
{
        if (%col.getType() & $TypeMasks::DamagableItemObjectType){
        
     %col.setDamageState(Destroyed);
     %col.schedule(999, "delete");
     }

   // Apply damage to the object all shape base objects
   if (%col.getType() & $TypeMasks::ShapeBaseObjectType)
      %col.damage(%obj,%pos,%this.directDamage,"LARBBullet");


   // Radius damage is a support scripts defined in radiusDamage.cs
   radiusDamage(%obj,%pos,%this.damageRadius,%this.radiusDamage,"LARBBullet",%this.areaImpulse);

}


//-----------------------------------------------------------------------------
// Ammo Item

datablock ItemData(LARBAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

	// Dynamic properties defined by the scripts
	pickUpName = "LARB ammo";
   maxInventory = 200;
};


//--------------------------------------------------------------------------
// LRM shell that's ejected during reload.

datablock DebrisData( LARBShell )
{
   //shapeFile = "~/data/shapes/weapons/shell.dts";
   lifetime = 3.0;
   minSpinSpeed = 300.0;
   maxSpinSpeed = 400.0;
   elasticity = 0.5;
   friction = 0.2;
   numBounces = 4;
   staticOnMaxBounce = true;
   snapOnMaxBounce = false;
   fade = true;
};


//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the LARBImage is used.

datablock ItemData(LARB)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/weapon.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

	// Dynamic properties defined by the scripts
	pickUpName = "a LARB";
	image = LARBImage;
};


//--------------------------------------------------------------------------
// LARB image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(LARBImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/weapon.dts";
   offset = "0 0 0";
   mountPoint = 1;
   emap = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = LARB;
   ammo = LARBAmmo;
   projectile = LARBProjectile;
   projectileType = Projectile;
   casing = LARBShell;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.9;
   stateSound[1] = LARBMountSound;
   stateSequence[1]                 = "Activated";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTimeoutValue[2]             = 0.9;
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does 
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 1.0;
   stateSound[3] = LARBFireSound;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateEmitter[3] = lrabFireEmitter; // -- new
   stateEmitterTime[3] = 0.33; // org 0 controls the flash emitter -- new

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 1.2;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "NoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 0.9;
   stateSound[6] = LARBMountSound;
   stateTransitionOnTimeout[6]      = "NoAmmo";
};

function baseLARB::onFire(%this, %obj, %slot,%ammo)
{
   %projectile = %this.projectile;
   %obj.decInventory(%this.ammo,%ammo);
  if(!isSpecialWeapon(%this.item))
   if(!%obj.aiPlayer)
      %obj.client.setAmmoAmountHud(%obj.getInventory(%this.ammo));

   if(!%obj.aiPlayer)
     %clientID = %obj.client;
   else
     %clientID = %obj;

   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));

   %p = new (%this.projectileType)() {
      dataBlock        = %projectile;
      initialVelocity  = %muzzleVelocity;
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      sourceSlot       = %slot;
      client           = %clientID;
   };
   MissionCleanup.add(%p);

   return %p;
}
