// ranges
// 180
// 420
// 1120

datablock AudioProfile(laserFireSound)
{
   filename = "~/data/sound/lasercannon.ogg";
   description = AudioDefault3d;
   preload = true;
};
datablock AudioProfile(altLaserFireSound)
{
   filename = "~/data/sound/brimstone.ogg";
   description = AudioDefault3d;
   preload = true;
};


//-----------------------------------------------------------------------------
// Weapon fire emitter
//-----------------------------------------------------------------------------

datablock ParticleData(laserFireParticle)
{
   textureName          = "~/data/specialfx/smoke1";

   dragCoeffiecient     = 0; // org 0
   gravityCoefficient   = 0;  // rises org -0.1
   inheritedVelFactor   = 0.3; // org 0.3

   lifetimeMS           = 350;   // Time in ms -- org 250
   lifetimeVarianceMS   = 10;    // ...more or less -- org 10
   //particleDensity = 10; // new
   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "1 0.84 0.44 0.66"; // org 1 1 1 1
   colors[1]     = "0.85 0 0 0.85"; // org 0.9 0.6 0.21 1
   colors[2]     = "0.5 0 0 0"; // org 1 0 0 0

   sizes[0]      = 2; // org 0.2
   sizes[1]      = 1.5; // org 1
   sizes[2]      = 0.25; // org 1

   times[0]      = 0.0;
   times[1]      = 0.2;
   times[2]      = 1.0;
   
  
};



datablock ParticleEmitterData(laserFireEmitter)
{
   ejectionPeriodMS = 10; // org 10
   periodVarianceMS = 0; // org 5

   ejectionVelocity = 3; // org 0.25
   velocityVariance = 0.25; // org 0.1

   thetaMin         = 0.0;
   thetaMax         = 180.0; // org 90

   particles = laserFireParticle;
   
    // Dynamic light
   lightStartRadius = 3.5; // org 5
   lightEndRadius = 0;
   lightStartColor = "0.5 0 0";  //org 1 1 1
   lightEndColor = "0.5 0 0";  //org 0 0 0
   
   
};


//-----------------------------------------------------------------------------
// Projectile Explosion
//-----------------------------------------------------------------------------

datablock ParticleData(laserExplosionParticle)
{
   dragCoefficient      = 2;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 750;
   lifetimeVarianceMS   = 150;
   textureName          = "~/data/specialfx/smoke2";
   colors[0]     = "1.00 0.00 0.00 1.0";
   colors[1]     = "0.26 0.00 0.00 0.0";
   sizes[0]      = 0.5;
   sizes[1]      = 1.0;
};

datablock ParticleEmitterData(laserExplosionEmitter)
{
   ejectionPeriodMS = 7;
   periodVarianceMS = 0;
   ejectionVelocity = 2;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "laserExplosionParticle";
};

datablock ExplosionData(laserExplosion)
{
   particleEmitter = laserExplosionEmitter;
   emitter = laserExplosionEmitter;
   particleDensity = 40;
   particleRadius = 1;

   faceViewer     = true;
   explosionShape = "~/data/fx/explosions/explosion02.dts";
   explosionScale = "1 1 1";

   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "5.0 5.0 5.0";
   camShakeDuration = 0.5;
   camShakeRadius = 100.0;

   // Dynamic light
   lightStartRadius = 6;
   lightEndRadius = 3;
   lightStartColor = "0.5 0 0";
   lightEndColor = "0 0 0";
};


//-----------------------------------------------------------------------------
// Projectile Object

datablock BeamData(laserProjectile)
{
   directDamage        = 5;
   radiusDamage        = 20;
   damageRadius        = 1.5;
   explosion           = laserExplosion;
   //particleEmitter     = laserEmitter;

   muzzleVelocity      = 1000;
   velInheritFactor    = 0.3;

   armingDelay         = 0;
   lifetime            = 500;
   fadeDelay           = 100;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = false;
   gravityMod = 0.0;

   hasLight    = true;
   lightRadius = 4.0;
   lightColor  = "0 0.5 0.5";
   beamStyle = 1;
   beamStartRadius = 0.4;
   beamEndRadius = 0.0;
   //beamlength not needed
   //beamLength = 1000;
   materialList = "~/data/shapes/weapons/red.dml";
   interval = 0;
};
datablock ItemData(laserAmmo)
{
   // Mission editor category
   category = "Ammo";
   className = "Ammo";
   shapeFile = "~/data/shapes/weapons/weapon.dts";
	pickUpName = "laserAmmo";
};
datablock ItemData(laser)
{
   // Mission editor category
   category = "Weapon";
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/weapon.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

	// Dynamic properties defined by the scripts
	pickUpName = "laser";
	image = laserImage;
   directUse = 1;
   laser = true;
};

datablock ShapeBaseImageData(laserImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/weapon.dts";
   emap = true;

   mountPoint = 0;
   offset = "0 0 0";
   correctMuzzleVector = true;
   className = "WeaponImage";

   projectile = laserProjectile;
   projectileType = Beam;

   usesEnergy = true; 
   useMountEnergy = false;
   minEnergy = 5;
   fireEnergy = 5;
   fireTimeout = 300;

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "Activate";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.5;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "Activate";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does 
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateSound[3]					      = "laserFireSound";
   stateTimeoutValue[3]             = 0.5;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateEnergyDrain[3]              = 50;
   stateEmitter[3]                  = laserFireEmitter; // new
   stateEmitterTime[3]              = 0.1; // org 0 controls the flash emitter -- new
   

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "Activate";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.08;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "Activate";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "Activate";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.0;
   stateTransitionOnTimeout[6]      = "Activate";
};

function baseLaser::onFire(%this, %obj, %slot)
{
   %projectile = %this.projectile;
   if(!%obj.aiPlayer)
     %clientID = %obj.client;
   else
     %clientID = %obj;
   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));

   %p = new (%this.projectileType)() {
      dataBlock        = %projectile;
      initialVelocity  = %muzzleVelocity;
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      sourceSlot       = %slot;
      client           = %clientID;
   };
   MissionCleanup.add(%p);
   return %p;
}
