//-----------------------------------------------------------------------------
// Game Beavers Sample App
// Copyright (C) Game Beavers, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Grenade Launcher weapon. This file contains all the items related to this weapon
// including explosions, ammo, the item and the weapon item image.
// These objects rely on the item & inventory support system defined
// in item.cs and inventory.cs
//-----------------------------------------------------------------------------

datablock AudioProfile(FragExpSound)
{
   filename    = "~/data/sound/big_explosion.wav";
   description = AudioDefault3d;
   preload = true;
};

//datablock AudioProfile(activateSound)
//{
 //  filename    = "~/data/sound/weapons/activate.WAV";
//   description = AudioDefault3d;
//   preload = true;
//};


//datablock AudioProfile(FragDeviceFireSound)
//{
//   filename    = "~/data/sound/weapons/fragdevice_launch.WAV";
//   description = AudioDefault3d;
//   preload = true;
//};

datablock AudioProfile(grenadelauncherFire)
{
  filename    = "~/data/sound/weapons/grenadelauncher_fire.wav";
  description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(grenadelauncherEmpty)
{
   filename    = "~/data/sound/weapons/grenadelauncher_noammo.wav";
   description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(grenadelauncherReload)
{
   filename    = "~/data/sound/weapons/grenadelauncher_reload.wav";
   description = AudioClosest3d;
   preload = true;
};

//datablock AudioProfile(grenadelauncherActivate)
//{
//   filename    = "~/data/sound/weapons/grenadelauncher_switch.wav";
//   description = AudioClosest3d;
//   preload = true;
//};

//-----------------------------------------------------------------------------


datablock ExplosionData(BeastSpecialExplosion)
{

   explosionShape = "~/data/shapes/weapons/effect_plasma_explosion.dts";
   playSpeed = 15.0;
   explosionScale = "0.1 0.1 0.1";
   scale = "0.1 0.1 0.1";

   lifeTimeMS = 1500;
   sound = FragExpSound;
   //particleEmitter = pipeBombExplosionEmitter;
     
   //subExplosion[0] = pipebombExplosion;
   delayMS = 0;
   
   particleDensity = 80;
   particleRadius = 1;

   faceViewer     = true;

   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "1.0 1.0 1.0";
   camShakeDuration = 0.5;
   camShakeRadius = 40.0;

   // Dynamic light
   lightStartRadius = 6;
   lightEndRadius = 3;
   lightStartColor = "0.5 0.5 0";
  lightEndColor = "0 0 0";
};



//-----------------------------------------------------------------------------
// Projectile Object

datablock ProjectileData(BeastSpecialProjectile)
{
   projectileShapeName = "~/data/shapes/weapons/grnadelauncher/projectile.dts";

   directDamage        = 60;
   radiusDamage        = 40;
   damageRadius        = 30;

   areaImpulse         = 2240;

   explosion           = BeastSpecialExplosion;

   //particleEmitter     = PipeBombCaseEmitter;

   muzzleVelocity      = 60;
   velInheritFactor    = 0.3;

   armingDelay         = 0;
   lifetime            = 15000;
   fadeDelay           = 750;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = true;
   gravityMod = 1;


};

function BeastSpecialProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal)
{
   //alxPlay("FragExpSound",%pos); //play rifle fire sound
   //playAudio("FragExpSound");

   // Apply damage to the object all shape base objects
   if (%col.getType() & $TypeMasks::ShapeBaseObjectType)
      %col.damage(%obj,%pos,%this.directDamage,"Grenade launcher");

   // Radius damage is a support scripts defined in radiusDamage.cs
   // Push the contact point away from the contact surface slightly
   // along the contact normal to derive the explosion center. -dbs
   radiusDamage
     (%obj, VectorAdd(%pos, VectorScale(%normal, 0.01)),
      %this.damageRadius,%this.radiusDamage,"Radius", %this.areaImpulse);

}


//-----------------------------------------------------------------------------
// Ammo Item

datablock ItemData(BeastSpecialAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/weapons/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

	// Dynamic properties defined by the scripts
   pickUpName = "grenade launcher ammo";
   maxInventory = 8;
};


//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the CrossbowImage is used.

datablock ItemData(BeastSpecial)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   //shapeFile = For now we borrow the laser gun
   shapeFile = "~/data/shapes/weapons/ac.dts";
   mass = 70;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

   gravity = 0;

   static = false;
   rotate = true;

	// Dynamic properties defined by the scripts
	pickUpName = "Grenade Launcher";
   displayName = "Grenade Launcher";
	image = BeastSpecialImage;
};


//--------------------------------------------------------------------------
// Crossbow image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(BeastSpecialImage)
{
   // Basic Item properties
   //shapeFile = For now we borrow the laser gun
   shapeFile = "~/data/shapes/weapons/ac.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0";
   eyeOffset = "0.1 0.2 -0.55";

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = false;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";
      usesEnergy                       = true;
   // Projectile && Ammo.
   item = BeastSpecial;
   ammo = BeastSpecialAmmo;
   projectile = BeastSpecialProjectile;
   projectileType = Projectile;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.6;
   stateSequence[1]                 = "Activate";
   //stateSound[1]                    = activateSound;
   
   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";
      stateEnergyDrain[3]              = 70;
   // Fire the weapon. Calls the fire script which does 
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Ready";
   stateTimeoutValue[3]             = 0.8;
   stateFire[3]                     = true;
   //stateRecoil[3]                   = HeavyRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Reload";
   stateScript[3]                   = "onFire";
   stateSound[3]                    = grenadelauncherFire;

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.8;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateSound[4]                    = grenadelauncherReload;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "NoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";
   //stateScript[5]                   = "noAmmo";
   
   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.0;
   stateTransitionOnTimeout[6]      = "NoAmmo";
   stateSound[6]                    = grenadelauncherEmpty;
   stateSequence[6]                 = "Fire";
   //stateScript[6]                   = "noAmmo";


};


//-----------------------------------------------------------------------------

function BeastSpecialImage::onFire(%this, %obj, %slot)
{
   %projectile = %this.projectile;

   // Decrement inventory ammo. The image's ammo state is update
   // automatically by the ammo inventory hooks.
   %obj.decInventory(%this.ammo,1);

   //%currentAmmo = %obj.getInventory(%this.ammo);
   //%obj.client.setAmmoAmountHud(%currentAmmo);

   // Determin initial projectile velocity based on the
   // gun's muzzle point and the object's current velocity
   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));

   // Create the projectile object
   %p = new (%this.projectileType)() {
      dataBlock        = %projectile;
      initialVelocity  = %muzzleVelocity;
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      sourceSlot       = %slot;
      client           = %obj.client;
   };
   MissionCleanup.add(%p);
   return %p;
}


