//------------------------------------------------------------------------------
// IRC (MAIN) Script v1.0.2
//------------------------------------------------------------------------------
// Contains all major functions to initialize IRC, connect to a server, and
// various functions used to send messages, or access/parse data
//------------------------------------------------------------------------------
// Original Scripts created or modified by Harold "LabRat" Brown
//
// By Sebastien Bourgon
// Last Updated February 26th, 2003
//------------------------------------------------------------------------------

$CHANNEL_STATUS = "STATUS";
$VERSION_STRING = "Torque IRC Chat 1.3.0";

$IRCClient::LastChannel = 0;
$IRCClient::AuthServer = "mgo.maxgaming.net:80";
$IRCClient::AuthServerPath = "/game/";
$IRCClient::Ingame = true;
$IRCClient::ChannelLimit = 8;
$IRCClient::path = "irc/";
$IRCClient::initialized = 0;
$IRCClient::Nick= $pref::Player::Name;
$IRCClient::User= "Torque Torque Torque Torque";
$IRCClient::Chan= "#DarkHorizons";
$IRCClient::Key = "betaonly";
$IRCClient::retries = 0;
$IRCClient::serverAttempt = 0;
$IRCClient::silentList = true;
$IRCClient::server = "irc.mgonetwork.com";
$IRCClient::port = ":7778";
$IRCClient::Status = 0;
$IRCClient::currentChannel = 0;
$IRCClient::State = "IDIRC_DISCONNECTED";
$IRCClient::loginStage = 0;

//------------ IRC Clients Initialization --------------------------------------
function IRCClient::init()
{

   $IRCClient::tcp = new TCPObject(IRCTCP);
   $IRCClient::people = new SimGroup(IRCPeople);
   $IRCClient::friends = new SimGroup(IRCPeople);
   $IRCClient::ignores = new SimGroup(IRCPeople);
   $IRCClient::channels = new SimGroup(IRCChannels);
   $IRCClient::people.add(new SimObject()
                       {
                         real = "";
                         identity = "";                         
                         nick = "";
                         flags = 0;
                         ping = 0;
                         ref = 1;
                       });
   %cl = new GuiTextListCtrl(%params)
	{
            profile = "GuiShellTextListProfile";
            horizSizing = "width";
            vertSizing = "bottom";
            position = "0 0";
            extent = "145 36";
            minExtent = "8 8";
            visible = "1";
            helpTag = "0";
            enumerate = "0";
            resizeCell = "1";
            columns = "0 120";
            fitParentWidth = "1";
            clipColumnText = "0";
    };
   $IRCClient::channels.add(new MessageVector("STATUS")
                        {
                           topic = "IRC Status";
                           key = "";
                           flags = 0;
                           personLimit = 0;
                           plist = %cl;
                           private = false;
                        });
                        
   $IRCClient::Status = $IRCClient::channels.getObject(0);
   mMM_ChanList.clear();
   mMM_ChanList.addRow(0,"STATUS");
   IRCUI::addChannel("STATUS",0);
   mMM_ChanList.setSelectedRow(0);
   $IRCClient::initialized = 1;
}

//------------ Server Connection Functions -------------------------------------
// ---- Connect To Server ------------------------------------------------------

function IRCClient::tcpConnect()
{
   IRCClient::tcpDisconnect();
   $INGAMEIRC = true;
   $IRCClient::Status.pushBackLine("Connecting to server",0);
   $IRCClient::tcp.connect($IRCClient::server @ $IRCClient::port);
}

// ----- Disconnect ------------------------------------------------------------
function IRCClient::tcpDisconnect()
{
   $INGAMEIRC = false;
   $IRCClient::state = IDIRC_DISCONNECTED;
   $IRCClient::tcp.disconnect();
   IRCTCP.disconnect();
   IRCClient::fileSave();
   IRCClient::tcpReset();
}


// ----- Reset -----------------------------------------------------------------
function IRCClient::tcpReset()
{
   // Delete all the channels except status
   while ($IRCClient::channels.getCount() > 1)
   {
     %channel = $IRCClient::channels.getObject(1);
     $IRCClient::channels.remove(%channel);
     %channel.delete();
   }
   while ($IRCClient::people.getCount() > 1)
   {
     %person = $IRCClient::people.getObject(1);
     $IRCClient::people.remove(%person);
     %person.delete();
   }
   $IRCClient::Status.clear();
   irc.detach();
   irc.attach($IRCClient::Status);
   IRCClient::guiRebuildList();
   $IRCClient::currentChannel = $IRCClient::channels.getObject(0);
   $IRCClient::loaded = 0;
}

//-------------------We're Connected--------------------------------------------
function IRCTCP::onConnected(%this)
{
   IRCClient::send("NICK " @ $IRCClient::Nick SPC %msg);
   IRCClient::send("USER " @ $IRCClient::User SPC %msg);
}

function IRCTCP::onDisconnect(%this)
{
   if ($IRCClient::loginStage != 0)
   {
      disconnect(); //Disconnect from game.  You were dropped (Or KILLed)
      $IRCClient::state = "IDIRC_DISCONNECTED";
      MessageBoxOk("Disconnection","You have been disconnected","Canvas.pushDialog(menuMultiLogin);");
   }
}

//-------- DNS Failed ----------------------------------------------------------

function IRCTCP::onDNSFailed(%this)
{
   $IRCClient::state = "IDIRC_DISCONNECTED";
   MessageBoxOk("DNS Failed","The DNS Failed. Could not locate server.","Canvas.pushDialog(menuMultiLogin);");
}

//-------- Connection Failed ---------------------------------------------------
function IRCTCP::onConnectFailed(%this)
{
   $IRCClient::state = "IDIRC_DISCONNECTED";
   MessageBoxOk("Connection Failed","Can not connect to server","Canvas.pushDialog(menuMultiLogin);");
}

// ----- Get Server Response ---------------------------------------------------
function IRCTCP::onLine(%this,%line)
{
   IRCClient::tcpProcessLine(%line);
}

// ----- Messaging Functions ---------------------------------------------------
//------- Send Message ---------------------------------------------------------
function IRCClient::send(%message)
{
   $IRCClient::tcp.send(%message @ "\r\n");
}

//------ Test Message for colors------------------------------------------------
function IRCClient::testcolors()
{
  $IRCClient::channels.getObject(0).pushBackLine("  \c0Test \c1Test \c2Test \c3Test \c4Test \c5Test \c6Test \c7Test \c8Test \c9Test",0);
}

// ----- Display Message -------------------------------------------------------
function IRCClient::newMessage(%channel,%message)
{
   if (%channel == $IRCClient::channels.getObject(0) || %channel $= "")
   {
      if ($IRCClient::channels.getObject(0).getNumLines() > 256)
      {
         $IRCClient::channels.getObject(0).popFrontLine();
      }
      $IRCClient::channels.getObject(0).pushBackLine(%message,0);
   }
   else
   {
      if (%channel.getNumLines() > 256)
      {
         %channel.popFrontLine();
      }
      %channel.pushBackLine(%message,0);
   }
}

// ----- Channel and 'Person' Creation or Search -------------------------------


//------ Find Person -----------------------------------------------------------
function IRCClient::findPerson(%nick)
{
   if (getSubStr(%nick,0,1) $= "@" )
   {
      %nick = getSubStr(%nick,1,strlen(%nick)-1);
   }
   if (getSubStr(%nick,0,1) $= "+")
   {
      %nick = getSubStr(%nick,1,strlen(%nick)-1);
   }
   for (%i = 0; %i < $IRCClient::people.getCount(); %i++)
   {
      %person = $IRCClient::people.getObject(%i);
      if (%person.displayName $= %nick)
        return %person;
   }

   return false;
}

//------------------------------------------------------------------------------
function IRCClient::createPerson(%nick,%create)
{
   // typical name
   // Yakuza|!yakuzasama@pool032-max7.ds23-ca-us.dialup.earthlink.net
   if (getSubStr(%nick,0,1) $= "@" )
   {
      %nick = getSubStr(%nick,1,strlen(%nick)-1);
   }
   if (getSubStr(%nick,0,1) $= "+")
   {
      %nick = getSubStr(%nick,1,strlen(%nick)-1);
   }
   if (%create)
   {
       if(%nick $= $IRCClient::Nick)
       {
           %p=$IRCClient::people.getObject(0);
      	   %p.displayName=%nick;
  	   }
       else
       {
          %p = new SimObject()
          {
             real = "";
             identity = "";
             nick = "";
             flags = 0;
             ping = 0;
             ref = 0;
             displayName = %nick;
          };
          $IRCClient::people.add(%p);
       }
   }
   return %p;
}

// ----- Find Channel (or Create) ----------------------------------------------
function IRCClient::findChannel(%name,%create)
{
   if(%name $= "")
   {
      return $IRCClient::currentChannel;
   }
   for (%i = 0; %i < $IRCClient::channels.getCount(); %i++)
   {
      %c = $IRCClient::channels.getObject(%i);
      if (%c.getName() $= %name)
      {
         return %c;
      }
   }
   if (%create)
   {
      %topic = "";
      %flags = $CHANNEL_NEW;
      %private = false;
      for (%i = 0; %i < $IRCClient::numChannels; %i++)
         if (%name $= $IRCClient::channelNames[%i])
         {
             %topic = $IRCClient::channelTopics[%i];
             break;
         }
         %l = strlen(%name);
         %cl = new GuiTextListCtrl(%params)
		 	{
            profile = "GuiShellTextListProfile";
            horizSizing = "width";
            vertSizing = "bottom";
            position = "0 0";
            extent = "145 36";
            minExtent = "8 8";
            visible = "1";
            helpTag = "0";
            enumerate = "0";
            resizeCell = "1";
            columns = "0 120";
            fitParentWidth = "1";
            clipColumnText = "0";
            };
          %c = new MessageVector(%name)
            {
             topic = %topic;
             key = $IRCClient::key;
             flags = $CHANNEL_NEW;
             personLimit = 0;
             private = %private;
             plist = %cl;
             name = %name;
            };
         $IRCClient::channels.add(%c);
      return %c;
   }
}

//-------- Dispatch Functions --------------------------------------------------
//---------Process Line --------------------------------------------------------
function IRCClient::tcpProcessLine(%line)
{
   // RFC_1459: Message Packet format
   //
   // <message>  ::= [':' <prefix> <SPACE> ] <command> <params> <crlf>
   // <prefix>   ::= <servername> | <nick> [ '!' <user> ] [ '@' <host> ]
   // <command>  ::= <letter> { <letter> } | <number> <number> <number>
   // <SPACE>    ::= ' ' { ' ' }
   // <params>   ::= <SPACE> [ ':' <trailing> | <middle> <params> ]
   //
   // <middle>   ::= <Any *non-empty* sequence of octets not including SPACE
   //                or NUL or CR or LF, the first of which may not be ':'>
   // <trailing> ::= <Any, possibly *empty*, sequence of octets not including
   //                   NUL or CR or LF>
   // <crlf>     ::= CR LF

   %src = %line;

   if (strlen(%src))
   {
      // check for prefix
      if (getSubStr(%src,0,1) $= ":")
        %src = nextToken(getSubStr(%src,1,strlen(%src)-1),prefix," ");

     // this is the command
     %src = nextToken(%src,command," :");

     // followed by its params
     %src = nextToken(%src,params,"");
     
     //Ingame Channel Check
     if ($INGAME.Playing = true)
     {
        if (%command $= "PRIVMSG")
        {
           %msg = nextToken(%params,channel," ");
           if (%channel $= $INGAME.Channel)
           {
              IRCClient::ingameRecieve(%prefix,%msg);
              return;
           }
        }
     }
     
     if (!IRCClient::dispatchInput(%prefix,%command,%params))
     {
     }
   }
}
//---------Output/User Dispatch --------------------------------

// Dispatch will check to see if its a valid command or not
// Commands sent to IRCClient::ignoreCommand() are purposefully
// Not supported by the IRC Client. They are either not needed
// or should not be used outside a "proper" irc client. (Like Admin commands)
// Supported Commands are listed first. Unsupported commands listed afterwards.
// The reason for the specific listing is due to an IRC Server or Services various
// Proprietary commands that deviate from the IRC RPL standard. Generally these are Oper only
// and not a worry.

// The following TORQUE Specific Aliases have been added
// IP - To get a users Ip, requires being on the users Friends list for it allow it
// CONNECT - To connect to an on-going game via IRC.
// OP/DEOP - Shortform for /mode #channel +o|-o Nickname
// VOICE/DEVOICE - Shortform for /mode #channel +o|-o Nickname
// UMODE - Shortform for /mode MyNick +|-mode
// CMODE - Shortform for /mode #channel +|- modes
     
function IRCClient::dispatchOutput(%message,%to)
{
   // Check to see if its a command or a normal message
   if (getSubStr(%message,0,1) $= "/")
   {
      %buffer = getSubStr(%message,1,strlen(%message)-1);
      %params = nextToken(%buffer,command," ");

     switch$(%command)
     {
       case "NOTICE":
         IRCClient::sendNotice(%params);
       case "ME":
         IRCClient::sendAction(%params);
       case "ACTION":
         IRCClient::sendAction(%params);
       case "KICK":
         IRCClient::sendKick(%params);
       case "JOIN":
         IRCClient::sendJoin(%params);
       case "PART":
         IRCClient::sendPart(%params);
       case "QUIT":
         IRCClient::sendQuit(%params);
       case "MODE":
         IRCClient::sendDefault(%params);
       case "MSG":
         IRCClient::sendMsg(%params);
       case "BAN":    // Use: /ban add|list|del username
         IRCClient::sendBan(%params);
       case "FRIENDS":// Use: /friends add|list|del|msg username
         IRCClient::sendFriends(%params);
       case "F":
         IRCClient::sendFriends(%params);
       case "IGNORE": // Use: /ignore add|list|del username
         IRCClient::sendIgnore(%params);
       case "OP":     // Use: /op Nickname
         IRCClient::sendOp(%params,1);
       case "DEOP":   // Use: /deop Nickname
         IRCClient::sendOp(%params,2);
       case "VOICE":  // Use: /voice Nickname
         IRCClient::sendVoice(%params,1);
       case "DEVOICE":// Use: /devoice Nickname
         IRCClient::sendVoice(%params,2);
       case "IP":     // Use: /ip nickname
         IRCClient::sendIP(%params);
       case "UMODE":  // Use: /umode +|- mode
         IRCClient::sendUMode(%params);
       case "CMODE":  // Use: /cmode +|- mode
         IRCClient::sendCMode(%params);
       case "LIST":
         IRCClient::sendDefault(%message);
       case "INVITE":
         IRCClient::sendDefault(%message);
       case "CONNECT":// Use: /connect TheAce [GamePassword]
         IRCClient::sendConnect(%params);
       case "ADMIN":
         IRCClient::sendDefault(%message);
       case "HELP":
         IRCClient::sendHelp(%params);
       case "TOPIC": // Use: /topic New Topic
         IRCClient::sendTopic(%params);
       case "WHOIS":
         IRCClient::sendWhois(%params);
       case "INFO":
         IRCClient::sendWhois(%params);
       case "ISON":
         IRCClient::sendDefault(%message);
       case "MOTD":
         IRCClient::sendDefault(%message);
       case "TIME":
         IRCClient::sendDefault(%message);

    // UNSUPPORTED OR IGNORED COMMANDS START HERE
       case "NICK":
         IRCClient::ignoreCommand(%command);
       case "VERSION":
         IRCClient::ignoreCommand(%command);
       case "STATS":
         IRCClient::ignoreCommand(%command);
       case "LINKS":
         IRCClient::ignoreCommand(%command);
       case "TRACE":
         IRCClient::ignoreCommand(%command);
       case "INFO":
         IRCClient::ignoreCommand(%command);
       case "WHOWAS":
         IRCClient::ignoreCommand(%command);
       case "WHO":
         IRCClient::ignoreCommand(%command);
       case "KILL":
         IRCClient::ignoreCommand(%command);
       case "SUMMON":
         IRCClient::ignoreCommand(%command);
       case "RESTART":
         IRCClient::ignoreCommand(%command);
       case "REHASH":
         IRCClient::ignoreCommand(%command);
       case "USERS":
         IRCClient::ignoreCommand(%command);
       case "WALLOPS":
         IRCClient::ignoreCommand(%command);
       case "SERVER":
         IRCClient::ignoreCommand(%command);
       case "USERHOST":
         IRCClient::ignoreCommand(%command);
       case "OPER":
         IRCClient::ignoreCommand(%command);
         
     // DEFAULT. To make up for the possibility of missed NEEDED commands
       default:
         IRCClient::send(getSubStr(%message,1,strlen(%message)-1));
     }
   }
   else
   if (strcmp($IRCClient::currentChannel.getName(),$CHANNEL_STATUS))
   {
     IRCClient::send("PRIVMSG " @ $IRCClient::currentChannel.getName() @ " :" @ %message);
     IRCClient::newMessage($IRCClient::currentChannel,"<" @ $IRCClient::Nick @ "> " @ %message);
   }
}

//------ Server Input ----------------------------------------------------------

// Dispatch will sort out incomming information
// There are 4 main types of data
// WORD 'coded': The most common. Messages, Notices, Joins, Parts
// ALL WORD 'code' data is parsed.
// Specific functions are found in irc-chat.cs

// Server Data: From 0-199
// In use currently for server to client is only 1 through 6. They all use the default parser.

// Command Responses: From 200-399
// Roughly half the command responses are parsed. This is due to the fact that this is
// a restricted IRC client and some commands aren't allowed, so thier responses wont show up.
// All command responses are listed so as to permit easy addition later on if its wanted.
// Specific Functions found in irc-info.cs

// Error Respones: 400+
// ALL errors are parsed for obvious reasons.
// They go through a general error specific parser. The more important errors
// use a notify system as to alert the user right away. Example would be Nickname in Use (for when connecting)
// or Cannot Join Channel.
// Specific Functions found in irc-error.cs

function IRCClient::dispatchInput(%prefix,%command,%params)
{
   nextToken(%prefix,nick,"!");
   // Check Ignore List
   for (%i = 0; %i < $IRCClient::ignores.getCount(); %i++)
   {
      %ignore = $IRCClient::ignores.getObject(%i);
      if (%ignore.nick $= %nick)
      {
         return;
      }
   }
   if (%command == 0)
   {
     switch$(%command)
     {
        case "PING":
           IRCClient::onPing(%prefix,%params);
        case "PONG":
           IRCClient::onPong(%prefix,%params);
        case "PRIVMSG":
           IRCClient::onPrivMsg(%prefix,%params);
        case "JOIN":
           IRCClient::onJoin(%prefix,%params);
        case "NICK":
           IRCClient::onNick(%prefix,%params);
        case "QUIT":
           IRCClient::onQuit(%prefix,%params);
        case "PART":
           IRCClient::onPart(%prefix,%params);
        case "KICK":
           IRCClient::onKick(%prefix,%params);
         case "MODE":
           IRCClient::onMode(%prefix,%params);
        case "NOTICE":
           IRCClient::onNotice(%prefix,%params);
        case "INVITE":
           IRCClient::onInvite(%prefix,%params);
        case "TOPIC":
           IRCClient::onTopic(%prefix,%params);
     }
   }
   else
   {
   //--------------------------------------------------------------
   // Client Server Connection messages (001 - 099)
      switch$(%command)
      {
      case "001":
          $IRCClient::state = "IDIRC_CONNECTED";
          IRCClient::friends(1);
          IRCClient::infoVerifyName(%prefix,%params);
          IRCClient::send("JOIN " @ $IRCClient::Chan SPC $IRCClient::Key);
          IRCClient::send("NICKSERV IDENTIFY " @ $IRCClient::Pass);
          $IRCClient::state = "IDIRC_CONNECTED_NICK_SENT";
          CloseMessageEvent();
          IRCClient::infoDefault(%prefix,%params);
      case "002":
          IRCClient::infoDefault(%prefix,%params);
      case "003":
          IRCClient::infoDefault(%prefix,%params);
      case "004":
          //IRCClient::infoDefault(%prefix,%params);
      case "005":
          //IRCClient::infoDefault(%prefix,%params);
      }
   //---------------------------------------------------------------------------
   //  Command Responses (200-399)
   if (%command >= 200 && %command < 300)
      {
         switch$(%command)
         {
             case "221": // User Mode Is
                IRCClient::infoUMode(%prefix,%params);
             case "251": // Lusers Client Count
                IRCClient::infoLusers(%prefix,%params,1);
             case "252": // Lusers Oper Count
                IRCClient::infoLusers(%prefix,%params,2);
             case "253": // Lusers Unknown Count
                IRCClient::infoLusers(%prefix,%params,3);
             case "254": // Lusers Channels
                IRCClient::infoLusers(%prefix,%params,4);
             case "255": // Lusers Local Count
                IRCClient::infoLusers(%prefix,%params,5);
             case "256": // Admin Info
                IRCClient::infoAdmin(%prefix,%params,1);
             case "257": // Admin Loc1
                IRCClient::infoAdmin(%prefix,%params,2);
             case "258": // Admin Loc2
                IRCClient::infoAdmin(%prefix,%params,3);
             case "259": // Admin Email
                IRCClient::infoAdmin(%prefix,%params,4);
             case "265": // Lusers Local Now and Max
                 IRCClient::infoLusers(%prefix,%params,6);
             case "266": // Lusers Global Now and Max
                 IRCClient::infoLusers(%prefix,%params,7);
         //    case "200": // Tracelink
         //    case "201": // Trace connection
         //    case "202": // Trace Handshake
         //    case "203": // Trace Unknown
         //    case "204": // Trace Operator
         //    case "205": // Trace User
         //    case "206": // Trace Server
         //    case "207": // Trace Service
         //    case "208": // Trace New Type
         //    case "209": // Trace Class
         //    case "210": // Trace Reconnect
         //    case "211": // Stats Link Info
         //    case "212": // Stats Commands
         //    case "213": // Stats C-Line
         //    case "214": // Stats N-Line
         //    case "215": // Stats I-Line
         //    case "216": // Stats K-Line
         //    case "218": // Stats Y-Line
         //    case "219": // End of Stats
         //    case "241": // Stats L-Lines
         //    case "242": // Stats Uptime
         //    case "243": // Stats O-Line
         //    case "244": // Stats H-Line
         //    case "261": // Trace Log
         //    case "262": // Trace End
         }
      }
      //------------------------------------------------------------------------
      //  Command Responses (200-399)
      if (%command >= 300 && %command < 400)
      {
         switch$(%command)
         {
            case "303": // IsOn
               IRCClient::infoNotify(%prefix,%params);
            case "304": // Password accepted
               IRCClient::infoPassword(%prefix,%params);
            case "307": // Whois Registered
               IRCClient::infoWhois(%prefix,%params,7);
            case "310": // Whois Help Op
               IRCClient::infoWhois(%prefix,%params,10);
            case "311": // Whois User
               IRCClient::infoWhois(%prefix,%params,11);
            case "312": // Whois Server
               IRCClient::infoWhois(%prefix,%params,12);
            case "313": // Whois Op
               IRCClient::infoWhois(%prefix,%params,13);
            case "317": // Whois Idle
               IRCClient::infoWhois(%prefix,%params,17);
            case "318": // Whois End of Whois
               IRCClient::infoWhois(%prefix,%params,18);
            case "319": // Whois Channels
               IRCClient::infoWhois(%prefix,%params,19);
            case "320": // Whois Services Op
               IRCClient::infoWhois(%prefix,%params,20);
            case "321": // List Start
               IRCClient::infoList(%prefix,%params,1);
            case "322": // List Info
               IRCClient::infoList(%prefix,%params,2);
            case "323": // List End
               IRCClient::infoList(%prefix,%params,3);
            case "324": // Channel Modes
               IRCClient::infoMode(%prefix,%params);
            case "331": // No Topic
               IRCClient::infoTopic(%prefix,%params,1);
            case "332": // Topic
               IRCClient::infoTopic(%prefix,%params,2);
            case "341": // Invite
               IRCClient::infoInvite(%prefix,%params);
            case "353": // Name reply
               IRCClient::infoNames(%prefix,%params,1);
            case "366": // Names End
               IRCClient::infoNames(%prefix,%params,2);
            case "367": // Ban List
               IRCClient::infoBanList(%prefix,%params,1);
            case "368": // End of Ban List
               IRCClient::infoBanList(%prefix,%params,2);
            case "372": // MOTD Info
               IRCClient::infoMOTD(%prefix,%params,2);
            case "375": // MOTD Start
               IRCClient::infoMOTD(%prefix,%params,1);
            case "376": // MOTD End
               IRCClient::infoMOTD(%prefix,%params,3);
            case "391": // Time
               IRCClient::infoTime(%prefix,%params);
         //   case "301": // Away message
         //   case "302": // User Host
         //   case "305": // Un Away
         //   case "306": // Now Away
         //   case "314": // WhoWas User
         //   case "315": // Who End
         //   case "342": // Summon
         //   case "351": // Version
         //   case "352": // Who Reply
         //   case "364": // Links
         //   case "365": // Links End
         //   case "369": // WhoWas End of Whowas
         //   case "371": // Info
         //   case "381": // You are now Oper
         //   case "382": // Rehash
         //   case "374": // Info Emd
         }
      }
      if (%command >= 400)
      {
         IRCClient::onError(%prefix,%command,%params);
      }
   }
   return true;
}

//------- Load Friends/Ignore settings -----------------------------------------
function IRCClient::fileLoad()
{
   %file = new FileObject();
   %fullPath = $IRCClient::path @ "irc.data";
   if(!%file.openForRead(%fullPath))
   {
      echo("File not found");
      // Since a file doesn't exist. Make a new empty one.
      %file.openForWrite(%fullPath);
      %file.writeLine(" ");
      %file.close();
      %file.delete();
   }
   else
   {
      echo("File Found");
      // File does exist, so lets load everything.
      // Lines are saved as "ignore==nickname or friend==nickname"
      while(!%file.isEOF())
      {
         %line = %file.readLine();
         %nickname = nextToken(%line,type,"==");
         if (%type $= "friend")
         {
            $IRCClient::friends.add(new SimObject() { nick = %nickname; num = $IRCClient::friends.getCount(); status="offline"; });
         }
         if (%type $= "ignore")
         {
            $IRCClient::ignores.add(new SimObject() { nick = %nickname; });
         }
      }
      %file.close();  //Close the I/O Stream
      %file.delete(); //Delete the file pointer
   }
}

function IRCClient::fileSave()
{
   %file = new FileObject();
   %fullPath = $IRCClient::path @ "irc.data";
   %file.openForWrite(%fullPath);

   for (%i = 0; %i < $IRCClient::friends.getCount(); %i++)
   {
     %name = $IRCClient::friends.getObject(%i);
     %name = %name.nick;
     %file.writeLine("friend==" @ %name);
   }
   for (%i = 0; %i < $IRCClient::ignores.getCount(); %i++)
   {
     %name = $IRCClient::ignores.getObject(%i);
     %name = %name.nick;
     %file.writeLine("ignore==" @ %name);
   }
   %file.close();
   %file.delete();
}

//------- Friends Notify Function
function IRCClient::friends(%toggle)
{
  if (%toggle == 1)
  {
     IRCClient::sendFriends("list");
  }
  if (%toggle == 2)
  {
    echo("Listing");
    $IRCClient::InternalCount = 0;
    for (%i = 0; %i < $IRCClient::friends.getCount(); %i++)
    {
      %friend = $IRCClient::friends.getObject(%i);
      if (%friend.status $= "Online")
      {
        $IRCClient::InternalCount++;
        IRCClient::send("PRIVMSG " @ %friend.nick @ " :\x01FRIEND\x01 query");
      }
    }
  }
}

function IRCClient::friendsReply(%nick,%msg)
{
  for (%i = 0; %i < $IRCClient::friends.getCount(); %i++)
  {
     %friend = $IRCClient::friends.getObject(%i);
     if (%friend.nick $= "nick")
     {
        $IRCClient::InternalCount--;
        %friend.status = %msg;
     }
  }
  if ($IRCClient::InternalCount == 0)
  {
    IRCClient::friendsList();
  }
}

function IRCClient::friendsList()
{
   IRCClient::newMessage($IRCClient::currentChannel, "\c7Your Friends list:");
   for (%i = 0; %i < $IRCClient::friends.getCount(); %i++)
   {
     %f = $IRCClient::friends.getObject(%i);
     %msg = "\c7  " @ %f.num @ ": " @ %f.nick @ " is " @ %f.status @ ".";
     IRCClient::newMessage($IRCClient::currentChannel, %msg);
   }
}
function IRCClient::friendsQuery(%nick)
{
  for (%i = 0; %i < $IRCClient::friends.getCount(); %i++)
  {
     %friend = $IRCClient::friends.getObject(%i);
     if (%friend.nick $= %nick)
     {
        if ($IRCClient::ingame)
        {
          IRCClient::send("PRIVMSG " @ %friend.nick @ " :\x01FRIEND\x01 reply Playing a game ");
        }
        else
        {
          IRCClient::send("PRIVMSG " @ %friend.nick @ " :\x01FRIEND\x01 reply Talking in " @ $IRCClient::Current);
        }
     }
  }
}
//------- Function Logged on time from Seconds to Days/Hours/Min ---------------
function IRCClient::logOnTime(%time)
{
  %seconds = %time % 60;
  %time    = (%time - %seconds) / 60;
  %minutes = %time % 60;
  %time    = (%time - %minutes) / 60;
  %hours   = %time % 60;
  %time    = (%time - %hours) / 24;
  %days    = %time % 365;
  %time    = (%time - %days) / 365;
  %years   = %time;

  //Deal with leap years
  %year2 = %years;
  %year2 -= 2;
  if (%year2 % 4 == 0)
  {
    %leapyear = 1;
    %year2 -= 4;
  }
  
  while (%year2 > 3)
  {
    %days--;
    %year2 -= 4;
  }
  %years += 1970;
  //Deal with months
  %month[0] = 0;
  %month[1] = 31;
  %month[2] = 59+%leapyear;
  %month[3] = 90+%leapyear;
  %month[4] = 120+%leapyear;
  %month[5] = 151+%leapyear;
  %month[6] = 181+%leapyear;
  %month[7] = 212+%leapyear;
  %month[8] = 243+%leapyear;
  %month[9] = 273+%leapyear;
  %month[10] = 304+%leapyear;
  %month[11] = 334+%leapyear;
  %month[12] = 365+%leapyear;
  for (%i = 0; %i < 12; %i++)
  {
    if (%days < %month[%i+1] && %days > %month[%i])
    {
       %days -= %month[%i];
       %month = %i+1;
    }
  }
  return("Logged on at " @ %month @"/"@%days@"/"@%years SPC %hours@":"@%minutes@":"@%seconds@" GMT");
}



