//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

package Mapping {

function onServerCreated()
{
   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.
   Parent::onServerCreated();

   // Override the fps game type
   $Server::GameType = "Mapping";
   $Server::MissionType = "NormalMapping";
   // beffy: $Game::Duration = 0 probably means no time limit...
   $GUI::DurationTime = 15000;
   $Game::Duration = 500000;

   // Load up our own scripts.
}

function startGame()
{
   // Called once the mission has finished loading
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }

   // Inform the client we're starting up
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      %cl.score = 0;
      %cl.kills = 0;
      %cl.deaths = 0;
      %cl.suicides = 0;
      %cl.objectivesTaken = 0;
      %cl.time = $Sim::Time;
      %cl.endtime = 0;
      //zero the score board
      %cl.incScore(%cl.score);
      %cl.incKills(%cl.kills);
      %cl.incDeaths(%cl.deaths);
      %cl.incObjectives(%cl.objectivesTaken);
      commandToClient(%cl, 'StopAllSounds',"");
   }
   messageAll('MsgClientTeamInfo',"",$Team[1].numPlayers, $Team[2].numPlayers,$Team[1].score, $Team[2].score,$Team[1].objectives,$Team[2],objectives);
   $Game::Running = true;
}
//-----------------------------------------------------------------------------

function GameConnection::createPlayer(%this, %spawnPoint)
{
}


function GameConnection::onConnect( %client, %name )
{
   // Send down the connection error info, the client is
   // responsible for displaying this message if a connection
   // error occures.
   messageClient(%client,'MsgConnectionError',"",$Pref::Server::ConnectionError);

   // Send mission information to the client
   sendLoadInfoToClient( %client );

   // if hosting this server, set this client to superAdmin
   if (%client.getAddress() $= "local") {
      echo("User is Admin");
      %client.isAdmin = true;
      %client.isSuperAdmin = true;
   }

   // Get the client's unique id:
   // %authInfo = %client.getAuthInfo();
   // %client.guid = getField( %authInfo, 3 );
   %client.guid = 0;
   addToServerGuidList( %client.guid );
   // Set admin status
   %client.isAdmin = false;
   %client.isSuperAdmin = false;
   %client.skin = addTaggedString( "base" );
   %client.setPlayerName(%name);
   %client.name = %name;
   %client.selectedMAV = "";
   %client.selectedTeam = "";
   %client.ready = true;
   
   $instantGroup = ServerGroup;
   $instantGroup = MissionCleanup;
   echo("CADD: " @ %client @ " " @ %client.getAddress());
   
   //ai stuff
   if ($localClientDefined != 1)
   {
      $localClient = %client;
      $localClientDefined = 1;
   }
   $latestClient = %client;

   $Server::PlayerCount++;
   commandToClient(%client, 'SyncClock', $Sim::Time - $Game::StartTime);
   %client.loadMission();
   commandToClient(%client, 'GameStart', "",$Server::ChanName, $Server::ChanKey);
   serverCmdRelayHostData(%client);
}


function GameConnection::onLaunch(%client)
{
   sendLoadInfoToClient( %client );
   %client.loadMission();
}

function clientCmdPlayWelcome(%handle)
{
   //alxPlay(%handle);
}

function GameConnection::onClientEnterGame(%this)
{
   commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);
   commandToClient(%this, 'StopMissionMusic',$GUI::MusicHandle);
   commandToClient(%this, 'PlayMissionMusic',"Mission_Music");

   // Setup game parameters, the onConnect method currently starts
   // everyone with a 0 score.
   %this.score = 0;
   %this.team = 0;
   %this.kills = 0;
   %this.deaths = 0;
   %this.objectivesTaken = 0;
   
   // Create a new camera object.
   %this.camera = new Camera() {
      dataBlock = Observer;
   };
   MissionCleanup.add( %this.camera );
   %this.spawnPoint = pickSpawnPoint(%this);
   %this.camera.scopeToClient(%this);
   %this.camera.setTransform(%this.spawnPoint);

   // do we want a mecha spawned?
   if($Pref::Server::spawnMech)
   {
	   echo("Spawning Mech!");
      commandToClient(%this, 'StopMissionMusic',"");
      sendLoadInfoToClient( %client );
      %client.loadMission();
      %client.missionLoaded = true;
      %this.selectedTeam = 1;
      %this.selectedMAV = "Locust";//"bluescout";
	   spawnPlayer(%this);
   }
   %this.camera.setVelocity("0 0 0");
   %this.setControlObject(%this.camera);
}

function GameConnection::onClientLeaveGame(%this)
{
  if(%this.team){
    %this.team.numPlayers--;
  }
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
      
}

};
